/**************************************************************************

 **   FILE NAME       : ifx_cm.h
 **   PROJECT         : INCA IP Phone - Config Module
 **   MODULES         : Config Module
 **   SRC VERSION     : V0.1
 **   DATE            : 15-06-2004
 **   AUTHOR          : Radvajesh.M
 **   DESCRIPTION     :
 **   FUNCTIONS       :
 **   COMPILER        : gcc
 **   REFERENCE       : Coding guide lines for VSS ,
 **
 **   COPYRIGHT       : Copyright ?2004 Infineon Technologies AG
 **                     St. Martin Strasse 53; 81669 Mnchen, Germany
 **                     Any use of this Software is subject to the conclusion
 **                     of a respective License Agreement.Without such a
 **						   License Agreement no rights to the Software are granted.
 **  Version Control Section  **
 **   $Author$
 **   $Date$
 **   $Revisions$
 **   $Log$       Revision history
***********************************************************************/
#ifndef __IFX_CM_H__
#define __IFX_CM_H__

#define IFX_CM_SUCCESS 0
#define IFX_CM_FAIL    -1
#define IFX_CM_ERROR   -1
#define IFX_CM_ON    1
#define IFX_CM_OFF   0
#define IFX_CM_MAX_FAX_PROTO 2

#ifdef IIP
#define IFX_CM_CFG_FILE "./tmp/phonecfg"
#define IFX_CM_DHCP_CFG_FILE "./tmp/dhcpcfg"
#endif /* IIP */
#ifdef ATA
#define IFX_CM_CFG_FILE "/nv/atacfg"
#define IFX_CM_DHCP_CFG_FILE "/tmp/dhcpcfg"
#endif


/* Permission to be used while opening the FIFO's */
#define IFX_CM_FIFO_PERM             0666

/* Info for SIP */
#define IFX_CM_SIP_MAX_REG_ADDR      128
#define IFX_CM_SIP_USER_NAME	       80  
#define IFX_CM_SIP_MAX_USR_NAME      128
#define IFX_CM_SIP_MAX_DOMAIN_NAME   255
#define IFX_CM_SIP_MAX_SUBS	       20

/* Updated : Shweta */
#define IFX_CM_MAX_CHAR   				 500

#ifdef ATA
#define IFX_CM_SIP_MAX_ENDPTS	     2
#endif
#ifdef IIP
#define IFX_CM_SIP_MAX_ENDPTS	     1
#endif

#define IFX_CM_SIP_MAX_TELCONFIGINFO  24
#define IFX_CM_TIME_STR_SIZE          16

/* Begin - SIP Config Info */

/* SIP Address Type */

typedef  enum 
{
    IFX_CM_SIP_IP_ADDR=1,
    IFX_CM_SIP_TEL_NUM,
    IFX_CM_SIP_URL,
    IFX_CM_SIP_EXTN
} e_IFX_CM_SipAddrType;


/* Enumerations for Transport Type */
typedef enum
{
     IFX_CM_SIP_TRPROTO_NONE,
     IFX_CM_SIP_TRPROTO_UDP,
     IFX_CM_SIP_TRPROTO_TCP
} e_IFX_CM_SipTransportType;


/* Address Info */ 
typedef struct 
{ 
    e_IFX_CM_SipAddrType  eAddrType;/* Type of address being given */ 

    /* Display Name of the destination */ 
    char8 szDisplayName[IFX_CM_SIP_MAX_USR_NAME];

    /* User Name of the destination/Phone Number */ 

    char8 szUserName[IFX_CM_SIP_USER_NAME];

    char8 cCalledAddr[IFX_CM_SIP_MAX_REG_ADDR];/* IP address/FQDN */ 

    /* Port Number on which th destination can be reached */ 

    uint32 uiPort; 
	 
	  /* Adddress Book Protocol can hold any value of e_IFX_CM_SipTransportType*/
    uchar8  ucAddrProto;

} x_IFX_CM_SipCalledAddr; 


/* Definitions related to Jitter Buffer */ 
#define IFX_CM_SIP_JB_FIXED       1 
#define IFX_CM_SIP_JB_ADAPTIVE    2 
 

/* Enumerations for DTMF */ 
typedef enum
{ 
   IFX_CM_SIP_DTMF_NONE, 
   IFX_CM_SIP_DTMF_INBAND_EVENT,
   IFX_CM_SIP_DTMF_INBAND_VOICE,
   IFX_CM_SIP_DTMF_BOTH
} e_IFX_CM_SipDtmfType; 
 

/* Proxy Info */ 
typedef struct 
{ 
   /* Enable or Disable Proxy settings*/
   char8 ucFlag;
   
   /* IP address of the Proxy */ 
   char8 cProxyAddr[IFX_CM_SIP_MAX_REG_ADDR]; 

   /* Port on which the proxy can be contacted */ 
   uint16 unProxyPort; 

   /* The protocol to be used for contacting the proxy */ 
   uint32 uiProtocol; 

} x_IFX_CM_SipProxyCfg; 

 

typedef enum 
{ 
   IFX_CM_SIP_AUTH_METHOD_NONE = 0, 
   IFX_CM_SIP_AUTH_METHOD_BASIC, 
   IFX_CM_SIP_AUTH_METHOD_DIGEST 

} e_IFX_CM_SipAuthMethod; 

 

/* Registrar Info */ 
typedef struct 
{ 

   /* Enable or Disable Registrar settings*/
   char8 ucFlag;

   /* IP address of the Regisrar */ 
   char8    cRegistrar[IFX_CM_SIP_MAX_REG_ADDR];

   /* The port to be used for Registration */ 
   uint16   unRegPort; 

   /* The protocol to be used for reg */ 
   uint32  uiRegProtocol;

   /* Time of registration with the registrar */ 
   int32   iRegExpirationTime;

} x_IFX_CM_SipRegistrarCfg; 

 

/* User related information */ 
typedef struct 
{ 

   /* User name to be used for this endpoint */ 
   char8 acUserName[IFX_CM_SIP_MAX_USR_NAME];
 
   /* Display Name to be used */ 
   char8 acDisplayName[IFX_CM_SIP_MAX_USR_NAME];

   /* local domain */ 
   char8 acDomain[IFX_CM_SIP_MAX_DOMAIN_NAME];
   
  /* User name to be used for Authentication */ 
   char8 acAuthUserName[IFX_CM_SIP_MAX_USR_NAME];

   /* password to be used for this endpoint */ 
   char8 acPasswd[IFX_CM_SIP_MAX_USR_NAME]; 

} x_IFX_CM_SipUserCfg; 

 

/* RTP port range */ 
typedef struct 
{ 
   /* The Lower limit of RTP port range */ 
   uint16 unStartRTPPort; 

   /* The upper limit of RTP port range */ 
   uint16 unEndRTPPort;

} x_IFX_CM_SipRTPPortRangeCfg; 

 

/* Gateway Info */ 
typedef struct 
{ 
   /* IP address of the Gateway */ 
   char8   cGatewayAddr[IFX_CM_SIP_MAX_REG_ADDR]; 

   /* Port on which the Gateway can be contacted */ 
   uint16   unGatewayPort; 

} x_IFX_CM_SipGatewayCfg; 

 

/* Other SIP Information */ 

typedef struct 
{ 
   /* Transport type to be used for the endpoint */ 
   char8 ucTransportType;

   /* Usage of compact headers */ 
   char8 bUseCompactHdrs;

#ifdef NO_REQ
   /* Hide the user details in the From URI */ 
   char8 bHideUserInfo;
#endif /*not REQ*/  
   /* Silence supression */ 
   char8 bSilenceSupp; 

   /* Comfort noise support */  
   char8 bComfortNoise;

   /* Echo Suppression */ 
   char8 bEchoSupp;

   /* Type of DTMF supported */ 
   e_IFX_CM_SipDtmfType eDtmfType;

   /* DTMF Payload Value */ 
   char8 ucDtmfPayloadValue;

} x_IFX_CM_SipOtherCfg; 

 

/* Jitter Buffer Info */ 

typedef struct 
{   

   /* To indicate if the jitter buffer is fixed or adaptive */ 
   char8   ucType;

  /* This shall be any value between 0 and 16 */ 
  /* Only to be defined for Adaptive JB - To be checked */ 
   char8   ucScalingFactor;

   /* Initial size of JB  */ 
   uint16  unInitialSize;

   /* Max Size of JB */ 
   uint16  unMaxSize;

   /* Min Size of JB */ 
   uint16  unMinSize;

} x_IFX_CM_SipJbConfig; 

 

typedef struct 
{ 

   uchar8 ucFlag;  /* On / Off Value */

} x_IFX_CM_SipCallOnHoldCfg; 

 

typedef enum 
{ 
   IFX_CM_SIP_HOTLINE_NUM_MODE =0, 
   IFX_CM_SIP_HOTLINE_URL_MODE 
} e_IFX_CM_SipHotlineAddType; 
 

typedef struct 
{ 
   e_IFX_CM_SipHotlineAddType eAddType; 
   char8 cDestAddr[IFX_CM_SIP_MAX_DOMAIN_NAME]; 

} x_IFX_CM_SipHotlineCfg; 



/* Country and Area code Info */
typedef struct
{

   /* Country Code */
   char acCountryCode[IFX_CM_SIP_MAX_TELCONFIGINFO];

   /* Acess Prefix for International Calls */
   char acIntAccPref[IFX_CM_SIP_MAX_TELCONFIGINFO];

   /* Local Area Code */
   char acLocAreaCode[IFX_CM_SIP_MAX_TELCONFIGINFO];

   /* Acess Prefix for Local Calls */
   char acLongDistCallPref[IFX_CM_SIP_MAX_TELCONFIGINFO];

} x_IFX_CM_SipTelUriCfg;



/* Call Forward Options */
typedef struct
{
	
   /* Call Forward Type */
   uchar8 ucType;

  #define IFX_CM_SIP_FWD_UNCOND    0x0001

  #define IFX_CM_SIP_FWD_ONBUSY    0x0002

  #define IFX_CM_SIP_FWD_NOANSW    0x0004

  /* Unconditional or No Answer Address */
  x_IFX_CM_SipCalledAddr xFwdAddrUncon;
  x_IFX_CM_SipCalledAddr xFwdAddrNoAns;
  x_IFX_CM_SipCalledAddr xFwdAddrBusy;

} x_IFX_CM_SipFwdCfg;



typedef struct
{

   /* use Username instead of channelid */
   char8 acUserName[IFX_CM_SIP_MAX_USR_NAME];
   char8 acPasswd[IFX_CM_SIP_MAX_USR_NAME];

   /* TCP or UDP transport */
   e_IFX_CM_SipTransportType eTransportType;

   /* Physical Port */
   uint16 unPort;

} x_IFX_CM_SIP_EndptInfo;


#if defined(ATA) || defined(IIP)
typedef struct
{

   uchar8 ucNoOfEndpts;
   x_IFX_CM_SIP_EndptInfo xEndptInfo[IFX_CM_SIP_MAX_ENDPTS];

} x_IFX_CM_SIP_EndptCfg; 
#endif //#if defined(ATA) || defined(IIP)


typedef struct
{
   /* Caller ID flag ON/OFF*/
   uchar8 ucCallIdFlag;

} x_IFX_CM_SIP_ObCallIdCfg;



typedef struct
{
   x_IFX_CM_SipCalledAddr  xOldAddr;
   x_IFX_CM_SipCalledAddr  xNewAddr;

}x_IFX_CM_SipModifyAddr;

/* End of SIP Config Info */

/* Begin - App Config Info */

#define IFX_CM_APP_USER_NAME             80
#define IFX_CM_APP_USER_PASSWD           16
#define IFX_CM_APP_DISPLAY_NAME          128
#define IFX_CM_APP_IP_ADDRESS_SIZE       128
#define IFX_CM_APP_MAX_TELNO_SIZE        16
#define IFX_CM_APP_MAX_PB_ENTRIES        5
#define IFX_CM_APP_MAX_CR_ENTRIES        5 
#define IFX_CM_SIP_MAX_FAST_DIAL 	 5



/* Address Book Data Structures */

typedef e_IFX_CM_SipAddrType e_IFX_CM_AppAddressType;

/*	Structure to store the SIP URL and IP Address */
typedef struct
{
  uchar8  aucDispName[IFX_CM_APP_DISPLAY_NAME];
  uchar8  aucUsrName[IFX_CM_APP_USER_NAME];
  uint8   aucIPAddr[IFX_CM_APP_IP_ADDRESS_SIZE];
  uint32  unDestPort;

} x_IFX_CM_AppSipUri, x_IFX_CM_AppIpAddr;


/* Structure to Send Fax Port Info */
typedef struct
{
   /* Port Number Range for TCP/UDP Connection */
   uint16 unPortStart;
   uint16 unPortEnd;
} x_IFX_CM_SipFaxPortCfg;


/* Structure to store the SIP Telephone Num */
typedef struct
{
   uchar8   aucDispName[IFX_CM_APP_DISPLAY_NAME];
   uint8    aucTelNumber[IFX_CM_APP_USER_NAME];
}x_IFX_CM_AppTelUri;



/*Structure to store the  Address */
typedef struct
{
  e_IFX_CM_AppAddressType  eAddrType;
  
  #ifdef ATA
  /* DTMF index code to be dialed to select this address entry*/
  uchar8  acDialCode[16];
  #endif

/* Added : Shweta */  

  uchar8	 ucAddrProto;
  #ifdef IIP
  /* The indec to the vector table containg training info */
  char8 cVectorIndex;
  #endif

  union
  {
    x_IFX_CM_AppSipUri   x_SipUri;
    x_IFX_CM_AppTelUri   x_TelUri;
    x_IFX_CM_AppIpAddr  x_IpAddr;
  } ux_IFX_CM_AppAddr;

} x_IFX_CM_AppAddressBookEntry;

#ifdef IIP
typedef struct
{
  #define IFX_CM_MAX_TRAIN_VECT 10
  #define IFX_CM_MAX_TRAIN_VECT_SIZE 188
  uchar8  ucTotVecEntries;	  
  uint16  unFreeIndex;
  uint16   unSpeechVector[IFX_CM_MAX_TRAIN_VECT][IFX_CM_MAX_TRAIN_VECT_SIZE];
}x_IFX_CM_VadCfg;

/* Updated : Shweta */
typedef struct
{
	uint8   ucMsgId;	
   uint8   ucReadFlag;	/*Set When Message has been read*/
   uint8   ucTotalPage;/*Total page of message*/
   x_IFX_CM_SipCalledAddr  xAddress;/* New Added  FROM SIP*/
   uint8   aucMsgData[IFX_CM_MAX_CHAR];/*Message Data*/
		  
}x_IFX_CM_AppMsgCfg;

#endif


/* Structure to Send Address Book, from Web to CM to APP and vice versa */
/* To be used during
 *   Get from CM
 *   Set from Web
 *   Delete from Web
 *   Address book update from APP (incase of IP Phone)
 */
typedef struct
{
   uchar8 ucNoOfEntries;
   x_IFX_CM_AppAddressBookEntry xAddrBookEntry[IFX_CM_APP_MAX_PB_ENTRIES];

} x_IFX_CM_AppAddrBookCfg;



/* Structure to store the  Call Register Info */
typedef struct
{ 
  e_IFX_CM_AppAddressType  eAddrType;
  char8 acTime[IFX_CM_TIME_STR_SIZE];
  char8 acDate[IFX_CM_TIME_STR_SIZE];
  uchar8 ucAddrProto;
  union
  { 
    x_IFX_CM_AppSipUri   x_SipUri;
    x_IFX_CM_AppTelUri   x_TelUri;
    x_IFX_CM_AppIpAddr   x_IpAddr;
  } ux_IFX_CM_AppAddr;

} x_IFX_CM_AppCallRegEntry;



typedef struct
{
   uchar8 ucNoOfEntries;
   x_IFX_CM_AppCallRegEntry xCallRegEntry[IFX_CM_APP_MAX_CR_ENTRIES];

} x_IFX_CM_AppCallRegCfg;



/* Structure for modifying Address Book Entry */
/* To be used during
 *   Modify Request from Web
 *   Modify Request from APP (incase of IP Phone)
 *  Add an entry only the xoldAddrEntry shall be used
 *  Update an entry only the xoldAddrEntry shall be used
 */

typedef struct
{
/* for update and delete info type we shall use only the old entry*/
  x_IFX_CM_AppAddressBookEntry   xOldAddrEntry;
  x_IFX_CM_AppAddressBookEntry   xNewAddrEntry;

} x_IFX_CM_AppModAddrBookCfg;





/* Structure for Configuring User Info */
/* To be used during
 *   New User Id creation (in this case ucNewPasswd is not used)
 *   Delete User (in this case, the passwd fields are not used)
 * Password has to be NUMERIC, should be validated at the Web
 */


typedef struct
{

   uint8 ucName[IFX_CM_APP_USER_NAME];
   uint8 ucPasswd[IFX_CM_APP_USER_PASSWD];

} x_IFX_CM_AppUserCfg;

/*To be used during
 * Modify User password and
 * Modify Admin password
*/
typedef struct
{

   uint8 ucName[IFX_CM_APP_USER_NAME];
   uint8 ucOldPasswd[IFX_CM_APP_USER_PASSWD];
   uint8 ucNewPasswd[IFX_CM_APP_USER_PASSWD];

} x_IFX_CM_AppModUserCfg;


typedef enum
{

   IFX_PA_ADDR_TYPE_IPV4 = 0,
   IFX_PA_ADDR_TYPE_FQDN

} e_IFX_CM_AppAddrType;



/* IP Address struct */
typedef  struct
{

  uint8   aucHostName[IFX_CM_APP_IP_ADDRESS_SIZE];
  uint8   ucDhcp; 
  #define IFX_CM_APP_DHCP_DISABLED 0
  #define IFX_CM_APP_DHCP_ENABLED  1
  e_IFX_CM_AppAddrType eAddrType;
  uint8   aucIPAddress[IFX_CM_APP_IP_ADDRESS_SIZE];
  uint8   aucGatewayAddress[IFX_CM_APP_IP_ADDRESS_SIZE];
  uint8   aucSubnetMask[IFX_CM_APP_IP_ADDRESS_SIZE];
} x_IFX_CM_AppNetAddrCfg;


typedef struct
{
   uchar8 ucExtnDialFlag; /* On/Off Value */
   int16    nPrefixDigit;

} x_IFX_CM_AppExtnDial;



typedef struct
{
   uchar8 ucFastDialFlag; /* On/Off Value */
  /* x_IFX_CM_SipCalledAddr xSipAddr[IFX_CM_SIP_MAX_FAST_DIAL];*/
   uchar8 ucFastDialName[IFX_CM_APP_DISPLAY_NAME];
	int16 nIndex; /* Fast dial Index */
} x_IFX_CM_AppFastDial;



typedef struct
{
  
  #ifdef IIP
   x_IFX_CM_AppExtnDial    xExtnDial;
   x_IFX_CM_AppFastDial    xFastDial[10];

	/* Updated: Shweta */
	/* Ring Mute Flag */
	uchar8 ucRingMuteFlag; /* On/Off Value */

  #endif /* only for IIP*/
  
   uchar8 ucDndFlag;  /* On/Off Value */
   
   x_IFX_CM_SipFwdCfg     xSipFwd;
  
} x_IFX_CM_AppCallCfg;



typedef struct
{
  int16 unRingVol;

}x_IFX_CM_RingVolCfg;

typedef enum
{
  IFX_CM_ROOM_NORMAL,
  IFX_CM_ROOM_NOISY,
  IFX_CM_ROOM_ECHOIC
}e_IFX_CM_RoomType;

typedef enum
{
	IFX_CM_VOL_LEVEL1,
	IFX_CM_VOL_LEVEL2,
	IFX_CM_VOL_LEVEL3,
	IFX_CM_VOL_LEVEL4,
	IFX_CM_VOL_LEVEL5,
	IFX_CM_VOL_LEVEL6,
	IFX_CM_VOL_LEVEL7,
	IFX_CM_VOL_LEVEL8
}e_IFX_CM_VolLevelType;

typedef struct
{
  /*Room Type*/
  e_IFX_CM_RoomType  eRoomType; 
  
  /*Voice Level */
  e_IFX_CM_VolLevelType  eVoiceVol;
  
}x_IFX_CM_VoiceVolCfg;


typedef struct
{

  uchar8  ucEthPcLed; /*LED ON/OFF Value*/
  uchar8  ucEthLanLed; /*LED ON/OFF Value*/
}x_IFX_CM_EthLedCfg;

/* Structure used by Configuration Module to provide Config Info  */
/* RM Receive Message */
typedef struct  
{	 
   uint16 unLinkBw;        /* Value in kbps */ 
   uint32 uiCodecList;
   /* Bit Pattern used by CM to send the codec list is described below */ 

   uint16 unFrameSize;
   uint32 uiPreferredCodec;

   /* G.723 5.3 Kbps - 64, G.723 6.3 Kbps - 128 */
   uint32 uiG723PrefCodec;

} x_IFX_CM_RmMediaCfg;

#define IFX_CM_RM_G711_ALAW     0x00000001
#define IFX_CM_RM_G711_ULAW     0x00000002
#define IFX_CM_RM_G729_8        0x00000004
#define IFX_CM_RM_G729_E        0x00000008
#define IFX_CM_RM_G723          0x00000010
#define IFX_CM_RM_G723_5_3      0x00000020
#define IFX_CM_RM_G723_6_3      0x00000040
#define IFX_CM_RM_G728          0x00000080
#define IFX_CM_RM_LINEAR_256    0x00000100
#define IFX_CM_RM_LINEAR_16     0x00000200
#define IFX_CM_RM_LINEAR_8      0x00000400
#define IFX_CM_RM_G726_16       0x00000800
#define IFX_CM_RM_G726_24       0x00001000
#define IFX_CM_RM_G726_32       0x00002000
#define IFX_CM_RM_G726_40       0x00004000
#define IFX_CM_RM_G722_64       0x00008000
#define IFX_CM_RM_T38           0x00010000

/* Structure used by CM to Configure Fax info in the RM */
typedef struct
{
   /* Protocol Type */

#define IFX_CM_RM_T38_TRANS_PROTO_TCP 0
#define IFX_CM_RM_T38_TRANS_PROTO_UDP 1

   uint32         uiTransportProtocol;

   /* T38 Protocol Version */
#define IFX_CM_RM_T38_VERSION                  1
   uchar8         ucVersion;

#define IFX_CM_RM_CAP_FL_TCF_LOCAL             0x01
#define IFX_CM_RM_CAP_FL_TCF_TRANSFERRED       0x02
   uchar8         ucRateManagement;

   /* Kilobits/s */
#define IFX_CM_RM_T38_MAX_BIT_RATE             14400
   uint16         unMaxBitRate;

   uint32         uiBitOptions;
#define IFX_CM_RM_CAP_FL_OPT_FILL_BIT_REMOVAL  0x0001
#define IFX_CM_RM_CAP_FL_OPT_TRANSCODING_MMR   0x0002
#define IFX_CM_RM_CAP_FL_OPT_TRANSCODING_JBIG  0x0004

   /* UDP options */
   uint16         unUDPMaxBufferSize;
   uint16         unUDPMaxDatagramSize;

   /* UDP Error Correction */
   uchar8         ucUDPErrCorrection;
#define IFX_CM_RM_CAP_FL_EC_REDUNDANCY         0x01
#define IFX_CM_RM_CAP_FL_EC_FEC                0x02

} x_IFX_CM_RM_FaxCfg;

/* Structure used by CM to Configure T38 Stack */
typedef struct
{
   uchar8 ucOldAsn1;  /* 0 - Off, 1 - On */

   /* Size of NSX Patch */
   uchar8 ucNsxSize;
 
   /* NSX Info */
#define IFX_CM_FA_NSX_MAX_SIZE  40
   uchar8 ucNsxInfoField[IFX_CM_FA_NSX_MAX_SIZE];

   /* Data Wait time in 10 ms */
   uchar8 ucDataWaitTime;

   /* Udp High Rate Error Recovery Packets */
   uint16 unUdpHRErrRecPkts;

   /* Udp Low Rate Error Recovery Packets */
   uint16 unUdpLRErrRecPkts;

   /* Udp Prior Packets for FEC */
   uint16 unUdpPriorFecPkts;

   /* Framelength */
   uint16 unFrameLength;

   /* Preferred T38 Connection */
#define IFX_CM_FA_T38_CONN_UDP   1
#define IFX_CM_FA_T38_CONN_TCP   2
   uchar8 ucT38Conn;
   
} x_IFX_CM_FA_T38Cfg;


/* Key managment Protocol for SRTP */
#ifdef SRTP

/* Maximum Number of Hosts that INCA IP can Communicate*/
#define IFX_CM_MAX_HOSTS 2

/* Different encyrption transforms that can be used */
typedef enum
{
 IFX_CM_NULL_ENCR =0,
 /* Advanced Encription Standard Tranforms */
 /* Counter Mode */
 IFX_CM_AES_CM,
 IFX_CM_AES_ECB,
 IFX_CM_AES_CBC,
 IFX_CM_AES_CFB,
 IFX_CM_AES_OFB,

 /* DES Transforms */
 IFX_CM_DES_CM,
 IFX_CM_DES_ECB,
 IFX_CM_DES_CBC,
 IFX_CM_DES_CFB,
 IFX_CM_DES_OFB,
 
 /*3DES Transforms */
 IFX_CM_3DES_CM,
 IFX_CM_3DES_ECB,
 IFX_CM_3DES_CBC,
 IFX_CM_3DES_CFB,
 IFX_CM_3DES_OFB,
}e_IFX_CM_EncrTransf;

/* Different Authentication Transforms*/
typedef enum
{
 IFX_CM_NULL_AUTH =0,
 IFX_CM_HMAC_SHA1,
}e_IFX_CM_AuthTransf;


/* The Pseduo random function to be used for Key derivation */
typedef e_IFX_CM_EncrTransf e_IFX_CM_KeyDervFunc;

/* Forward Error Correction Parameters section 10 of RFC*/
typedef enum
{
 IFX_CM_NONE,
 IFX_CM_FIRST_FEC,
 IFX_CM_FIRST_SRTP,
}e_IFX_CM_FecOrder;

/* Key management Parmeters */
#define IFX_CM_MAX_SRTP_LIFE_TIME  281474976710655LL /* 2^48-1 */
#define IFX_CM_MAX_SRTCP_LIFE_TIME  2147483647 /*2^31-1*/

#define IFX_CM_MASTER_KEY_LENGTH 32    /* 256 bits */
#define IFX_CM_MATER_SALT_LENGTH 32   /* 256 bits */

/* Maximun Master Keys that a single SRC can have so that it can be rekeyed */
/* This value is not specified by the RFC*/
#define IFX_CM_MAX_MASTER_KEYS 10

/* For each Master key all the below params should exist*/
typedef struct
{
 /* Master Key related parmeters */

   /* Master Key Length*/
   uint16 unMasterKeyLen;
   
   /* Master Key*/
   uchar8 acMasterKey[IFX_CM_MASTER_KEY_LENGTH];
   
   /* Master Salt Key length*/
   uint32 unMasterSaltLen;
  
   /* Master Salt */
   uchar8 acMasterSalt[IFX_CM_MASTER_KEY_LENGTH];

   /* n_e Session encryption Key Length */
   uint16 unSesEncrKeyLen;

   /* n_a Session Authentication Key Length*/
   uint16 unSesAuthKeyLen;

   /* n_s Session Salt Key Length*/
   uint16 unSesSaltKeyLen;

   /* Key Derivation Rate: {1,2,4,...2^24}  since its is fixed 
	 * for the lifetime of the Master key section 4.3.1*/
   uint32 uiKeyDerivationRate:24; /* only 24 bit are supposed to be used */

   /*<From , To> Pair used for rekeying and identifing Master Key */
   uint64 ulFromIndex:48; /* only 48 bits are to be used */
   uint64:16;
   uint64 ulToIndex:48;   /* Only 48 bits are to be used */
   uint64:16;

   /* SRTP Master key counter to count the number of 
	 * SRTP packets sent out with the same master key */
   uint64 ulSrtpMaxLifeTime;

   /* SRTCP Master key counter to count the number of 
	 * SRTP packets sent out with the same master key */
   uint32 uiSrtcpMaxLifeTime;

   /*Master Key Identifier related parameters*/

   /* MKI ON or OFF */
   uchar8 cMkiIndicator; /* ON =1 and OFF=0 */

   /* Length of the MKI*/
   uint16 unMkiLength; /* Value in Octent*/

   /* MKI Value identifying each Master key */
   uint32 uiMkiValue;

}x_IFX_CM_MasterKeyParam;


typedef struct
{
   /* crypto content index Parameters */
  
   /* SSRC value, it it is assume we have just one RTP stream,
     if there exists more than one RTP stream this should be
	  a vector of [IFX_CM_MAX_STREAMS]
   */
   uint32 uiSsrcValue;
  
   /* Roll over counter, it is assume we have just one RTP stream,
     if there exists more than one RTP stream this 
	  should be a vector of [IFX_CM_MAX_STREAMS]
   */
   /* sender Roc */
   uint32 uiRoc;

   /* Sequence number */
   uint16 unSeqNumber;

   /* SRTCP Index */
   uint64 ulSrtcpIndex;
  
   /* Transport Address */
   #define IFX_CM_MAX_IP_ADDRESS_LEN 16
   uchar8 acIpaddress[IFX_CM_MAX_IP_ADDRESS_LEN];
 
   /* Port Number */
   uint16 unPort;
 
}x_IFX_CM_IndexParams;

/* Cryptographic Context Parameters*/
typedef struct
{
  /* Encryption Algorithm to be used, can hold any one 
	* of the values of e_ifx_rtp_encr_transf*/
   uint8 eEncrAlgo;
 
   /* Authentication Algorithm to be used, can hold any one
	 *  of the values of e_ifx_rtp_auth_transf*/
   uint8 eAuthAlgo;

   /* Authentication Tag Length*/
   uint16 unAuthTagLen;

   /* SRTP PREFIX_LENGTH*/
   uint16 unSrtpPrefixLen;

   /*SRTP Pseudo Random Function, can hold any one 
	 * of the values of e_IFX_RTP_KeyDervFunc */
   uint8 unPseudoRandomFunc;

   /*Masterkey or TGK  Related parameters or the 
	 * information obtained in the MIKEY payload KEMAC */
   x_IFX_CM_MasterKeyParam xMasterCryptoParam[IFX_CM_MAX_MASTER_KEYS];
 
   /* Parameters that are partialy obtained by the Key 
	 * mangement MIKEY payload SRTP ID*/
   /* Parametrs that are obtained are SSRCi, ROCi  were i is the session ID */
   x_IFX_CM_IndexParams  xCryptoIndexParam;

   /* Relation to other RTP Profiles*/
   /*Forward Error Correction  Order can Take any one of the
	 *  values of e_IFX_RTP_FecOrder*/
   /* if FIRST_FEC then FEC processing has to be performed  prior
	 *  SRTP on sender side
    * and SRTP processing before FEC at the reciveing side
    */
   uchar8 ucFecOder;
  
}x_IFX_CM_SrtpKeyCfg;

#endif

#define IFX_CM_MAX_RESPONSES 20

/* Begin InfoTypes */
typedef enum
{
   /* SIP Info type */
   IFX_CM_SIP_CFG_PROXY=1, 
   IFX_CM_SIP_CFG_REGISTRAR, 
   IFX_CM_SIP_CFG_USER, 
   IFX_CM_SIP_CFG_GATEWAY, 
   IFX_CM_SIP_CFG_RTPPORTRANGE, 
   IFX_CM_SIP_CFG_JITTERBUFF, 
   IFX_CM_SIP_CFG_OTHER,
   IFX_CM_SIP_CFG_TELURI,
   IFX_CM_SIP_CFG_HOTLINE,   
   IFX_CM_SIP_CFG_HOLD,/*10*/
   IFX_CM_SIP_CALLER_ID,
   /* APP Info Type */
   IFX_CM_APP_CALL_CFG,
   IFX_CM_APP_ADDR_ENTRY,
   IFX_CM_APP_ADD_ADDR_ENTRY,
   IFX_CM_APP_MODIFY_ADDR_ENTRY, 
   IFX_CM_APP_DELETE_ADDR_ENTRY,
   IFX_CM_APP_ADD_NEW_USR,
   IFX_CM_APP_MODIFY_USER_PASSWD,
   IFX_CM_APP_MODIFY_ADMIN_PASSWD,
   IFX_CM_APP_DELETE_USER,/*20*/
   IFX_CM_APP_NETWORK,
   IFX_CM_APP_ETH_LED_CFG,
   /* RM Info Type*/
   IFX_CM_RM_CONFIG,
   /* Fax agent info type*/
   IFX_CM_RM_FAX_CFG,
   IFX_CM_SIP_CFG_FAX_PORT,
   /* Fax Agent Info Type */
   IFX_CM_FA_T38_CFG,
   /* Key Managment Protocol */
   IFX_CM_SRTP_KEY_CFG,
   IFX_CM_SRTCP_KEY_CFG,
   /* ROOM Settings */ 
   IFX_CM_APP_ROOM_CFG,
   /* Local App to CM info type*/
   IFX_CM_APP_BCAST_INIT,
   IFX_CM_APP_INIT_CONF,/* 31 */
   IFX_CM_APP_SHUTDOWN,
   IFX_CM_APP_RING_VOLUME,
   IFX_CM_APP_VOICE_VOLUME,
   IFX_CM_APP_CR_MISSED_CALL,
   IFX_CM_APP_CR_DIALED_CALL,
   IFX_CM_APP_CR_RECEIVED_CALL,
   IFX_CM_APP_CONFIG_PREF_CODEC,
   IFX_CM_APP_ADMIN_PROF,
   IFX_CM_APP_USR_PROF,
   IFX_CM_APP_CFG_PROXY, 
   IFX_CM_APP_CFG_REGISTRAR, 
   IFX_CM_APP_CFG_GATEWAY, 
   IFX_CM_APP_CFG_RTPPORTRANGE, 
   IFX_CM_APP_CFG_TELURI,
   IFX_CM_APP_CFG_HOLD,
   IFX_CM_APP_CALLER_ID,
   IFX_CM_APP_LOCAL_CALL_CFG,
   IFX_CM_APP_IPADDR,
   IFX_CM_APP_VAD_CFG,
	/* Updated : Shweta */
	IFX_CM_APP_ADD_MSG_ENTRY,/* 50 */
	IFX_CM_APP_MODIFY_MSG_ENTRY,
	IFX_CM_APP_DELETE_MSG_ENTRY,
   /* Local from SIP Messages*/
   IFX_CM_SIP_EDIT_ADDR_BOOK,
	/* Local from srtp  to KMP or CM */
	IFX_CM_SRTP_RE_KEY, /* for retriger change of master key*/
} e_IFX_CM_InfoType;



/* Begin - ErrorCodes */
typedef enum
{
   IFX_CM_ERR_NONE = 0,
   IFX_CM_ERR_GENERAL,
   IFX_CM_ERR_REQ_TYPE,
   IFX_CM_ERR_INFO_TYPE,
   IFX_CM_ERR_CHANNEL_ID,
   IFX_CM_ERR_CALL_IN_PROGRESS,

   /*********** Error Msgs from SIP ***********/

   /* SIP Proxy Errors */
   IFX_CM_ERR_PROXY_IP,
   IFX_CM_ERR_PROXY_PORT,
   IFX_CM_ERR_PROXY_PROTOCOL,
   /* SIP Registrar Errors */
   IFX_CM_ERR_REG_IP,
   IFX_CM_ERR_REG_PORT,
   IFX_CM_ERR_REG_PROTOCOL,
   IFX_CM_ERR_REG_EXP_TIME,
   IFX_CM_ERR_REG_AUTH,

   /* SIP Subscriber Errors */
   IFX_CM_ERR_USR_NAME,
   IFX_CM_ERR_USR_DNAME,
   IFX_CM_ERR_USR_DOMAIN,
   IFX_CM_ERR_USR_AUTH_NAME,
   IFX_CM_ERR_USR_PSWD,
   /* SIP Gateway Errors */
   IFX_CM_ERR_GWY_IP,
   IFX_CM_ERR_GWY_PORT,
   IFX_CM_ERR_GWY_LIS_PORT,
   /* SIP Other Config Errors */
   IFX_CM_ERR_OTH_TRA_TYPE,
   IFX_CM_ERR_OTH_COMP_HDR,
   IFX_CM_ERR_OTH_HIDE_INFO,
   IFX_CM_ERR_OTH_SILENCE_SUPP,
   IFX_CM_ERR_OTH_COMF_NOISE,
   IFX_CM_ERR_OTH_ECHO_SUPP,
   IFX_CM_ERR_OTH_DTMF_TYPE,
   IFX_CM_ERR_OTH_DTMF_PAY_TYPE,

   /* SIP Tel Uri Errors */
   IFX_CM_ERR_TEL_URI_CNTRY_CODE,
   IFX_CM_ERR_TEL_URI_AREA_CODE,
   IFX_CM_ERR_TEL_URI_AUTH,

   /* SIP Call Fwd Errors */
   IFX_CM_ERR_CALL_FWD_TYPE,
   IFX_CM_ERR_CALL_FWD_ADDR,

   /*********** Media Config Errors ***********/

   /* Jitter Buffer Errors */
   IFX_CM_ERR_JB_TYPE,
   IFX_CM_ERR_JB_SCAL_FACT,
   IFX_CM_ERR_JB_INIT_SIZE,
   IFX_CM_ERR_JB_MAX_SIZE,
   IFX_CM_ERR_JB_MIN_SIZE,
   
   /* RTP Errors */
   IFX_CM_ERR_RTP_PORT,

   /* Fax Errors */
   IFX_CM_ERR_FAX_TRANS_PROTO,
   IFX_CM_ERR_FAX_VERSION,
   IFX_CM_ERR_FAX_RATE_MGMT,
   IFX_CM_ERR_FAX_BIT_RATE,
   IFX_CM_ERR_FAX_FILL_OPT,
   IFX_CM_ERR_FAX_UDP_BUFF_SIZE,
   IFX_CM_ERR_FAX_ERR_CORRECTION,

   /********** Application Errors ***********/

   /* Call On Hold Error */
 
   /* Hotline Error */

   /* Call Config Errors */
   IFX_CM_ERR_EXTN_DIAL_PREFIX,
   IFX_CM_ERR_FAST_DIAL_ADDR,
   IFX_CM_ERR_FAST_DIAL_INDEX,

   /* Network Config Errors */
   IFX_CM_ERR_NW_ADDR_TYPE,
   IFX_CM_ERR_NW_IP_ADDR,
   IFX_CM_ERR_NW_MASK,
   IFX_CM_ERR_NW_DHCP,
   IFX_CM_ERR_NW_HOST_NAME,
   IFX_CM_ERR_NW_GATEWAY,

   /* User Config Errors */
   IFX_CM_ERR_USR_INV_NAME,
   IFX_CM_ERR_USR_INV_PASSWD,
   IFX_CM_ERR_ADD_ADDR_ENTRY,
   IFX_CM_ERR_MODIFY_ADDR_ENTRY,
   IFX_CM_ERR_DELETE_ADDR_ENTRY,
   IFX_CM_ERR_CONFIG_CODEC,
   IFX_CM_ERR_ADD_IPADDR,
   IFX_CM_ERR_ADD_USR_PROFILE,
   IFX_CM_ERR_DEL_USR_PROFILE,

   /************ RM Errors *****************/
   IFX_CM_ERR_BANDWIDTH,
   IFX_CM_ERR_FRAMESIZE,
   IFX_CM_ERR_PREF_CODEC,
   IFX_CM_ERR_PREF_G723_CODEC,
   IFX_CM_ERR_CODEC_LIST,
   IFX_CM_ERR_CODEC_NOT_SUPPORTED,

   /* Local Error Types - Not to be used for Web */
   IFX_CM_ERR_CM_INIT_FAIL,
   IFX_CM_ERR_CM_BCAST_INIT,
   IFX_CM_ERR_CM_SHUTDOWN,
   IFX_CM_ERR_RING_VOLUME,
   IFX_CM_ERR_VOICE_VOLUME,
   IFX_CM_ERR_CR_MISSED_CALL,
   IFX_CM_ERR_CR_DIALED_CALL,
   IFX_CM_ERR_CR_RECEIVED_CALL,
	IFX_CM_ERR_SRTP_DUP_SSRC,
	IFX_CM_ERR_SRTP_NO_KEY_CFG,

} e_IFX_CM_ErrorCodes;


typedef struct
{
   uchar8 ucNoOfErrors;
   /* Use enums in e_IFX_CM_ErrorCodes*/
   e_IFX_CM_ErrorCodes eErr[IFX_CM_MAX_RESPONSES];
  	      
} x_IFX_CM_ErrorInfo;

/* Begin - CmMsg */

typedef union
{
   /* Following structures shall be used with
    * Set Req
    * Get Rsp
    * In case of a Get Req, the Req type and Info type are enuf
    * In case of a Set Rsp, the values are already with web server
    * In case of Get or Set Error Responses, use the xErrorMsg
    */
   /* SIP Info */
   x_IFX_CM_SipProxyCfg          xProxyCfg; 
   x_IFX_CM_SipRegistrarCfg      xRegCfg; 
   x_IFX_CM_SipUserCfg           xSipUserCfg; /* channel based */
   x_IFX_CM_SipRTPPortRangeCfg   xRtpCfg; 
   x_IFX_CM_SipGatewayCfg        xGatewayCfg; 
   x_IFX_CM_SipJbConfig          xJbConfig; 
   x_IFX_CM_SipOtherCfg          xOtherCfg; 
   x_IFX_CM_SipTelUriCfg         xTelUriCfg;
   x_IFX_CM_SIP_ObCallIdCfg      xCallIdCfg;/* channel based */
   x_IFX_CM_SipModifyAddr	 xSipModAddrCfg;      
#ifdef ATA
   x_IFX_CM_SipFaxPortCfg        xFaxPortCfg;
#endif

   /* APP Info */
   x_IFX_CM_AppAddrBookCfg       xAddrBookCfg;/* channel based */
   x_IFX_CM_AppModAddrBookCfg    xModAddrBookCfg;/* channel based */
   x_IFX_CM_AppUserCfg           xAppUserCfg;/* channel based */
   x_IFX_CM_AppModUserCfg        xAppModUserCfg;/* channel based */
   x_IFX_CM_AppNetAddrCfg        xNetAddrCfg;
   x_IFX_CM_AppCallCfg           xCallCfg;/* channel based */
   x_IFX_CM_EthLedCfg            xEthLedCfg; /*Ethernet Led ON,OFF */
   /* RM Info */
   x_IFX_CM_RmMediaCfg           xMediaCfg;
#ifdef ATA
	x_IFX_CM_RM_FaxCfg            xFaxCfg[IFX_CM_MAX_FAX_PROTO];
   /* T38 Config */
   x_IFX_CM_FA_T38Cfg            xT38Cfg;
#endif

   /* Local Data Structures, used for internal communications */
   /* The following data structures are defined for internal
    * communication between ATA/APP modules.
    * These data structures SHOULD NOT be used by the Web Server 
    */
#ifdef IIP   
    
    x_IFX_CM_VoiceVolCfg         xVoiceVolCfg;/* channel based */
    x_IFX_CM_RingVolCfg          xRingVolCfg;/* channel based */
    x_IFX_CM_AppCallRegCfg       xCallRegCfg;/* channel based */
    x_IFX_CM_VadCfg              xVadCfg; /* For voice activated dialing */
    
	/* Updated : Shweta */
	 x_IFX_CM_AppMsgCfg				xMsgCfg;	/* channel based : Messaging */

#endif /* end of IIP */    

#ifdef SRTP
    x_IFX_CM_SrtpKeyCfg         xSrtpKeyCfg;
#endif

   /* Error Info */
    x_IFX_CM_ErrorInfo           xErrorInfo;

} ux_IFX_CM_DataType;

typedef enum
{
   IFX_CM_GET_REQ=0,
   IFX_CM_SET_REQ,
   IFX_CM_GET_RSP,
   IFX_CM_GET_ERR_RSP,
   IFX_CM_SET_RSP,
   IFX_CM_SET_ERR_RSP

}e_IFX_CM_MsgType;

typedef struct
{
   uchar8 ucMsgType; /*can hold any one of the value e_IFX_CM_MsgType*/
   uchar8 ucInfoType; /*can hold any one of the value  e_IFX_CM_InfoType*/

#define IFX_CM_CHANNEL_BCAST 0xFF

   uchar8 ucChannel;
   ux_IFX_CM_DataType uxData; 
} x_IFX_CM_Msg;

/* PUBLIC Functions */

/* Following function should be used only by the Application */

PUBLIC char8 IFX_CM_Init(void);

PUBLIC char8 IFX_CM_Shut(void);

/* Following function should be used only by the Web Server */

PUBLIC char8 IFX_CM_WebInit(void);

PUBLIC char8 IFX_CM_WebShut(void);

PUBLIC char8 IFX_CM_RecvDataFromCM(IN_OUT x_IFX_CM_Msg * pxCmMsg);

PUBLIC char8 IFX_CM_SendDataToCM(IN_OUT x_IFX_CM_Msg * pxCmMsg);


#endif /* __IFX_CM_H__ */
