#define _GNU_SOURCE
#include <stdio.h>
#include <syslog.h>
#include <unistd.h>
#include <stdlib.h>
#include <string.h>
#include <sys/wait.h>
#include "ifx_common.h"

#define IFX_MAX_PID_FILENAME	100
#define IFX_PID_FILE_PATH		"/var/run"

#undef DHCLIENT
#define UDHCPC

#ifdef DHCLIENT
	#define DHCP_COMMAND "dhclient"
#endif

#ifdef UDHCPC
	#define DHCP_COMMAND "udhcpc -i"
#endif


#if 0
#define IFX_DBG(fmt, args...)	printf("(%s)" fmt, __func__, ##args)
#else
#define IFX_DBG(fmt, args...)

#endif

#if 1
#define IFX_ERR(fmt, args...)	printf("(%s)-Error:" fmt, __func__, ##args)
#else
#define IFX_ERR(fmt, args...)

#endif

static char g_ifx_pidfile_prefix[IFX_MAX_PID_FILENAME];
static char ifx_command_buff[200] = {'\0'};


int ifx_create_pid_file(char *file_prefix)
{
	int process_id = 0;
	FILE *fp;
	char file_name[IFX_MAX_PID_FILENAME];

	process_id = getpid();

	sprintf(file_name,"%s/%s.pid",IFX_PID_FILE_PATH, file_prefix);	
	fp = fopen(file_name,"w");
	if (fp == NULL) {
		syslog(LOG_INFO, "Error : Couldn't open file [%s] in write mode!",file_name);
		return -1;
	} else {
		fprintf(fp,"%d",process_id);
		fclose(fp);
	}
	strcpy(g_ifx_pidfile_prefix, file_prefix);
	atexit(ifx_rm_pid_file_atexit);
	return 0;
}


int ifx_get_process_pid(char *file_prefix)
{
	int process_id=-1;
	char file_name[IFX_MAX_PID_FILENAME];
	FILE *fp = NULL;

	sprintf(file_name,"%s/%s.pid",IFX_PID_FILE_PATH, file_prefix);	
	fp = fopen(file_name,"r");
	if (fp == NULL) {
		syslog(LOG_INFO, "Error : Couldn't open file [%s] in read mode!",file_name);
		return (process_id);
	} else {
		if (fscanf(fp,"%d",&process_id) < 0) {
			process_id=-1; /* Error */
		}
		fclose(fp);
	}
	return process_id;
}


int ifx_rm_pid_file(char *file_prefix)
{
	char file_name[IFX_MAX_PID_FILENAME];


	sprintf(file_name,"%s/%s.pid",IFX_PID_FILE_PATH, file_prefix);	

	return (unlink(file_name));
}

void ifx_rm_pid_file_atexit(void)
{
	ifx_rm_pid_file(g_ifx_pidfile_prefix);
}

/* Check if this pid is indeed the process that we wanted. We do this
 * by opening the process' command line in /proc. It is not an exact
 * match, in that one may want to call this fn multiple times with different
 * parameters to match. For eg., first with <progname> as cmd_arg, then 
 * <arg1> as cmd_arg, and so on ...
 */
int ifx_validate_pid(pid_t pid, char *cmd_arg)
{
	FILE *fp = NULL;
	char file_name[IFX_MAX_PID_FILENAME];
	char *cmd_buff = NULL;
	int len=0, ret=-1;
	
	sprintf(file_name,"/proc/%d/cmdline",pid);
	IFX_DBG("Trying to open file %s ...\n",file_name);
	
	fp = fopen(file_name,"r");
	if (fp == NULL) {
		IFX_ERR("Could Not open File %s",
				file_name);
		goto lbl_end0;
	}
	
	ret = getline(&cmd_buff,&len,fp);
	if (ret < 0) {
		IFX_ERR("getline() function fail ret[%d], len[%d]\n",
			ret, len);
		ret =  -1;
		goto lbl_end1;
	}
  
	IFX_DBG("Command in file is %s",cmd_buff);

	if(strstr(cmd_buff, cmd_arg) == NULL ) {
		IFX_ERR("[%s] Not found in cmd line for pid[%d] \n"
				, cmd_arg, pid);
		ret =  -1;
		goto lbl_end2;
	} else {
		IFX_DBG("[%s] found in cmd line for pid[%d]!\n",
				cmd_arg, pid);
		ret = 0;
	}
 
lbl_end2:	
	if (cmd_buff)
		free(cmd_buff);
lbl_end1:
	fclose(fp);

lbl_end0:
	return (ret);
}

/////////////////////////////////////////////////////////////////////////////////
//ifx_GetCfgDatafromString(...)
//      DESCRIPTION:    find the data in the string pString
//      pString :
//      pSymbol : Search Name
//      Return Value :  a pointer to the beginning of the Data which found,
//                      or NULL if the pSymbol is not found
/////////////////////////////////////////////////////////////////////////////////
char *ifx_GetCfgDatafromString(char *pString, char *pSymbol)
{
        char  seps[]   = "\t\n\"";
	char* pDest;
        if ((pString = strstr(pString, pSymbol)) == NULL)
        {
                return NULL;
        }

	// Support not content variable
	if ((pDest = strstr(pString, "\"")) != NULL)	
	{
	    int pos = 0;
	    // Found ", check value whether exist? (xxx="" means no value)
	    pos = pDest - pString;
	    if (pString[pos] == pString[pos+1])
	    {
		IFX_DBG("Oooooooop!!! No data exist.\n");
		return NULL;
	    }
	}

        if (strtok(pString, "=") == NULL)
        {
                return NULL;
        }

        pString = strtok(NULL, seps);
        return pString;
}

/////////////////////////////////////////////////////////////////////////////////
//ifx_GetCfgData(...)
//      Variable Name   ==>     File Name               ;       Pipe Command
//-------------------------------------------------------------------------------
//      pFileName               ==>     Open File Name  ;       Open pipe command
//      pTag                    ==>     TAG_xxx                 ;       NULL
//      pData                   ==>     Search Name             ;       which Line obtained
//      pRetValue               ==>     Return string.  ;       Return string.
//
//      Return Value :  0 is FALSE. 1 is TRUE
//
//      ex1. File Name ==>      if ( ifx_GetCfgData(FILE_RC_CONF, TAG_DHCP_BINPOND, "Server", sValue) == 1)
//      ex2. Pipe          ==>  if ( ifx_GetCfgData(&command, NULL, "1", sValue) == 1 )
///////////////////////////////////////////////////////////////////////////////////

#define RC_SIZE_K 32

int ifx_GetCfgData(char *pFileName, char *pTag, char *pData, char *pRetValue)
{
        FILE    *fd;
        char  *pString = NULL;
#if 0
        char  *buffer = NULL;
#else
		char  buffer[1024*RC_SIZE_K];
#endif
        char  arrBeginTag[64];
        char  sValue[1024];
        int             nReturn = 0;
        int             nLine = 0;
        int             nIndex = 1;

        if ( pTag && *pTag )
        {
                if ((fd = fopen(pFileName, "r")) != NULL)
                {
#if 0
                        buffer = (char *)malloc(1024*RC_SIZE_K*sizeof(char));
#endif
//						printf("\nIn ifx_GetCfgData : buffer allocated.\n");
                        if (buffer != NULL)
                        {
                                memset(buffer, 0x00, 1024*RC_SIZE_K);
                        }
                        else
                        {
                                nReturn = 0;
                                goto exit;
                        }

                        fread(buffer, sizeof(char), 1024*RC_SIZE_K, fd);
                        sprintf(arrBeginTag, "#<< %s", pTag);

                        if ((pString = strstr(buffer, arrBeginTag)) == NULL)
                        {
                                nReturn = 0;
                                goto exit;
                        }


                        pString = ifx_GetCfgDatafromString(pString, pData);

                        if (pString == NULL)
                        {
                                nReturn = 0;
                                goto exit;
                        }

                        strcpy(pRetValue, pString);
                        nReturn = 1;
                }
                else
                {
                        nReturn = 0;
                }
        }
        else
        {
                memset(sValue, 0x00, 1024);
                nLine = atoi(pData);

                if ((fd = popen(pFileName, "r")) == NULL)
                {
                        nReturn = 0;
                        goto exit;
                }

                while ( fgets(sValue, 1024, fd) )
                {
                        if (nIndex == nLine)
                        {
                                sValue[strlen(sValue)-1] = 0; // remove "\n" and keep string only
                                strcpy(pRetValue, sValue);
                                nReturn = 1;
                                goto exit;
                        }
                        nIndex++;
                }

                nReturn = 0;
        }

exit:

		if(pTag)
        {
                if (fd)
                {
                        fclose(fd);
                }
        }
        else
        {
                if (fd)
                {
                        pclose(fd);
                }
        }

        return nReturn;
}




int ifx_run_command(char *command)
{
	int status =0,ret = 0; 

	status = system(command);
	printf("system status is %d \n",status);
		if(status != -1)
		{
			waitpid(-1,&status,0);

			if (WIFEXITED(status))     
    	{
					ret = WEXITSTATUS(status);
        	IFX_DBG("\n Command status is %d\n", ret);
	    } 
			else
			{
    		printf("\n ERROR : system command not exited properly \n");
				exit(0);
			}

		}

  return ret;
}



int ifx_dhcp_renew(char * itf_name)
{
	sprintf(ifx_command_buff,"%s %s",DHCP_COMMAND,itf_name);
	IFX_DBG("\n Now Command is %s\n",ifx_command_buff);
	return ifx_run_command(ifx_command_buff);
}




int ifx_dhcp_release(char * itf_name)
{
	sprintf(ifx_command_buff,"%s -r %s",DHCP_COMMAND,itf_name);
	IFX_DBG("\n Now Command is %s\n",ifx_command_buff);
	return ifx_run_command(ifx_command_buff);
}



