/*
    Copyright (C) 2002  Thomas Ries <tries@gmx.net>

    This file is part of Siproxd.
    
    Siproxd is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.
    
    Siproxd is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
    
    You should have received a copy of the GNU General Public License
    along with Siproxd; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA 
*/

#include "config.h"

#include <stdio.h>
#include <time.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include <sys/types.h>

#include <netinet/in.h>
#include <arpa/inet.h>

#include <osipparser2/osip_parser.h>

#include "siproxd.h"
#include "log.h"


static char const ident[]="$Id: register.c,v 1.41 2004/06/21 18:09:36 hb9xar Exp $";

/* configuration storage */
extern struct siproxd_config configuration;

struct urlmap_s urlmap[URLMAP_SIZE];		/* URL mapping table     */
int force_lcl_masq;
extern int errno;
/*
 * initialize the URL mapping table
 */


void register_init(void) {
   FILE *stream;
   int sts, size, i;
   char buff[128];
     DEBUGC(DBCLASS_REG,"urlmap size=%i",sizeof(urlmap));

   memset(urlmap, 0, sizeof(urlmap));
//   register_shut() ;
//	return;
   if (configuration.registrationfile) {
      stream = fopen(configuration.registrationfile, "r");

      if (!stream) {
         /*
          * the file does not exist, or the size was incorrect,
          * delete it and start from scratch
          */
         unlink(configuration.registrationfile);
//         WARN("registration file not found, starting with empty table");
      } else {
         /* read the url table from file */
         for (i=0;i < URLMAP_SIZE; i++) {
            fgets(buff, sizeof(buff), stream);
            sts=sscanf(buff, "***:%i:%i", &urlmap[i].active, &urlmap[i].expires);
            if (urlmap[i].active) {
               osip_uri_init(&urlmap[i].true_url);
               osip_uri_init(&urlmap[i].masq_url);
               osip_uri_init(&urlmap[i].reg_url);

	       #define R(X) {\
               fgets(buff, sizeof(buff), stream);\
               buff[sizeof(buff)-1]='\0';\
               if (strchr(buff, 10)) *strchr(buff, 10)='\0';\
               if (strchr(buff, 13)) *strchr(buff, 13)='\0';\
               if (strlen(buff) > 0) {\
                  size = strlen(buff);\
                  X    =(char*)malloc(size+1);\
                  sts=sscanf(buff,"%s",X);\
               } else {\
                  X = NULL;\
               }\
               }

               R(urlmap[i].true_url->scheme);
               R(urlmap[i].true_url->username);
               R(urlmap[i].true_url->host);
               R(urlmap[i].true_url->port);
               R(urlmap[i].masq_url->scheme);
               R(urlmap[i].masq_url->username);
               R(urlmap[i].masq_url->host);
               R(urlmap[i].masq_url->port);
               R(urlmap[i].reg_url->scheme);
               R(urlmap[i].reg_url->username);
               R(urlmap[i].reg_url->host);
               R(urlmap[i].reg_url->port);
            }
         }
         fclose(stream);
      }
   }
   return;
}


/*
 * shut down the URL mapping table
 */
void register_shut(void) {
   int i;
#define registrationfile1 "/var/user_r"   
   FILE *stream, *stream1;

   if (configuration.registrationfile) {
      /* write urlmap back to file */
      stream = fopen(configuration.registrationfile, "w+");
      if (!stream) {
         /* try to unlink it and open again */
         unlink(configuration.registrationfile);
         stream = fopen(configuration.registrationfile, "w+");

         /* open file for write failed, complain */
         if (!stream) {
            ERROR("unable to write registration file");
            return;
         }
      }
   if (registrationfile1) {
      /* write urlmap back to file */
      stream1 = fopen(registrationfile1, "w+");
      if (!stream) {
         /* try to unlink it and open again */
         unlink(registrationfile1);
         stream1 = fopen(registrationfile1, "w+");

         /* open file for write failed, complain */
         if (!stream1) {
            ERROR("unable to write registration file");
            return;
         }
      }
    }
      for (i=0;i < URLMAP_SIZE; i++) {
//         fprintf(stream, "***:%i:%i\n", urlmap[i].active, urlmap[i].expires);
         if (urlmap[i].active) {
         fprintf(stream, "***:%i:%i\n", urlmap[i].active, urlmap[i].expires);
            #define W(X) fprintf(stream, "%s\n", (X)? X:"");
            #define W1(X) fprintf(stream1, "%s\n", (X)? X:"");

            W(urlmap[i].true_url->scheme);
            W(urlmap[i].true_url->username);
//            if (strcmp(urlmap[i].true_url->username,"299") != 0)
            W1(urlmap[i].true_url->username);
            W(urlmap[i].true_url->host);
            W(urlmap[i].true_url->port);
#if 1//peter 1210
            W(urlmap[i].masq_url->scheme);
            W(urlmap[i].masq_url->username);
            W(urlmap[i].masq_url->host);
            W(urlmap[i].masq_url->port);
            W(urlmap[i].reg_url->scheme);
            W(urlmap[i].reg_url->username);
            W(urlmap[i].reg_url->host);
            W(urlmap[i].reg_url->port);
#endif
         }
      }
      fclose(stream);
      fclose(stream1);
   }
   return;
}


/*
 * handles register requests and updates the URL mapping table
 *
 * RETURNS:
 *    STS_SUCCESS : successfully registered
 *    STS_FAILURE : registration failed
 *    STS_NEED_AUTH : authentication needed
 */
int register_client(sip_ticket_t *ticket, int force_lcl_masq1) {
   int i, j, n, sts;
   int expires;
   time_t time_now;
   osip_uri_t *url1_to, *url1_contact;
   osip_uri_t *url2_to;
   osip_header_t *expires_hdr;
   osip_uri_param_t *expires_param=NULL;
   struct in_addr addr;	  //peter
   struct in_addr addr1;  //peter
   unsigned short pt; //peter
   char *addrstr;
   char tmp[80];
   char p_chr[8];
   pt = (ticket->from.sin_port >>8) | (ticket->from.sin_port << 8);  //peter

   /*
    * Authorization - do only if I'm not just acting as outbound proxy
    * but am ment to be the registrar
    */
#if 1 //peter 1109
   if (force_lcl_masq == 0) {
      /*
       * RFC 3261, Section 16.3 step 6
       * Proxy Behavior - Request Validation - Proxy-Authorization
       */
      sts = authenticate_proxy(ticket);
      if (sts == STS_FAILURE) {
         /* failed */
         WARN("proxy authentication failed for %s@%s",
              (ticket->sipmsg->to->url->username)? 
              ticket->sipmsg->to->url->username : "*NULL*",
              ticket->sipmsg->to->url->host);
         return STS_FAILURE;
      } else if (sts == STS_NEED_AUTH) {
         /* needed */
         DEBUGC(DBCLASS_REG,"proxy authentication needed for %s@%s",
                ticket->sipmsg->to->url->username,
                ticket->sipmsg->to->url->host);
         return STS_NEED_AUTH;
      }
   }
#endif //peter 1109
/*
   fetch 1st Via entry and remember this address. Incoming requests
   for the registered address have to be passed on to that host.

   To: -> address to be registered
   Contact: -> host is reachable there
               Note: in case of un-REGISTER, the contact header may
                     contain '*' only - which means "all registrations
                     made by this UA"
   
   => Mapping is
   To: <1--n> Contact
   
*/
   time(&time_now);

   DEBUGC(DBCLASS_REG,"sip_register:");

   /* evaluate Expires Header field */
   osip_message_get_expires(ticket->sipmsg, 0, &expires_hdr);

  /*
   * look for an Contact expires parameter - in case of REGISTER
   * these two are equal. The Contact expires has higher priority!
   */
   osip_contact_param_get_byname(
           (osip_contact_t*) ticket->sipmsg->contacts->node->element,
           EXPIRES, &expires_param);

   if (expires_param && expires_param->gvalue) {
      /* get expires from contact Header */
      expires=atoi(expires_param->gvalue);
   } else if (expires_hdr && expires_hdr->hvalue) {
      /* get expires from expires Header */
      expires=atoi(expires_hdr->hvalue);
   } else {
      /* it seems, the expires field is not present everywhere... */
      DEBUGC(DBCLASS_REG,"no 'expires' header found - set time to 600 sec");
      expires=120;
      osip_message_set_expires(ticket->sipmsg, "120");

   }

   url1_to=ticket->sipmsg->to->url;



   /*
    * REGISTER
    */
   if (expires > 0) {
      /*
       * First make sure, we have a prober Contact header:
       *  - url
       *  - url -> hostname
       *
       * Libosip parses an:
       * "Contact: *"
       * the following way (Note: Display name!! and URL is NULL)
       * (gdb) p *((osip_contact_t*)(sip->contacts->node->element))
       * $5 = {displayname = 0x8af8848 "*", url = 0x0, gen_params = 0x8af8838}
       */
      url1_contact=((osip_contact_t*)
                   (ticket->sipmsg->contacts->node->element))->url;
      if ((url1_contact == NULL) || (url1_contact->host == NULL)) {
         /* Don't have reqiured Contact fields */
         ERROR("tried registration with empty Contact header");
         return STS_FAILURE;
      }
         
      DEBUGC(DBCLASS_REG,"register: %s@%s expires=%i seconds",
          (url1_contact->username) ? url1_contact->username : "*NULL*",
          (url1_contact->host) ? url1_contact->host : "*NULL*",
          expires);

      /*
       * Update registration. There are two possibilities:
       * - already registered, then update the existing record
       * - not registered, then create a new record
       */

      j=-1;
      for (i=0; i<URLMAP_SIZE; i++) {
         if (urlmap[i].active == 0) {
	    if (j < 0) j=i; /* remember first hole */
            continue;
         }

         url2_to=urlmap[i].reg_url;
 

         /* check address-of-record ("public address" of user) */

 //012205 peter     if (strcmp(url1_contact->username,url2_to->username) ==0)  //check username only
      if (strcmp(ticket->sipmsg->to->url->username,url2_to->username) ==0)  //check username only
 //peter        if (compare_url(url1_to, url2_to)==STS_SUCCESS)
		 {
            DEBUGC(DBCLASS_REG, "found entry for %s@%s <-> %s@%s at "
                   "slot=%i, exp=%li",
	           (url1_contact->username) ? url1_contact->username : "*NULL*",
                   (url1_contact->host) ? url1_contact->host : "*NULL*",
	           (url2_to->username) ? url2_to->username : "*NULL*",
                   (url2_to->host) ? url2_to->host : "*NULL*",
		   i, urlmap[i].expires-time_now);
//peter 051227		   
      if (expires > 0) expires+=30;

      /* update registration timeout */
      urlmap[i].expires=time_now+expires;
      DEBUGC(DBCLASS_REG,"expire[%i]=%i",i,urlmap[i].expires);   
//peter 051227		   
		   
            break;
         }
      }


      if ( (j < 0) && (i >= URLMAP_SIZE) ) {
         /* oops, no free entries left... */
         ERROR("URLMAP is full - registration failed");
         return STS_FAILURE;
      }

      if (i >= URLMAP_SIZE) {

 #if 0 //peter 1109
      sts = authenticate_proxy(ticket);
      if (sts == STS_FAILURE) {
         /* failed */
         WARN("proxy authentication failed for %s@%s",
              (ticket->sipmsg->to->url->username)? 
              ticket->sipmsg->to->url->username : "*NULL*",
              ticket->sipmsg->to->url->host);
         return STS_FAILURE;
      } else if (sts == STS_NEED_AUTH) {
         /* needed */
         DEBUGC(DBCLASS_REG,"proxy authentication needed for %s@%s",
                ticket->sipmsg->to->url->username,
                ticket->sipmsg->to->url->host);
         return STS_NEED_AUTH;
      }
#endif //peter 1109

         /* entry not existing, create new one */
         i=j;

         /* write entry */
         urlmap[i].active=1;
         /* Contact: field */
         osip_uri_clone( ((osip_contact_t*)
                         (ticket->sipmsg->contacts->node->element))->url, 
        	         &urlmap[i].true_url);
#if  1 //051215 do not reset expire time
       if (strcmp(urlmap[i].true_url->host,utils_inet_ntoa(ticket->from.sin_addr)) !=0) {
		   urlmap[i].active=2;
       }
#endif    
#if 1   
          addrstr=utils_inet_ntoa(ticket->from.sin_addr)  ;
           urlmap[i].true_url->host=realloc(urlmap[i].true_url->host,
                                             strlen(addrstr)+1);
            strcpy(urlmap[i].true_url->host, addrstr);
               urlmap[i].true_url->port=realloc(urlmap[i].true_url->port, 16);
               sprintf(urlmap[i].true_url->port, "%i",pt);
#endif
        DEBUGC(DBCLASS_REG,"create new entry for %s@%s:%s from.sin_addr at slot=%i pt=%i",
        urlmap[i].true_url->username,urlmap[i].true_url->host,urlmap[i].true_url->port,i,pt);
//peter
         /* To: field */
         osip_uri_clone( ticket->sipmsg->to->url, 
        	    &urlmap[i].reg_url);

         DEBUGC(DBCLASS_REG,"create new entry for %s@%s <-> %s@%s at slot=%i",
                (url1_contact->username) ? url1_contact->username : "*NULL*",
                (url1_contact->host) ? url1_contact->host : "*NULL*",
	        (urlmap[i].reg_url->username) ? urlmap[i].reg_url->username : "*NULL*",
                (urlmap[i].reg_url->host) ? urlmap[i].reg_url->host : "*NULL*",
                i);

         /*
          * try to figure out if we ought to do some masquerading
          */
         osip_uri_clone( ticket->sipmsg->to->url,
        	         &urlmap[i].masq_url);
#if 1 //peter 1221
//051122       if (urlmap[i].reg_url->port == NULL) 
        {
               urlmap[i].reg_url->port=realloc(urlmap[i].reg_url->port, 16);
               sprintf(urlmap[i].reg_url->port, "%i",
                       configuration.sip_listen_port);
               urlmap[i].masq_url->port=realloc(urlmap[i].masq_url->port, 16);
               sprintf(urlmap[i].masq_url->port, "%i",
                       configuration.sip_listen_port);

       }
       
         DEBUGC(DBCLASS_REG,"p_chr");      
       
#endif //peter 1221
#if 1 //peter 012205 for windows messenger 5.1. sip function
//      if (strcmp(urlmap[i].true_url->username,urlmap[i].reg_url->username) !=0)
      {
         DEBUGC(DBCLASS_REG,"p_chr");      
      	
      addrstr=urlmap[i].reg_url->username;
         urlmap[i].true_url->username=realloc(urlmap[i].true_url->username,
                                             strlen(addrstr)+1);
            strcpy(urlmap[i].true_url->username, addrstr);
      }
#endif
         DEBUGC(DBCLASS_REG,"p_chr");      

         n=configuration.mask_host.used;
         if (n != configuration.masked_host.used) {
            ERROR("# of mask_host is not equal to # of masked_host in config!");
            n=0;
         }

         DEBUG("%i entries in MASK config table", n);
         for (j=0; j<n; j++) {
            DEBUG("compare [%s] <-> [%s]",configuration.mask_host.string[j],
                  ticket->sipmsg->to->url->host);
            if (strcmp(configuration.mask_host.string[j],
                ticket->sipmsg->to->url->host)==0)
               break;
         }
         if (j<n) { 
            /* we are masquerading this UA, replace the host part of the url */
            DEBUGC(DBCLASS_REG,"masquerading UA %s@%s as %s@%s",
                   (url1_contact->username) ? url1_contact->username : "*NULL*",
                   (url1_contact->host) ? url1_contact->host : "*NULL*",
                   (url1_contact->username) ? url1_contact->username : "*NULL*",
                   configuration.masked_host.string[j]);
            urlmap[i].masq_url->host=realloc(urlmap[i].masq_url->host,
                                    strlen(configuration.masked_host.string[j])+1);
            strcpy(urlmap[i].masq_url->host, configuration.masked_host.string[j]);
         }

         /*
          * for transparent proxying: force device to be masqueraded
          * as with the outbound IP
          */
    //     force_lcl_masq = 1 ;//peter
#if 0 //peter 051225    
         if (force_lcl_masq ) {
     //       struct in_addr addr;
            char *addrstr;

            if (get_ip_by_ifname(configuration.outbound_if, &addr) !=
                STS_SUCCESS) {
               ERROR("can't find outbound interface %s - configuration error?",
                     configuration.outbound_if);
               return STS_FAILURE;
            }

            /* host part */
            addrstr = utils_inet_ntoa(addr);
            DEBUGC(DBCLASS_REG,"masquerading UA %s@%s local %s@%s",
                   (url1_contact->username) ? url1_contact->username : "*NULL*",
                   (url1_contact->host) ? url1_contact->host : "*NULL*",
                   (url1_contact->username) ? url1_contact->username : "*NULL*",
                   addrstr);
            urlmap[i].masq_url->host=realloc(urlmap[i].masq_url->host,
                                             strlen(addrstr)+1);
            strcpy(urlmap[i].masq_url->host, addrstr);

            /* port number if required */
            if (configuration.sip_listen_port != SIP_PORT) {
               urlmap[i].masq_url->port=realloc(urlmap[i].masq_url->port, 16);
               sprintf(urlmap[i].masq_url->port, "%i",
                       configuration.sip_listen_port);
            }
         }

      } else 
#endif       
      { /* if new entry */
//       DEBUGC(DBCLASS_REG,"found entry for %s@%s:%s from.sin_addr at slot=%i",
//        urlmap[i].true_url->username,urlmap[i].true_url->host,urlmap[i].true_url->port,i);

      /*
       * Some phones (like BudgeTones *may* dynamically grab a SIP port
       * so we might want to update the true_url and reg_url each time
       * we get an REGISTER
       */
         /* Contact: field */

      	urlmap[i].active=1;  //1228 peter
         osip_uri_free(urlmap[i].true_url);
         osip_uri_clone( ((osip_contact_t*)
               (ticket->sipmsg->contacts->node->element))->url, 
        	         &urlmap[i].true_url);
 
#if 1 //peter 051225
       if (strcmp(urlmap[i].true_url->host,utils_inet_ntoa(ticket->from.sin_addr)) !=0) {
		 urlmap[i].active=2;
       }
#endif    
#if 1    
          addrstr=utils_inet_ntoa(ticket->from.sin_addr)  ;
           urlmap[i].true_url->host=realloc(urlmap[i].true_url->host,
                                             strlen(addrstr)+1);
            strcpy(urlmap[i].true_url->host, addrstr);
               urlmap[i].true_url->port=realloc(urlmap[i].true_url->port, 16);
               sprintf(urlmap[i].true_url->port, "%i",pt);
#endif
//peter change contact to from 
#if 0
         osip_uri_set_host(urlmap[i].true_url, osip_strdup(utils_inet_ntoa(ticket->from.sin_addr)));

         sprintf(p_chr, "%i", pt);
         osip_uri_set_port(urlmap[i].true_url, osip_strdup(p_chr));

#endif
       DEBUGC(DBCLASS_REG,"new found entry for %s@%s:%s from.sin_addr at slot=%i",
        urlmap[i].true_url->username,urlmap[i].true_url->host,urlmap[i].true_url->port,i);
//peter

         /* To: field */
         osip_uri_free(urlmap[i].masq_url);
       osip_uri_clone( ticket->sipmsg->to->url, 
        	         &urlmap[i].masq_url);
 
         osip_uri_free(urlmap[i].reg_url);
         osip_uri_clone( ticket->sipmsg->to->url, 
        	    &urlmap[i].reg_url);
      }
#if 1 //peter 1221
//051205       if (urlmap[i].reg_url->port == NULL) 
        {
               urlmap[i].reg_url->port=realloc(urlmap[i].reg_url->port, 16);
               sprintf(urlmap[i].reg_url->port, "%i",
                       configuration.sip_listen_port);
               urlmap[i].masq_url->port=realloc(urlmap[i].masq_url->port, 16);
               sprintf(urlmap[i].masq_url->port, "%i",
                       configuration.sip_listen_port);
	       

       }
#endif //peter 1221
#

#if 1 //peter 012205 for windows messenger 5.1. sip function
//051209      if (strcmp(urlmap[i].true_url->username,urlmap[i].reg_url->username) !=0)
      {
      addrstr=urlmap[i].reg_url->username;
         urlmap[i].true_url->username=realloc(urlmap[i].true_url->username,
                                             strlen(addrstr)+1);
            strcpy(urlmap[i].true_url->username, addrstr);
      }
#endif
 
      /* give some safety margin for the next update */
      if (expires > 0) expires+=30;

      /* update registration timeout */
      urlmap[i].expires=time_now+expires;
      DEBUGC(DBCLASS_REG,"expire[%i]=%i",i,urlmap[i].expires);  
       

//peter     
           if (get_ip_by_ifname(configuration.outbound_if, &addr) !=
                STS_SUCCESS) {
               ERROR("can't find outbound interface %s - configuration error?",
                     configuration.outbound_if);
               return STS_FAILURE;
            }
           if (get_ip_by_ifname(configuration.inbound_if, &addr1) !=
                STS_SUCCESS) {
               ERROR("can't find outbound interface %s - configuration error?",
                     configuration.outbound_if);
               return STS_FAILURE;
            }
#if 1 //051227
            if (strcmp(urlmap[i].reg_url->host,"wan") ==0 ){
	    
          addrstr=utils_inet_ntoa(addr)  ;
         DEBUGC(DBCLASS_REG,"out_if=%s  %s",utils_inet_ntoa(addr),addrstr);      
           urlmap[i].reg_url->host=realloc(urlmap[i].reg_url->host,
                                             strlen(addrstr)+1);
            strcpy(urlmap[i].reg_url->host, addrstr);
           urlmap[i].masq_url->host=realloc(urlmap[i].masq_url->host,
                                             strlen(addrstr)+1);
            strcpy(urlmap[i].masq_url->host, addrstr);
	    
            }
#endif

	    	    
 //peter 1112
		 if ((memcmp(&addr1,&ticket->from.sin_addr,2) == 0) && (urlmap[i].active !=2 ))	urlmap[i].active=3;
#if 1
      if (strcmp(urlmap[i].true_url->host,urlmap[i].reg_url->host) ==0) {

          addrstr="127.0.0.1"  ;
           urlmap[i].true_url->host=realloc(urlmap[i].true_url->host,
                                             strlen(addrstr)+1);
            strcpy(urlmap[i].true_url->host, addrstr);
       		urlmap[i].active=1;  //peter 1113  internal  register

//           strcpy(urlmap[i].true_url->host,"127.0.0.1");	  
        }
#endif         
#if 1
       if (strcmp(urlmap[i].reg_url->host,utils_inet_ntoa(addr1)) ==0) {
          addrstr=utils_inet_ntoa(addr)  ;
           urlmap[i].masq_url->host=realloc(urlmap[i].masq_url->host,
                                             strlen(addrstr)+1);
            strcpy(urlmap[i].masq_url->host, addrstr);

//            strcpy(urlmap[i].masq_url->host,utils_inet_ntoa(addr));
	    	if (urlmap[i].active!=2) urlmap[i].active=3;  //peter 1113  internal  register
	    } 
//051122		 if ((memcmp(&addr1,&ticket->from.sin_addr,2) == 0) && (urlmap[i].active !=2 ))	urlmap[i].active=3;
    //   else   strcpy(urlmap[i].masq_url->host,utils_inet_ntoa(addr));
      if (strcmp(urlmap[i].true_url->host,urlmap[i].masq_url->host) ==0) {	 //1109
          addrstr="127.0.0.1"  ;
           urlmap[i].true_url->host=realloc(urlmap[i].true_url->host,
                                             strlen(addrstr)+1);
            strcpy(urlmap[i].true_url->host, addrstr);

//            strcpy(urlmap[i].true_url->host,"127.0.0.1");
   		urlmap[i].active=1;  //peter 1113  internal  register
		  }
#endif
#if 0
      if (strcmp(urlmap[i].true_url->host,urlmap[i].reg_url->host) ==0) {

          addrstr="127.0.0.1"  ;
           urlmap[i].true_url->host=realloc(urlmap[i].true_url->host,
                                             strlen(addrstr)+1);
            strcpy(urlmap[i].true_url->host, addrstr);
       		urlmap[i].active=1;  //peter 1113  internal  register

//           strcpy(urlmap[i].true_url->host,"127.0.0.1");	  
        }
#endif         
}   
#if 1 //peter 051225
   
   if (force_lcl_masq != 0) {
      if (strcmp(urlmap[i].true_url->username,"299") !=0) {
      strcpy(tmp,urlmap[i].true_url->username);
      if (auth_getpwd(tmp) == NULL){	 
      sprintf(tmp,"echo \"%s=%s\" >> /nv/user",urlmap[i].true_url->username,urlmap[i].true_url->username);
      system(tmp);
      system("rawaccess -s");
      }     
      }  
   }    
#endif //051225   

  //peter          
   /*
    * un-REGISTER
    */
   } else { /* expires > 0 */
      /*
       * Remove registration
       * Siproxd will ALWAYS remove ALL bindings for a given
       * address-of-record
       */
      for (i=0; i<URLMAP_SIZE; i++) {
         if (urlmap[i].active == 0) continue;

         url2_to=urlmap[i].reg_url;

         if (compare_url(url1_to, url2_to)==STS_SUCCESS) {
            DEBUGC(DBCLASS_REG, "removing registration for %s@%s at slot=%i",
	           (url2_to->username) ? url2_to->username : "*NULL*",
                   (url2_to->host) ? url2_to->host : "*NULL*", i);
            urlmap[i].expires=0;
            urlmap[i].active=0;
         osip_uri_free(urlmap[i].true_url);
         osip_uri_free(urlmap[i].masq_url);
         osip_uri_free(urlmap[i].reg_url);

            break;
         }
      }
   }
   register_shut();
   return STS_SUCCESS;
}



/*
 * cyclically called to do the aging of the URL mapping table entries
 * and throw out expired entries.
 */
void register_agemap(void) {
   int i,j;
   time_t t;
   j=1;    
   time(&t);
   if (RWReg()) force_lcl_masq=t+120;
   if (t > force_lcl_masq) force_lcl_masq=0;
//   return;//peter
//   DEBUGC(DBCLASS_REG,"sip_agemap, t=%i",(int)t);
   for (i=0; i<URLMAP_SIZE; i++) {
      if ((urlmap[i].active != 0) && (urlmap[i].expires < t)) {
	 DEBUGC(DBCLASS_REG,"cleaned entry:%i %s@%s", i,
	        urlmap[i].true_url->username,  urlmap[i].true_url->host);
         urlmap[i].active=0;
         urlmap[i].expires=0;
         j=0;
         osip_uri_free(urlmap[i].true_url);
         osip_uri_free(urlmap[i].masq_url);
         osip_uri_free(urlmap[i].reg_url);
//	 osip_via_free(urlmap[i].via);
      }
   }
   if (j==0) register_shut();
   return;
}


/*
 * send answer to a registration request.
 *  flag = STS_SUCCESS    -> positive answer (200)
 *  flag = STS_FAILURE    -> negative answer (503)
 *  flag = STS_NEED_AUTH  -> proxy authentication needed (407)
 *
 * RETURNS
 *	STS_SUCCESS on success
 *	STS_FAILURE on error
 */
int register_response(sip_ticket_t *ticket, int flag) {
   osip_message_t *response;
   int code;
   int sts;
   osip_via_t *via;
   int port;
   char *buffer;
   struct in_addr addr;
   osip_header_t *expires_hdr;
	   unsigned short pt; //peter

   /* ok -> 200, fail -> 503 */
   switch (flag) {
   case STS_SUCCESS:
      code = 200;	/* OK */
      break;
   case STS_FAILURE:
      code = 503;	/* failed */
      break;
   case STS_NEED_AUTH:
      code = 407;	/* proxy authentication needed */
      break;
   default:
      code = 503;	/* failed */
      break;
   }

   /* create the response template */
   if ((response=msg_make_template_reply(ticket, code))==NULL) {
      ERROR("register_response: error in msg_make_template_reply");
      return STS_FAILURE;
   }

   /* insert the expiration header */
   osip_message_get_expires(ticket->sipmsg, 0, &expires_hdr);
   if (expires_hdr) {
      osip_message_set_expires(response, expires_hdr->hvalue);
   }

   /* if we send back an proxy authentication needed, 
      include the Proxy-Authenticate field */
   if (code == 407) {
//      auth_include_authrq(ticket);
      auth_include_authrq1(response);	  //peter 
   }

   /* get the IP address from existing VIA header */
   osip_message_get_via (response, 0, &via);
   if (via == NULL) {
      ERROR("register_response: Cannot send response - no via field");
      return STS_FAILURE;
   }

   /* name resolution needed? */
   if (utils_inet_aton(via->host,&addr) == 0) {
      /* yes, get IP address */
      sts = get_ip_by_host(via->host, &addr);
      if (sts == STS_FAILURE) {
         DEBUGC(DBCLASS_REG, "register_response: cannot resolve VIA [%s]",
                via->host);
         return STS_FAILURE;
      }
   }   
#if 1 //peter 
   sts = osip_message_to_str(response, &buffer);
   if (sts != 0) {
      ERROR("register_response: msg_2char failed");
      return STS_FAILURE;
   }

   /* send answer back */
   if (via->port) {
      port=atoi(via->port);
   } else {
      port=configuration.sip_listen_port;
   }
//   sipsock_send(addr, port, ticket->protocol, buffer, strlen(buffer));
#endif //peter
//   unsigned
//   short pt;

   pt = (ticket->from.sin_port >>8) | (ticket->from.sin_port << 8);
   sipsock_send(ticket->from.sin_addr, pt, ticket->protocol, buffer, strlen(buffer));

   /* free the resources */
   osip_message_free(response);
   free(buffer);
   return STS_SUCCESS;
}

