/* ====================================================================
 * Copyright (c) 2004 - Infeneon-ADMtek Technologies AG.
 *
 * All rights reserved.
 * ====================================================================
 *
 * ====================================================================
 *
 * This document contains proprietary information belonging to Infineon
 * Technologies AG. Passing on and copying of this document, and communication
 * of its contents is not permitted without prior written authorisation.
 *
 * ====================================================================
 */

/* ====================================================================
 *
 * File Name: ifx_raw_drv.c
 * Author: Eric Tsai
 * Date:
 *
 * ====================================================================
 *
 * Projects: <ADM5120/Raw_Access>
 * Block: 
 *
 * ====================================================================
 * Contents:
 *
 * ====================================================================
 * References:
 */

#ifndef __KERNEL__
#  define __KERNEL__
#endif				/*  */
#ifndef MODULE
#  define MODULE
#endif				/*  */

#include <linux/config.h>
#include <linux/module.h>
#include <linux/init.h>
#include <linux/netdevice.h>

#include <asm/uaccess.h>
#include <asm/delay.h>
#ifdef ADMPLAT
#include <asm/am5120/bsp_cfg.h>
#include <asm/am5120/am5120.h>
#include <asm/am5120/if5120.h>
#include <asm/am5120/flash.h>
#include <asm/atomic.h>
#endif				/*  */
#include "ifx_raw_drv.h"

char8 *cBuf = NULL;
int32 iBufOff = 0;
int32 iBufSize = FLASH_BLOCK_SIZE;
#ifdef ADMPLAT
char8 cBoardResv[AM5120_BOARD_RESV_SIZE];
uint32 uiRawAcsAddr = RAW_ACCESS_ADDR;
char8 icode[] = "RAWACS";
Boolean bMflag;
#endif

#ifdef ADMPLAT
void atomic_add(int i, atomic_t * v)
{
  i = i;
  v = v;
} void atomic_sub(int i, atomic_t * v)
{
  i = i;
  v = v;
}
#endif				/*  */

int raw_open(struct inode *inode, struct file *filp)
{
#ifdef DEBUG_RAWDEV
  printk("(raw_acs)raw_open\n");
#endif				/*  */
  MOD_INC_USE_COUNT;
  return 0;
}

ssize_t raw_read(struct file * filp, char *buf, size_t count,
		 loff_t * ppos)
{
  int32 ret = -1;
  T_IFX_RAW_INFO tRawInfo;

#ifdef ADMPLAT
#ifdef CONFIG_ADM5120_NOR_CFG
  memset(cBuf, 0, iBufSize);
  memcpy((char *) cBuf, (char *) PA2VA(uiRawAcsAddr), iBufSize);
  if (cBuf != NULL) {
#ifdef DEBUG_RAWDEV
    printk("(raw_acs)flash_read successfully!\n");
#endif				/*  */
  } else {
#ifdef DEBUG_RAWDEV
    printk("(raw_acs)flash_read failed!\n");
#endif				/*  */
    return 0;
  }

  iBufOff = 0;
#elif CONFIG_ADM5120_NAND_CFG
  nand_read((unsigned char *) cBuf,
	    (unsigned char *) iRawAcsAddr, iBufSize);
#ifdef DEBUG_RAWDEV
  printk("(raw_acs)nand_read\n");
#endif				/*  */
#endif				/*  */
  if (uiRawAcsAddr == RAW_ACCESS_ADDR) {
    iBufOff += AM5120_BOARD_RESV_SIZE;
    /*memcpy(cTmp, cBuf + iBufOff, sizeof(icode));
    iBufOff += sizeof(icode);
    memcpy(&rcount, cBuf + iBufOff, sizeof(uint32));
    iBufOff += sizeof(uint32);
    if ((memcmp(cTmp, icode, sizeof(icode)) != 0)
	|| (rcount >
	    (iBufSize - AM5120_BOARD_RESV_SIZE - sizeof(icode) -
	     sizeof(uint32)))) {
      printk("(raw_acs)Data error!\n");
      return 0;
    }*/
    memcpy(&tRawInfo, cBuf + iBufOff, sizeof(T_IFX_RAW_INFO));
    if ((memcmp(tRawInfo.cICode, icode, sizeof(icode)) != 0)
	|| (tRawInfo.iCompSize >
	    (iBufSize - AM5120_BOARD_RESV_SIZE - sizeof(T_IFX_RAW_INFO)))) {
      printk("(raw_acs)Data error!\n");
      return 0;
    }

    if (count >= (tRawInfo.iCompSize + sizeof(T_IFX_RAW_INFO))) {
      ret = copy_to_user(buf, cBuf + iBufOff, (tRawInfo.iCompSize+sizeof(T_IFX_RAW_INFO)));
      if (ret == 0) {
	return (tRawInfo.iCompSize + sizeof(T_IFX_RAW_INFO));
      }
    } else {
      ret = copy_to_user(buf, cBuf + iBufOff, count);
      if (ret == 0) {
	return count;
      }
    }
  } else {
    if (count >= (iBufSize - iBufOff)) {
      ret = copy_to_user(buf, cBuf + iBufOff, iBufSize - iBufOff);
      if (ret == 0) {
	return (iBufSize - iBufOff);
      }
    } else {
      ret = copy_to_user(buf, cBuf + iBufOff, count);
      if (ret == 0) {
	return count;
      }
    }
  }
#else
  iBufOff = 0;
  if (count >= (iBufSize - iBufOff)) {
    ret = copy_to_user(buf, cBuf + iBufOff, iBufSize - iBufOff);
    if (ret == 0) {
      return (iBufSize - iBufOff);
    }
  } else {
    ret = copy_to_user(buf, cBuf + iBufOff, count);
    if (ret == 0) {
      return count;
    }
  }
#endif				/* ADMPLAT */

  return 0;
}

#ifdef ADMPLAT
#if defined(ADMTYPE) && (ADMTYPE == ADM5120P)
void jump_up_map(void)
{
  int i;
                                                                                
  /* Set GPIO[4](CS) to high */
  for (i = 0; i < 10; i++)
    udelay(1000);
  ADM5120_SW_REG(GPIO_conf0_REG) = 0x080800f7;
  for (i = 0; i < 10; i++)
    udelay(1000);
  bMflag = 1;
  printk("(raw_acs)Jump to Up\n");
}

void jump_low_map(void)
{
  int i;
                                                                                
  /* Set GPIO[4](CS) to low */
  for (i = 0; i < 10; i++)
    udelay(1000);
  ADM5120_SW_REG(GPIO_conf0_REG) = 0x3ff;
  for (i = 0; i < 10; i++)
    udelay(1000);
  bMflag = 0;
  printk("(raw_acs)Jump to Low\n");
}
#endif
#endif

ssize_t raw_write(struct file * filp, const char *buf, size_t count,
		  loff_t * ppos)
{
  int32 ret = -1;
  T_IFX_RAW_INFO tRawInfo;

  if ((count > iBufSize) || (count <= 0)){
    printk("(raw_acs)%d bytes to be written\n", count);
    return 0;
  }

#ifdef DEBUG_RAWDEV
  printk("(raw_acs)AM5120_BOARD_CFG_ADDR: %x\n", AM5120_BOARD_CFG_ADDR);
  printk("(raw_acs)AM5120_BOARD_RESV_SIZE: %x\n", AM5120_BOARD_RESV_SIZE);
  printk("(raw_acs)Raw Access Addr: %x\n", uiRawAcsAddr);
#endif				/*  */

  memset(cBuf, 0, iBufSize);
  iBufOff = 0;
#ifdef ADMPLAT
  if ((uiRawAcsAddr == RAW_ACCESS_ADDR)&&(bMflag == 0)) {
    if (count > (iBufSize - AM5120_BOARD_RESV_SIZE)) {
      printk("(raw_acs)Data size error!\n");
      return 0;
    }
    memcpy((char *) cBuf, (char *) cBoardResv, AM5120_BOARD_RESV_SIZE);
    iBufOff += AM5120_BOARD_RESV_SIZE;
  }
#endif

  ret = copy_from_user(cBuf + iBufOff, buf, count);
  if (ret == 0) {
#ifdef ADMPLAT
    if ((uiRawAcsAddr == RAW_ACCESS_ADDR)&&(bMflag == 0)) {
      memcpy(&tRawInfo, cBuf + iBufOff, sizeof(T_IFX_RAW_INFO));
      if ((memcmp(tRawInfo.cICode, icode, sizeof(icode)) != 0)){
        printk("(raw_acs)Write data error!\n");
        return 0;
      }
    }
    iBufOff += count;
#ifdef CONFIG_ADM5120_NOR_CFG
    flash_erase((char *) PA2VA(uiRawAcsAddr), iBufSize);
    ret = flash_write((char *) PA2VA(uiRawAcsAddr), cBuf, iBufOff);
    if (ret != FLASH_ERROR) {
#ifdef DEBUG_RAWDEV
      printk("(raw_acs)flash_write successfully!\n");
#endif				/*  */
    } else {
#ifdef DEBUG_RAWDEV
      printk("(raw_acs)flash_write failed!\n");
#endif				/*  */
      return 0;
    }

#elif CONFIG_ADM5120_NAND_CFG
    nand_erase((unsigned char *) uiRawAcsAddr, iBufSize);
    nand_write((unsigned char *) uiRawAcsAddr,
	       (unsigned char *) cBuf, iBufOff);

#ifdef DEBUG_RAWDEV
    printk("(raw_acs)nand_write\n");
#endif				/*  */
#endif				/*  */
#endif				/* ADMPLAT */
    printk("(raw_acs)Write %d\n", count);
    return count;
  }else{
    printk("(raw_acs)ret: %d\n", ret);
  }
  return 0;
}

#ifdef ADMPLAT
static void erase_flash(void)
{
#ifdef CONFIG_ADM5120_NOR_CFG
  flash_erase((char *) PA2VA(uiRawAcsAddr), iBufSize);
  flash_write((char *) PA2VA(uiRawAcsAddr), (char *) cBoardResv,
	      AM5120_BOARD_RESV_SIZE);
#elif CONFIG_ADM5120_NAND_CFG
  nand_erase((unsigned char *) uiRawAcsAddr, iBufSize);
  nand_write((unsigned char *) uiRawAcsAddr, (unsigned char *) cBoardResv,
	     AM5120_BOARD_RESV_SIZE);
#endif				/*  */
}
#endif				/*  */

int raw_release(struct inode *inode, struct file *flip)
{

#ifdef DEBUG_RAWDEV
  printk("(raw_acs)raw_release\n");

#endif				/*  */
  MOD_DEC_USE_COUNT;
  return 0;
}

int raw_ioctl(struct inode *inode, struct file *flip, unsigned int cmd,
	      unsigned long arg)
{
  int *test;
  int rtvalue;
  test = kmalloc(100, GFP_KERNEL);
  switch (cmd) {
  case RAWDRV_IOCSFUNCTION:
    printk("(raw_acs)ioctl: RAWDRV_IOCSFUNCTION\n");
    rtvalue = copy_from_user(test, (int *) arg, sizeof(int));
    if (rtvalue != 0) {
#ifdef DEBUG_RAWDEV
      printk("(raw_acs)error\n");
#endif				/*  */
      break;
    }
#ifdef DEBUG_RAWDEV
    printk("(raw_acs)test value is %i\n", *test);
#endif				/*  */
    *test = 200;
    copy_to_user((int *) arg, test, sizeof(test));
    break;
  case RAWDRV_IOCGOUTLEN:
    printk("(raw_acs)ioctl: RAWDRV_IOCGOUTLEN\n");
    break;
  case RAWDRV_IOCSRAWACSADDR:	// Set raw access addr
    printk("(raw_acs)ioctl: RAWDRV_IOCSRAWACSADDR\n");
#ifdef ADMPLAT
    copy_from_user(&uiRawAcsAddr, (int *) arg, sizeof(int));
    if((uiRawAcsAddr + AM5120_BOARD_RESV_SIZE)>(AM5120_BASE_ADDR + FLASHSIZE)){
      printk("(raw_acs)Wrong address: %x!\n", uiRawAcsAddr);
      break;
    }
#if defined(ADMTYPE) && (ADMTYPE == ADM5120P)
    if(uiRawAcsAddr >= 0x1fe00000){
      if(bMflag == 0)
        jump_up_map(); 
      uiRawAcsAddr -= 0x200000;
    }
#endif
    printk("(raw_acs)Raw Access Addr: %x\n", uiRawAcsAddr);
#endif
    break;
  case RAWDRV_IOCRSRAWDEV:	// Reset raw acs
    printk("(raw_acs)ioctl: RAWDRV_IOCRSRAWACS\n");
    memset(cBuf, 0, iBufSize);
    iBufOff = 0;
#ifdef ADMPLAT
#if defined(ADMTYPE) && (ADMTYPE == ADM5120P)
    if(bMflag == 1) jump_low_map(); 
#endif
    uiRawAcsAddr = RAW_ACCESS_ADDR;
    printk("(raw_acs)Raw Access Addr: %x\n", uiRawAcsAddr);
#endif
    break;
  case RAWDRV_IOCRSFLASH:	// Erase Flash 
    printk("(raw_acs)ioctl: RAWDRV_IOCRSFLASH\n");
#ifdef ADMPLAT
    erase_flash();
#else
    memset(cBuf, 0, iBufSize);
    iBufOff = 0;
#endif
    break;
  case RAWDRV_IOCTEST:
    printk("(raw_acs)ioctl: RAWDRV_IOCTEST\n");
#ifdef ADMPLAT
    printk("(raw_acs)Board reserved size: %x\n", AM5120_BOARD_RESV_SIZE);
    printk("(raw_acs)Raw Access Addr: %x\n", uiRawAcsAddr);
#endif				/*  */
    break;
  default:
    return -EFAULT;
    break;
  }
  kfree(test);
  return 0;
}

struct file_operations raw_acs_fops = {
read:raw_read,
write:raw_write,
open:raw_open,
release:raw_release,
ioctl:raw_ioctl,
};

int init_module(void)
{
  int result;
  result = register_chrdev(100, "ifx_raw_acs", &raw_acs_fops);
  if (result < 0) {

#ifdef DEBUG_RAWDEV
    printk("(raw_acs)raw_acs can not get major number\n");
#endif				/*  */
    printk("(raw_acs)Init raw_acs failed!\n");
    return result;
  }
#ifdef DEBUG_RAWDEV
  printk("(raw_acs)raw_acs got a major number!\n");
#endif				/*  */
  printk("(raw_acs)Init raw_acs successfully!\n");
  cBuf = kmalloc(iBufSize, GFP_KERNEL);
  memset(cBuf, 0, sizeof(cBuf));
  iBufOff = 0;

#ifdef ADMPLAT
  bMflag = 0;
  flash_init();
#ifdef CONFIG_ADM5120_NOR_CFG
  memcpy((char *) cBoardResv, (char *) PA2VA(AM5120_BOARD_CFG_ADDR),
	 AM5120_BOARD_RESV_SIZE);
#elif CONFIG_ADM5120_NAND_CFG
  nand_read((unsigned char *) cBoardResv
	    (unsigned char *)NAND_ADDR_CONFIG, AM5120_BOARD_RESV_SIZE);
#endif				/*  */
#endif				/*  */
  return 0;
}

void cleanup_module(void)
{
  printk("(raw_acs)clenaup Module\n");
  kfree(cBuf);
  unregister_chrdev(100, "ifx_raw_acs");
}

MODULE_DESCRIPTION("Raw Access Driver");
MODULE_AUTHOR("Eric Tsai<eric.tsai@infineon.com.tw>");
MODULE_LICENSE("GPL");

/* ====================================================================
 * Revision History:
 *
 * $Log: ifx_raw_drv.c,v $
 * Revision 1.1.1.1  2004/12/02 05:25:31  erict
 *
 *
 *
 *
 * ====================================================================
 */
