/* ====================================================================
 * Copyright (c) 2004 - Infeneon-ADMtek Technologies AG.
 *
 * All rights reserved.
 * ====================================================================
 *
 * ====================================================================
 *
 * This document contains proprietary information belonging to Infineon
 * Technologies AG. Passing on and copying of this document, and communication
 * of its contents is not permitted without prior written authorisation.
 *
 * ====================================================================
 */

/* ====================================================================
 *
 * File Name: ifx_raw_test.c
 * Author: Eric Tsai
 * Date:
 *
 * ====================================================================
 *
 * Projects: <ADM5120/Raw_Access>
 * Block: 
 *
 * ====================================================================
 * Contents:
 *
 * ====================================================================
 * References:
 */

#include <stdio.h>
#include <stdlib.h>
#include <stdarg.h>
#include <fcntl.h>
#include <sys/ioctl.h>
#include <string.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>
#include <dirent.h>

#include "ifx_raw_drv.h"
#include "zlib.h"

#define iMaxBufSize	FLASH_BLOCK_SIZE

#define iFnameSize	40
#define iPathSize	40

char8 rcconf[] = "/etc/rc.conf";
char8 udhcpdconf[] = "/etc/udhcpd.conf";
char8 rcfirewall[] = "/etc/rc.d/rc.firewall";
char8 rciptables[] = "/etc/rc.d/rc.iptables";
char8 chapsecrets[] = "/etc/ppp/chap-secrets";
char8 optionspptp[] = "/etc/ppp/options.pptp";
char8 pptp[] = "/etc/ppp/peers/pptp";
char8 ripdconf[] = "/etc/ripd.conf";

char8 icode[] = "RAWACS";	/* It is used to identify the raw access area */

char8 * IFX_RAW_API_DoUnzip(char8 *cBuf, int32 iStartPos, int32 *iBufOff, int32 iSize2beStored);
int32 IFX_RAW_API_StoreConf(T_IFX_RAW_INFO *tRawInfo, int fd, char8 *cFname, char8 *cRefDir);
int32 IFX_RAW_API_ReadBackConf(T_IFX_RAW_INFO *tRawInfo, int fd, char8 *cFname, char8 *cRefDir);
int32 IFX_RAW_API_BackupFlash(int fd, char8 * filename);
int32 IFX_RAW_API_RestoreFlash(int fd, char8 * filename);
char8 IFX_RAW_DBG(char8 * cStr, ...);

/* Static variables */
int16 ADMType, ZIP, FlashType;
char8 *cBuf = NULL;

void usage(void)
{
  printf("raw_access v1.1.3(20050125) by Infineon-ADMtek\n");
  if (ADMType == ADM5120)
    printf(" Platform: ADM5120\n");
  else if (ADMType == ADM5120P)
    printf(" Platform: ADM5120P\n");
  else 
    printf(" Platform: \n");

  if (FlashType == MX29LV160)
    printf(" Flash type: MX29LV160\n");
  else if (FlashType == MX29LV320)
    printf(" Flash type: MX29LV320\n");
  else 
    printf(" Flash type: \n");

  if (ZIP)
    printf(" ZIP included\n");
  else
    printf(" No ZIP\n");

  printf("\nUsage:\n");
  printf
      ("	./rawaccess [-s [filename]]|[-l [filename]]|[-d [filename]]|\n");
  printf
      ("		[-b filename]|[-t filename]|[-a filename...]|[-r filename...]\n");
  printf("	-s store default files to flash or a backup file\n");
  printf("	-l list the content of flash or a backup file\n");
  printf
      ("	-d read data from flash or from a backup file if flash is empty\n");
  printf("	-b backup flash data to a file\n");
  printf("	-t restore a backup file to flash\n");
  printf("	-a append files to be stored\n");
  printf("	-r reset files to be stored\n");
  printf("	-e erase the raw access area of flash\n\n");
  printf("	Examples:\n");
  printf("	#./rawaccess -s\n");
  printf("	It backs up files located within /nv to flash.\n\n");
  printf("	#./rawaccess -s ./conf.bak\n");
  printf("	It backs up files located within /nv to conf.bak.\n\n");
  printf("	#./rawaccess -d\n");
  printf("	It restores the data of flash to be files.\n\n");
  printf("	#./rawaccess -d ./conf.bak\n");
  printf("	It restores the data of flash to be files. If there is no data in\n");
  printf("	the flash, it restores the conf.bak.\n\n");
}

int32 IFX_RAW_API_AddDefault(T_IFX_RAW_INFO * tRawInfo, char8 *cRefDir)
{
  DIR *pDir;
  struct dirent *dp;
  struct stat file_info;
  int32 iSize2beStored = 0;
  int fd = 0;
  char8 cFilename[iFnameSize], cPath[iPathSize], cRecname[iFnameSize];

  memcpy(tRawInfo->cICode, icode, sizeof(icode));
  memset(cPath, 0, sizeof(cPath));
  if(cRefDir)
    strcpy(cPath, cRefDir);
  strcat(cPath, "/nv");
  pDir = opendir(cPath);
  while ((dp = readdir(pDir)) != NULL) {
    if ((strcmp(dp->d_name, ".") != 0) && (strcmp(dp->d_name, "..") != 0)) {
      if(tRawInfo->iNfile >= MAX_FILES) break;
      memset(cFilename, 0, sizeof(cFilename));
      memset(cRecname, 0, sizeof(cRecname));
      strcpy(cFilename, cPath);
      strcat(cFilename, "/");
      strcat(cFilename, dp->d_name);
      strcpy(cRecname, "/nv/");
      strcat(cRecname, dp->d_name);
      if ((fd = open(cFilename, O_RDONLY)) != 0) {
        fstat(fd, &file_info);
        if(file_info.st_size > 0){
          strcpy(tRawInfo->cFilename[tRawInfo->iNfile], cRecname);
          tRawInfo->iSize[tRawInfo->iNfile] = file_info.st_size;
          iSize2beStored += file_info.st_size;
          printf("%s: %d bytes\n", cFilename, file_info.st_size);
          tRawInfo->iNfile++;
        }
	close(fd);
      }
    }
  }
  printf("Files: %d\n", tRawInfo->iNfile);
  closedir(pDir);
  return iSize2beStored;
}

int32 IFX_RAW_API_AppendFile(T_IFX_RAW_INFO * tRawInfo, char8 *cFilename)
{
  struct stat file_info;
  int32 iSize = 0;
  int fd = 0;

  if(tRawInfo->iNfile >= MAX_FILES) return 0;
  if ((fd = open(cFilename, O_RDONLY)) != 0) {
    fstat(fd, &file_info);
    if(file_info.st_size > 0){
      strcpy(tRawInfo->cFilename[tRawInfo->iNfile], cFilename);
      tRawInfo->iSize[tRawInfo->iNfile] = file_info.st_size;
      iSize = file_info.st_size;
      printf("%s: %d bytes\n", cFilename, file_info.st_size);
      tRawInfo->iNfile++;
    }
    close(fd);
  }
  return iSize;
}

int32 IFX_RAW_API_GetUncompSize(T_IFX_RAW_INFO * tRawInfo)
{
  char8 i;
  int32 iSize2beStored = 0;

  for(i=0; i<tRawInfo->iNfile; i++) {
    iSize2beStored += tRawInfo->iSize[i];
  }
  return iSize2beStored;
}

int32 IFX_RAW_API_ViewContent(T_IFX_RAW_INFO *tRawInfo, int fd, char8 *cFname)
{
  int i;
  FILE *fptr = NULL;
  int32 iRet, iBufOff;
  int32 iSize2beStored = 0;
  char8 *cUncompr = NULL;

  /* If there is a file name, view the content of the file */
  if (cFname) {
    if ((fptr = fopen(cFname, "r")) != NULL) {
      memset(cBuf, 0, iMaxBufSize);
      iBufOff = fread(cBuf, sizeof(char8), iMaxBufSize, fptr);
      if (iBufOff > 0) {
	printf("Read %d bytes from %s\n", iBufOff, cFname);
	memcpy(tRawInfo, cBuf, sizeof(T_IFX_RAW_INFO));
	printf("tRawInfo in backup: %d files\n", tRawInfo->iNfile);
	if ((tRawInfo->iNfile <= 0) || (tRawInfo->iCompSize <= 0)
	    || (strcmp(tRawInfo->cICode, icode) != 0)) {
	  printf("Wrong data!\n");
	  fclose(fptr);
	  return -1;
	}
#ifndef NOZIP
        iSize2beStored = IFX_RAW_API_GetUncompSize(tRawInfo);
	printf("%d to be stored\n", iSize2beStored);
	iBufOff -= sizeof(T_IFX_RAW_INFO);
	cUncompr = IFX_RAW_API_DoUnzip(cBuf, sizeof(T_IFX_RAW_INFO), &iBufOff, iSize2beStored);
        if (cUncompr){
          free(cBuf);
          cBuf = cUncompr;
        }else{
	  printf("Unzip failed!\n");
	  fclose(fptr);
	  return -1;
        }
#endif
      } else {
	fclose(fptr);
	return -1;
      }
      fclose(fptr);
    } else {
      printf("Open %s failed!\n", cFname);
      return -1;
    }
  } else {
    /* View the content of flash */
    memset(cBuf, 0, iMaxBufSize);
    if(fd)
      iBufOff = read(fd, cBuf, iMaxBufSize);
    else
      iBufOff = 0;
    if (iBufOff <= 0) {
      IFX_RAW_DBG("Read no data from flash\n");
      return -1;
    } else {
      printf("Read %d bytes from flash\n", iBufOff);
      memcpy(tRawInfo, cBuf, sizeof(T_IFX_RAW_INFO));
      printf("tRawInfo in flash: %d files\n", tRawInfo->iNfile);
      if (tRawInfo->iNfile <= 0) {
	return -1;
      }
      if (strcmp(tRawInfo->cICode, icode) != 0) {
	printf("Wrong data!\n");
	return -1;
      }
#ifndef NOZIP
      iSize2beStored = IFX_RAW_API_GetUncompSize(tRawInfo);
      iBufOff -= sizeof(T_IFX_RAW_INFO);
      cUncompr = IFX_RAW_API_DoUnzip(cBuf, sizeof(T_IFX_RAW_INFO), &iBufOff, iSize2beStored);
      if (cUncompr){
        free(cBuf);
        cBuf = cUncompr;
      }else{
	printf("Unzip failed!\n");
	return -1;
      }
#endif
    }
  }

  for (i = 0; i < tRawInfo->iNfile; i++) {
    printf("%s: %d bytes\n", tRawInfo->cFilename[i], tRawInfo->iSize[i]);
  }
  return 0;
}

#ifndef NOZIP
int32 IFX_RAW_API_DoZip(char8 * cBuf, int32 * iBufOff)
{
  char8 *cCompr = NULL;
  int err;
  unsigned long ComprLen;

  if (*iBufOff <= 0)
    return -1;
  cCompr = (char8 *) malloc(*iBufOff);
  if (!cCompr)
    return -1;
  printf("%d bytes...Compressed to...", *iBufOff);
  err = compress((Byte *) cCompr, &ComprLen, (Byte *) cBuf, *iBufOff);
  if (err != Z_OK) {
    printf("\nCompress error %d\n", err);
    free(cCompr);
    return -1;
  }
  memcpy(cBuf, cCompr, ComprLen);
  *iBufOff = ComprLen;
  printf("%d bytes\n", ComprLen);
  free(cCompr);
  return ComprLen;
}

char8 * IFX_RAW_API_DoUnzip(char8 *cBuf, int32 iStartPos, int32 *iBufOff, int32 iSize2beStored)
{
  char8 *cUncompr = NULL;
  char8 *cTmp = NULL;
  int err;
  unsigned long UncomprLen;

  if (*iBufOff <= 0)
    return NULL;
  cUncompr = (char8 *) malloc(iSize2beStored + iStartPos);
  if (!cUncompr){
    IFX_RAW_DBG("Mem alloc for uncompress failed!\n");
    return NULL;
  }else{
    IFX_RAW_DBG("Alloc %d bytes for uncompress.\n", iSize2beStored + iStartPos);
  }
  printf("%d bytes...Uncompressed to...", *iBufOff);
  err =
      uncompress((Byte *) cUncompr + iStartPos, &UncomprLen, (Byte *) cBuf + iStartPos,
		 (unsigned long) *iBufOff);
  if (err != Z_OK) {
    printf("\nUncompress error %d\n", err);
    free(cUncompr);
    return NULL;
  }
  memcpy(cUncompr, cBuf, iStartPos);
  *iBufOff = UncomprLen;
  printf("%d bytes\n", UncomprLen);
  return cUncompr;
}
#endif

int main(int argc, char *argv[])
{
  int fd = 0;
  int32 iBufOff;
  T_IFX_RAW_INFO tRawInfo;
  char8 cFilename[iFnameSize], cRefDir[iFnameSize];
  int32 iSize2beStored;

  ADMType = -1;  
  FlashType = -1;

#if defined(ADMTYPE) 
#if(ADMTYPE == ADM5120P)
  ADMType = ADM5120P;  
#endif
#if(ADMTYPE == ADM5120)
  ADMType = ADM5120;  
#endif

#if defined(FLASHTYPE) 
#if(FLASHTYPE == MX29LV160)
  FlashType = MX29LV160;
#endif
#if(FLASHTYPE == MX29LV320)
  FlashType = MX29LV320;
#endif
#endif
#endif

#ifndef NOZIP
  ZIP = 1;
#else
  ZIP = 0;
#endif

  if (argc < 2) {
    usage();
    return 0;
  }
#ifdef RAWACS
  fd = open("/dev/rawdev", O_RDWR, 0);
  if (fd <= 0) {
    printf("Open rawdev error!\n");
    return 0;
  }
  printf("rawdev opened!\n");
#endif

  cBuf = (char8 *) malloc(iMaxBufSize);
  memset(cBuf, 0, iMaxBufSize);
  memset(&tRawInfo, 0, sizeof(T_IFX_RAW_INFO));
  iBufOff = 0;
  if (!strcmp(argv[1], "-s")) {
    if(argc == 4){
      memset(cRefDir, 0, sizeof(cRefDir));
      strcpy(cRefDir, argv[3]);
      iSize2beStored = IFX_RAW_API_AddDefault(&tRawInfo, cRefDir);
    }else{
      iSize2beStored = IFX_RAW_API_AddDefault(&tRawInfo, NULL);
    }
    iSize2beStored += sizeof(T_IFX_RAW_INFO);
    if(iSize2beStored > iMaxBufSize){
      cBuf = realloc(cBuf, iSize2beStored + 10);
      if(cBuf){
        memset(cBuf, 0, iSize2beStored);
        IFX_RAW_DBG("realloc %d bytes successfully\n", iSize2beStored + 10);
      }else{
        IFX_RAW_DBG("realloc failed\n");
        goto Bye;
      }
    }
    ioctl(fd, RAWDRV_IOCRSRAWDEV, NULL);
    if (argc == 3) {
      memset(cFilename, 0, sizeof(cFilename));
      strcpy(cFilename, argv[2]);
      if (IFX_RAW_API_StoreConf(&tRawInfo, fd, cFilename, NULL) >= 0) {
	printf("Store /nv/* to %s completely!\n", cFilename);
      } else {
	printf("Store /nv/* to %s failed!\n", cFilename);
      }
    }else if(argc == 4){
      memset(cFilename, 0, sizeof(cFilename));
      strcpy(cFilename, argv[2]);
      if (IFX_RAW_API_StoreConf(&tRawInfo, fd, cFilename, cRefDir) >= 0) {
	printf("Store /nv/* to %s completely!\n", cFilename);
      } else {
	printf("Store /nv/* to %s failed!\n", cFilename);
      }
    } else {
      if (IFX_RAW_API_StoreConf(&tRawInfo, fd, NULL, NULL) >= 0) {
	printf("Store /nv/* to flash completely!\n");
      } else {
	printf("Store /nv/* to flash failed!\n");
      }
    }
  } else if (!strcmp(argv[1], "-d")) {
    if (argc == 3) {
      memset(cFilename, 0, sizeof(cFilename));
      strcpy(cFilename, argv[2]);
      if (IFX_RAW_API_ReadBackConf(&tRawInfo, fd, cFilename, NULL) >= 0) {
	printf("Read data completely!\n");
      } else {
	printf("Read data failed!\n");
      }
    }else if(argc == 4){
      memset(cFilename, 0, sizeof(cFilename));
      strcpy(cFilename, argv[2]);
      memset(cRefDir, 0, sizeof(cRefDir));
      strcpy(cRefDir, argv[3]);
      if (IFX_RAW_API_ReadBackConf(&tRawInfo, fd, cFilename, cRefDir) >= 0) {
	printf("Read data completely!\n");
      } else {
	printf("Read data failed!\n");
      }
    } else {
      if (IFX_RAW_API_ReadBackConf(&tRawInfo, fd, NULL, NULL) >= 0) {
	printf("Read data completely!\n");
      } else {
	printf("Read data failed!\n");
      }
    }
  } else if (!strcmp(argv[1], "-b")) {
    if (argc == 3) {
      memset(cFilename, 0, sizeof(cFilename));
      strcpy(cFilename, argv[2]);
      IFX_RAW_API_BackupFlash(fd, cFilename);
    }
  } else if (!strcmp(argv[1], "-t")) {
    if (argc == 3) {
      memset(cFilename, 0, sizeof(cFilename));
      strcpy(cFilename, argv[2]);
      IFX_RAW_API_RestoreFlash(fd, cFilename);
    }
  } else if (!strcmp(argv[1], "-a")) {
    int i;
    if (argc < 3) {
      usage();
      goto Bye;
    }
    iSize2beStored = IFX_RAW_API_AddDefault(&tRawInfo, NULL);
    iSize2beStored += sizeof(T_IFX_RAW_INFO);
    for (i = 2; i < argc; i++) {
      printf("Append file: %s\n", argv[i]);
      iSize2beStored += IFX_RAW_API_AppendFile(&tRawInfo, argv[i]);
    }
    if(iSize2beStored > iMaxBufSize){
      cBuf = realloc(cBuf, iSize2beStored);
      memset(cBuf, 0, iSize2beStored);
    }
    ioctl(fd, RAWDRV_IOCRSRAWDEV, NULL);
    if (IFX_RAW_API_StoreConf(&tRawInfo, fd, NULL, NULL) >= 0) {
      printf("Store /nv/* to flash completely!\n");
    } else
      printf("Store /nv/* to flash failed!\n");
  } else if (!strcmp(argv[1], "-r")) {
    int i;
    if (argc < 3) {
      usage();
      goto Bye;
    }
    memcpy(tRawInfo.cICode, icode, sizeof(icode));
    iSize2beStored += sizeof(T_IFX_RAW_INFO);
    for (i = 2; i < argc; i++) {
      printf("Store file: %s\n", argv[i]);
      iSize2beStored += IFX_RAW_API_AppendFile(&tRawInfo, argv[i]);
    }
    ioctl(fd, RAWDRV_IOCRSRAWDEV, NULL);
    if (IFX_RAW_API_StoreConf(&tRawInfo, fd, NULL, NULL) >= 0) {
      printf("Store /nv/* to flash completely!\n");
    } else
      printf("Store /nv/* to flash failed!\n");
  } else if (!strcmp(argv[1], "-e")) {
    ioctl(fd, RAWDRV_IOCRSFLASH, NULL);
  } else if (!strcmp(argv[1], "-l")) {
    if (argc == 3) {
      memset(cFilename, 0, sizeof(cFilename));
      strcpy(cFilename, argv[2]);
      IFX_RAW_API_ViewContent(&tRawInfo, fd, cFilename);
    } else {
      IFX_RAW_API_ViewContent(&tRawInfo, fd, NULL);
    }
  } else if (!strcmp(argv[1], "-x")) {
    ioctl(fd, RAWDRV_IOCTEST, NULL);
  } else if (!strcmp(argv[1], "-f")) {
    uint32 uiaddr;
    sscanf(argv[2], "%x", &uiaddr);
    printf("Set raw access addr: %x\n", uiaddr);
    ioctl(fd, RAWDRV_IOCSRAWACSADDR, &uiaddr);
  } else {
    usage();
    goto Bye;
  }

Bye:
#ifdef RAWACS
  close(fd);
#endif
  if (cBuf)
    free(cBuf);
  return 0;
}

int32 IFX_RAW_API_StoreConf(T_IFX_RAW_INFO * tRawInfo, int fd, char8 * cFname, char8 *cRefDir)
{
  int i;
  FILE *fptr = NULL;
  int32 iRet, iBufOff;
  char8 *cInfo = NULL;
  char8 cAbsDir[iFnameSize + iPathSize];

  if (tRawInfo->iNfile <= 0) {
    printf("No file to be stored!\n");
    return -1;
  }
  iBufOff = 0;
  memset(cBuf, 0, sizeof(cBuf));
  cInfo = cBuf + sizeof(T_IFX_RAW_INFO);
  for (i = 0; i < tRawInfo->iNfile; i++) {
    memset(cAbsDir, 0, sizeof(cAbsDir));
    if(cRefDir){
      strcpy(cAbsDir, cRefDir);
      strcat(cAbsDir, tRawInfo->cFilename[i]);
    }else{
      strcpy(cAbsDir, tRawInfo->cFilename[i]);
    }
    if ((fptr = fopen(cAbsDir, "r")) != NULL) {
      iRet = fread(cInfo + iBufOff, sizeof(char8), tRawInfo->iSize[i], fptr);
      if (iRet > 0) {
	printf("Got %d bytes from %s\n", iRet, cAbsDir);
	tRawInfo->iPosOffset[i] = iBufOff;
	iBufOff += iRet;
	tRawInfo->iSize[i] = iRet;
      }else if(iRet == 0) {
	printf("File %s cannot be read\n", cAbsDir);
      }
      fclose(fptr);
    } else {
      printf("Open %s failed!\n", cAbsDir);
    }
  }
#ifndef NOZIP
  iRet = IFX_RAW_API_DoZip(cBuf + sizeof(T_IFX_RAW_INFO), &iBufOff);
  if (iRet < 0) {
    printf("Zip failed!\n");
    return -1;
  }else{
    tRawInfo->iCompSize = iRet;
  }
#endif
  memcpy(cBuf, tRawInfo, sizeof(T_IFX_RAW_INFO));
  printf("Copy %d bytes to buffer\n", sizeof(T_IFX_RAW_INFO));
  iBufOff += sizeof(T_IFX_RAW_INFO);
  if(iBufOff > (iMaxBufSize - AM5120_BOARD_RESV_SIZE)) {
    printf("Too large to write!\n");
    return -1;
  }

  /* If there is a file name, then store to the file. Otherwise, store to 
     the flash */
  if (cFname) {
    if ((fptr = fopen(cFname, "w")) != NULL) {
      iRet = fwrite(cBuf, sizeof(char8), iBufOff, fptr);
      if (iRet != iBufOff) {
	printf("Write to file %s error\n", cFname);
	fclose(fptr);
	return -1;
      } else {
	printf("Write %d bytes to %s!\n", iRet, cFname);
	fclose(fptr);
      }
    } else {
      printf("Open %s failed!\n", cFname);
      return -1;
    }
  } else {
    if (!fd) {
      printf("rawdev error!\n");
      return -1;
    }
    iRet = write(fd, cBuf, iBufOff);
    if (iRet != iBufOff) {
      printf("Write to rawdev error!\n");
      return -1;
    }
    printf("Write %d bytes to rawdev!\n", iRet);
  }
  return 0;
}

int32 IFX_RAW_API_ReadBackConf(T_IFX_RAW_INFO *tRawInfo, int fd, char8 *cFname, char8 *cRefDir)
{
  int i;
  FILE *fptr = NULL;
  int32 iRet, iBufOff;
  char8 *cInfo = NULL;
  char8 *cUncompr = NULL;
  char8 cAbsDir[iFnameSize + iPathSize];
  int32 iSize2beStored;
  mode_t st_mode = S_IRWXU | S_IRGRP | S_IXGRP | S_IROTH | S_IXOTH;

  memset(cBuf, 0, iMaxBufSize);
  if (fd <= 0)
    iBufOff = 0;
  else
    iBufOff = read(fd, cBuf, iMaxBufSize);
  printf("Read %d bytes from flash\n", iBufOff);
  if (iBufOff <= 0) {
    printf("Get flash data error!\n");
    memset(tRawInfo, 0, sizeof(T_IFX_RAW_INFO));
  } else {
    /* Check flash data first */
    memcpy(tRawInfo, cBuf, sizeof(T_IFX_RAW_INFO));
    if ((tRawInfo->iNfile <= 0) || (strcmp(tRawInfo->cICode, icode) != 0)) {
      printf("Flash data error!\n");
      memset(tRawInfo, 0, sizeof(T_IFX_RAW_INFO));
    }else {
#ifndef NOZIP
      iSize2beStored = IFX_RAW_API_GetUncompSize(tRawInfo);
      iBufOff -= sizeof(T_IFX_RAW_INFO);
      cUncompr = IFX_RAW_API_DoUnzip(cBuf, sizeof(T_IFX_RAW_INFO), &iBufOff, iSize2beStored);
      if (cUncompr){
        free(cBuf);
        cBuf = cUncompr;
      }else{
        printf("Unzip failed, flash data error!\n");
        memset(tRawInfo, 0, sizeof(T_IFX_RAW_INFO));
      }
#endif
    }
  }

  if ((tRawInfo->iNfile <= 0) || (strcmp(tRawInfo->cICode, icode) != 0)) {
    printf("Wrong data in flash\n");
    if (!cFname) {
      return -1;
    } else {
      if (fd) { 
        /* Restore config from cFname to flash */
	if (IFX_RAW_API_RestoreFlash(fd, cFname) < 0) {
	  return -1;
	}
      }
      if ((fptr = fopen(cFname, "r")) != NULL) {
	memset(cBuf, 0, iMaxBufSize);
	iBufOff = fread(cBuf, sizeof(char8), iMaxBufSize, fptr);
	printf("Read %d bytes from %s\n", iBufOff, cFname);
        if(iBufOff <= 0) {
	  printf("Restore data to /nv failed!\n");
          return -1;
        }
        memcpy(tRawInfo, cBuf, sizeof(T_IFX_RAW_INFO));
        if ((tRawInfo->iNfile <= 0)
	    || (strcmp(tRawInfo->cICode, icode) != 0)) {
	  printf("Wrong data in %s\n", cFname);
	  return -1;
        }
        printf("tRawInfo in backup: %d files\n", tRawInfo->iNfile);
#ifndef NOZIP
        iSize2beStored = IFX_RAW_API_GetUncompSize(tRawInfo);
	iBufOff -= sizeof(T_IFX_RAW_INFO);
	cUncompr = IFX_RAW_API_DoUnzip(cBuf, sizeof(T_IFX_RAW_INFO), &iBufOff, iSize2beStored);
	if (cUncompr) {
          free(cBuf);
          cBuf = cUncompr;
        }else{
	  printf("Unzip failed!\n");
	  fclose(fptr);
	  return -1;
	}
#endif
	fclose(fptr);
      } else {
	printf("Restore data to /nv failed!\n");
	return -1;
      }
    }
  }

  cInfo = cBuf + sizeof(T_IFX_RAW_INFO);
  for (i = 0; i < tRawInfo->iNfile; i++) {
    memset(cAbsDir, 0, sizeof(cAbsDir));
    if(cRefDir){
      strcpy(cAbsDir, cRefDir);
      strcat(cAbsDir, tRawInfo->cFilename[i]);
    }else{
      strcpy(cAbsDir, tRawInfo->cFilename[i]);
    }
    if ((fptr = fopen(cAbsDir, "w")) != NULL) {
      iRet =
	  fwrite(cInfo + tRawInfo->iPosOffset[i], sizeof(char8),
		 tRawInfo->iSize[i], fptr);
      if (iRet > 0) {
	printf("Write %d bytes to %s\n", iRet, cAbsDir);
      }
      fclose(fptr);
      chmod(cAbsDir, st_mode);
    } else {
      printf("Open %s failed\n", cAbsDir);
    }
  }

  return 0;
}

/* Backup flash data to a file */
int32 IFX_RAW_API_BackupFlash(int fd, char8 * filename)
{
  FILE *fptr = NULL;
  int32 iRet = 0;
  int32 iBufOff = 0;
  T_IFX_RAW_INFO tRawInfo;

  if(!fd){
    printf("No rawdev!\n");
    return -1;
  }
  if ((fptr = fopen(filename, "w")) != NULL) {
    memset(cBuf, 0, iMaxBufSize);
    iBufOff = read(fd, cBuf, iMaxBufSize);
    printf("Read %d bytes from flash\n", iBufOff);
    if (iBufOff <= 0) {
      fclose(fptr);
      return -1;
    }

    iRet = fwrite(cBuf, sizeof(char8), iBufOff, fptr);
    if (iRet > 0) {
      printf("Backup %d bytes flash data to %s\n", iRet, filename);
    }
    fclose(fptr);
    return iRet;
  } else {
    printf("Open file failed!\n");
    return -1;
  }
}

/* Restore config from backup file to flash */
int32 IFX_RAW_API_RestoreFlash(int fd, char8 * filename)
{
  FILE *fptr = NULL;
  int32 iRet = 0;
  int32 iBufOff = 0;

  if ((fptr = fopen(filename, "r")) != NULL) {
    memset(cBuf, 0, iMaxBufSize);
    iBufOff = fread(cBuf, sizeof(char8), iMaxBufSize, fptr);
    if (iBufOff > 0) {
      printf("Read %d from backup file %s\n", iBufOff, filename);
    } else {
      printf("No data should be restored!\n");
      fclose(fptr);
      return -1;
    }
    if (!fd) {
      printf("rawdev error!\n");
      fclose(fptr);
      return -1;
    }
    iRet = write(fd, cBuf, iBufOff);
    if (iRet != iBufOff) {
      printf("Write to rawdev error!\n");
      fclose(fptr);
      return -1;
    }
    printf("Restore data to rawdev\n");
    fclose(fptr);
    return iRet;
  } else {
    printf("Open file failed!\n");
    return -1;
  }
}

char8
IFX_RAW_DBG(char8 * cStr, ...)
{
#ifdef DEBUG
   va_list vaList;

   va_start(vaList, cStr);
   vprintf(cStr, vaList);
   va_end(vaList);
#else
#endif
   return 1;
}
/* ====================================================================
 * Revision History:
 *
 * $Log: ifx_raw_test.c,v $
 * Revision 1.3  2004/12/17 13:28:54  erict
 * Fix the buffer re-allocation problem.
 *
 * Revision 1.2  2004/12/03 03:50:55  erict
 * Print out compiling options
 *
 * Revision 1.1.1.1  2004/12/02 05:25:31  erict
 *
 *
 *
 *
 * ====================================================================
 */
