/************************************************************************
 *
 *    Copyright (c) 2005
 *    Infineon Technologies AG
 *    St. Martin Strasse 53; 81669 Muenchen; Germany
 *
 *    THE DELIVERY OF THIS SOFTWARE AS WELL AS THE HEREBY GRANTED
 *    NON-EXCLUSIVE, WORLDWIDE LICENSE TO USE, COPY, MODIFY, DISTRIBUTE
 *    AND SUBLICENSE THIS SOFTWARE IS FREE OF CHARGE.
 *    
 *    THE LICENSED SOFTWARE IS PROVIDED "AS IS" AND INFINEON EXPRESSLY
 *    DISCLAIMS ALL REPRESENTATIONS AND WARRANTIES, WHETHER EXPRESS OR
 *    IMPLIED, INCLUDING WITHOUT LIMI?TATION, WAR?RANTIES OR REPRESENTATIONS
 *    OF WORKMANSHIP, MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE,
 *    DURABILITY, THAT THE OPERATING OF THE LICENSED SOFTWARE WILL BE ERROR
 *    FREE OR FREE OF NY THIRD PARTY CALIMS, INCLUDING WITHOUT LIMITATION
 *    CLAIMS OF THIRD PARTY INTELLECTUAL PROPERTY INFRINGEMENT.
 *    
 *    EXCEPT FOR ANY LIABILITY DUE TO WILFUL ACTS OR GROSS NEGLIGENCE AND
 *    EXCEPT FOR ANY PERSONAL INJURY INFINEON SHALL IN NO EVENT BE LIABLE
 *    FOR ANY CLAIM OR DAMAGES OF ANY KIND, WHETHER IN AN ACTION OF CONTRACT,
 *    TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
 *    SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 *
 *    --------------------------------------------------------------------
 *    
 *        Project : 
 *        Block   : 
 *        Creator : 
 *        File    : 
 *        Abstract: 
 *        Date    : 
 *    
 *    Modification History:
 *           By              Date     Ver.   Modification Description
 *           --------------- -------- -----  -----------------------------
 *
 ************************************************************************/
#include	<string.h>
#include	<netdb.h>
#include	"ifx_emf.h"
#include	<signal.h>
#include	<unistd.h> 
#include	<sys/types.h>
#include	<sys/wait.h>
#include 	<sys/socket.h>
#include 	<sys/ioctl.h>
#include 	<arpa/inet.h>
#include 	<netinet/in.h>
#include 	<net/if.h>
#include 	<net/if_arp.h>
#include	<ifx_common.h>
#include	"./ifx_src_cgi.h"
#include 	"ifx_httpd_method.h"

#include 	"ifx_common_defs.h"

#if defined(ATA) || defined(IIP)
#include 	"ifx_iip_fifos.h"
#include 	"ifx_cm.h"
#endif

#include	"ifx_web_common.h"

#define IFX_WEB_MAX_PB_ENTRIES 50
#define IFX_WEB_IPADDRESS	  0
#define IFX_WEB_NETMASK		  1

x_IFX_CM_AppAddressBookEntry xPBEntry[IFX_WEB_MAX_PB_ENTRIES];

#ifdef DMALLOC
#include "dmalloc.h"
#endif

//**************************** Global variables *********************************
extern x_IFX_CM_Msg vxCmMsg;
extern x_IFX_CM_AppAddressBookEntry xPBEntry[50];
extern int16 NoOfPbEntries;
#ifdef SRTP
extern x_IFX_CM_SrtpKeyCfg xCrypto[5];
#endif /*SRTP*/
extern int g_nCurrentUserLevel;//0:admin,1:common user,2:general user

//***************************** Common API **************************************
extern char_t *websGetHost();
extern const int websGetPortNums2(char_t* pData, ST_NAT_PORT_MAP_LIST *pstPortList);

//////////////////////////////////////////////////////////////////////////////////
//websGetIfaceInfo(...)
//	Variable Name	==>	File Name		;	Pipe Command
//-------------------------------------------------------------------------------
//	pInterface		==>	Interface Name				ex. eth0/adm0/adm1 255.255.0.0
//	nType			==>	IP Address or Netmask 		0:  ip , 1: mask
//	pRetValue		==>	Return IP
int websGetIfaceInfo (const char* pInterface, int nType, char_t* pRetValue) 
{
	struct  in_addr	 self, mask;
	int     			 skfd	= 0;                // socket fd for receive packets
	struct  ifreq   	 ifr;            			// ifr structure	
	struct sockaddr_in *sin_ptr;

	memset(&ifr, 0x0, sizeof(ifr));
	memset(&self, 0x0, sizeof(self));
	memset(&mask, 0x0, sizeof(mask));
	
	if ((skfd = socket(AF_INET, SOCK_DGRAM, 0)) == -1)
	{
	    perror( "open socket error");
	    exit(-1);
	}

	strcpy(ifr.ifr_name, pInterface);
	ifr.ifr_addr.sa_family = AF_INET;		// Default: AF_INET
	ifx_debug_printf(">>>> Currnet interface name %s \n", pInterface);

	if (nType == IFX_WEB_IPADDRESS)		// Wanna IP address
	{
		// Get IP Address of indicate interface
		if (ioctl(skfd, SIOCGIFADDR, &ifr) < 0) 
		{
			perror("ioctl SIOCGIFADDR error");
			close(skfd);
			return 0;
		}
		sin_ptr = (struct sockaddr_in *)&ifr.ifr_addr;
		self  = sin_ptr->sin_addr;    	
		strcpy(pRetValue, inet_ntoa(self));
	}
	else if (nType == IFX_WEB_NETMASK)
	{
		// Get network mask of indicate interface
		if (ioctl(skfd, SIOCGIFNETMASK, &ifr) < 0) 
		{
			perror("ioctl SIOCGIFNETMASK error");
			close(skfd);
			return 0;
		}
		sin_ptr = (struct sockaddr_in *)&ifr.ifr_addr;
		mask  = sin_ptr->sin_addr;
		strcpy(pRetValue, inet_ntoa(mask));    	
	}

	close(skfd);
	return 1;
}

void websGetGateway(int nReqIdx, bool_t bBreak, char_t* pRetValue)
{
	char_t	sGateway[MAX_DATA_LEN];
	char_t	getCommand[MAX_FILELINE_LEN];	
	char_t	*ip;
	int		i = 0;
	int 	nIndex = 0;		

	memset(sGateway, 0x00, MAX_DATA_LEN);
	gstrcpy(pRetValue, "");		
	
	gsprintf(getCommand, T("netstat -nre | grep ^0.0.0.0 | awk '{print $2}'"));
	if (ifx_GetCfgData((char_t*)getCommand, NULL, "1", getCommand) == 0)
	{
		return;
	}	
	
	if (bBreak == FALSE && nReqIdx == 4)	// Wanna full IP address
	{
		gstrcpy(pRetValue, getCommand);	
		return;
	}

	for (i = 1; i <= nReqIdx; i++)
	{
		gstrcpy(sGateway, getCommand);
		ip = strtok(sGateway, ".");	
		nIndex = 1;		
		while (ip != NULL && nIndex < i )
		{
			nIndex++;
			ip = strtok(NULL, ".");	
		}

		if (bBreak == TRUE && i == nReqIdx)
		{	
			gstrcpy(pRetValue, (const char_t*)ip);	
			return;
		}
		else if(bBreak == FALSE)
		{
			gstrcat(pRetValue, (const char_t*)ip);
			gstrcat(pRetValue, ".");
		}
	}
}

//////////////////////////////////////////////////////////////////////////////////
//websGetNetmask(...)
//	ex. ==>	websGetNetmask(ADM_LAN_IF, 4, FALSE, sLanIP)
//	Variable Name	==>	File Name		;	Pipe Command
//-------------------------------------------------------------------------------
//	pInterface		==>	Interface Name				ex. eth0/adm0/adm1 255.255.0.0
//	nReqIdx			==>	Interface's address index	ex. nIdx = 2 ====> 255.255
//	bBreak			==>	TRUE is to obtain one(255). FALSE is to obtain as above.(255.255)
//	pRetValue		==>	Return string.
//
//	Return Value :	N/A
//
void websGetNetmask(const char_t* pInterface, int nReqIdx, bool_t bBreak, char_t* pRetValue)
{
	char_t	getCommand[MAX_FILELINE_LEN];
	char_t	sNetmask[MAX_DATA_LEN];
	char_t	*ip;
	int		i = 0;
	int 	nIndex = 0;
	
	memset(getCommand, 0x00, MAX_FILELINE_LEN);
	memset(sNetmask, 0x00, MAX_DATA_LEN);
	gstrcpy(pRetValue, "");

	//gsprintf(getCommand, T("ifconfig %s | grep %s | cut -f4 -d:"),
	//					 pInterface, "\"Mask:\"");
	//if (ifx_GetCfgData((char_t*)getCommand, NULL, "1", getCommand) == 0)
	//{
	//	return;
	//}
	if (websGetIfaceInfo(pInterface, IFX_WEB_NETMASK, getCommand) == 0)
		return;

	if (bBreak == FALSE && nReqIdx == 4)	// Wanna full IP address
	{
		gstrcpy(pRetValue, getCommand);
		return;
	}

	for (i = 1; i <= nReqIdx; i++)
	{
		gstrcpy(sNetmask, getCommand);
		ip = strtok(sNetmask, ".");	
		nIndex = 1;		
		while (ip != NULL && nIndex < i )
		{
			nIndex++;
			ip = strtok(NULL, ".");	
		}

		if (bBreak == TRUE && i == nReqIdx)
		{	
			gstrcpy(pRetValue, (const char_t*)ip);			
			return;
		}
		else if(bBreak == FALSE)
		{
			gstrcat(pRetValue, (const char_t*)ip);
			gstrcat(pRetValue, ".");
		}
	}
}

//////////////////////////////////////////////////////////////////////////////////
//	websGetWanMode()
//
//	Return Value :	IP_BOOT_DHCPC, IP_BOOT_FIXED, IP_BOOT_PPPOE, IP_BOOT_PPTP
//
int websGetWanMode()
{
	char_t  sValue[MAX_DATA_LEN];
	int		nReturn = 0;
	
	if (ifx_GetCfgData(FILE_RC_CONF, TAG_WAN_MAIN, "ipoption_adm1", sValue) == 1)
	{
		if (!gstrcmp(sValue, T("DHCPC")))
			nReturn = IP_BOOT_DHCPC;
		else if (!gstrcmp(sValue, T("FIXED")))
			nReturn = IP_BOOT_FIXED;
		else if (!gstrcmp(sValue, T("PPPOE")))
			nReturn = IP_BOOT_PPPOE;
		else 
                        nReturn = IP_BOOT_PPTP;
	}

	return nReturn;
}

void websGetWanIface(char_t* pRetValue)
{
	int nWanMode = websGetWanMode();
	if (nWanMode == IP_BOOT_DHCPC || nWanMode == IP_BOOT_DHCPC)
		gsprintf(pRetValue, T("%s"), ADM_WAN_IF);
	else if (nWanMode == IP_BOOT_PPPOE || nWanMode == IP_BOOT_PPTP)
		gsprintf(pRetValue, T("%s"), ADM_PPP_IF);
	
}
//sharont08262003 begin add
//////////////////////////////////////////////////////////////////////////////////
//websGetInterfaceIP(...)
//	ex. ==>	websGetInterfaceIP(ADM_LAN_IF, 4, FALSE, sLanIP)
//	Variable Name	==>	File Name		;	Pipe Command
//-------------------------------------------------------------------------------
//	pInterface		==>	Interface Name			ex. eth0/adm0/adm1 10.1.1.1
//	nIdx			==>	Interface's IP index	ex. nIdx = 2 ====> 10.1.
//	bBreak			==>	TRUE is to obtain one. FALSE is to obtain as above.
//	pRetValue		==>	Return string.
//
//	Return Value :	N/A
//
void websGetInterfaceIP(const char_t* pInterface, int nReqIdx, bool_t bBreak, char_t* pRetValue)
{	
	char_t	getCommand[MAX_FILELINE_LEN];
	char_t	sIpaddr[MAX_DATA_LEN];
	char_t	*ip;
	int		i = 0;
	int 	nIndex = 0;
	
	memset(getCommand, 0x00, MAX_FILELINE_LEN);
	memset(sIpaddr, 0x00, MAX_DATA_LEN);
	gstrcpy(pRetValue, "");

//	gsprintf(getCommand, T("ifconfig %s | grep %s | cut -f2 -d: | cut -f1 -d\" \""),
//						 pInterface, "\"inet addr:\"");
//	if (ifx_GetCfgData((char_t*)getCommand, NULL, "1", getCommand) == 0)
//	{
//		return;
//	}

	if (websGetIfaceInfo(pInterface, IFX_WEB_IPADDRESS, getCommand) == 0)
		return;
	if (bBreak == FALSE && nReqIdx == 4)	// Wanna full IP address
	{
		gstrcpy(pRetValue, getCommand);
		return;
	}

	for (i = 1; i <= nReqIdx; i++)
	{
		gstrcpy(sIpaddr, getCommand);
		ip = strtok(sIpaddr, ".");	
		nIndex = 1;		
		while (ip != NULL && nIndex < i )
		{
			nIndex++;
			ip = strtok(NULL, ".");	
		}

		if (bBreak == TRUE && i == nReqIdx)
		{	
			gstrcpy(pRetValue, (const char_t*)ip);			
			return;
		}
		else if(bBreak == FALSE)
		{
			gstrcat(pRetValue, (const char_t*)ip);
			gstrcat(pRetValue, ".");
		}
	}	
}

//////////////////////////////////////////////////////////////////////////////////
//websGetDNS(...)
//	ex. ==>	websGetDNS(1, 4, FALSE, sLanIP)
//	Variable Name	==>	File Name		;	Pipe Command
//-------------------------------------------------------------------------------
//	nDnsNum			==>	DNS Priority index		ex. nDnsNum = 2 ====> 1 means DNS1
//	nReqIdx			==>	Interface's IP index	ex. nReqIdx = 2 ====> 10.1.
//	bBreak			==>	TRUE is to obtain one. FALSE is to obtain as above.
//	pRetValue		==>	Return string.
//
//	Return Value :	N/A
//
void websGetDNS(int nDnsNum, int nReqIdx, bool_t bBreak, char_t* pRetValue)
{	
	char_t	getCommand[MAX_FILELINE_LEN];
	char_t	sIpaddr[MAX_DATA_LEN];
	char_t	*ip;
	int		i = 0, nIndex = 0;
	
	memset(getCommand, 0x00, MAX_FILELINE_LEN);
	memset(sIpaddr, 0x00, MAX_DATA_LEN);
	gstrcpy(pRetValue, "");

	if (nDnsNum == 1)
	{
		if (ifx_GetCfgData(FILE_RC_CONF, TAG_WAN_DNS, "dns1", getCommand) == 0)
		{
			return;	
		}									
	}
	else if (nDnsNum == 2)
	{
		if (ifx_GetCfgData(FILE_RC_CONF, TAG_WAN_DNS, "dns2", getCommand) == 0)
		{
			return;	
		}
	}
	

	if (getCommand[0] == 0)
	{
		return;
	}

	if (bBreak == FALSE && nReqIdx == 4)	// Wanna full IP address
	{
		gstrcpy(pRetValue, getCommand);
		return;
	}

	for (i = 1; i <= nReqIdx; i++)
	{
		gstrcpy(sIpaddr, getCommand);
		ip = strtok(sIpaddr, ".");	
		nIndex = 1;		
		while (ip != NULL && nIndex < i )
		{
			nIndex++;
			ip = strtok(NULL, ".");	
		}

		if (bBreak == TRUE && i == nReqIdx)
		{	
			gstrcpy(pRetValue, (const char_t*)ip);			
			return;
		}
		else if(bBreak == FALSE)
		{
			gstrcat(pRetValue, (const char_t*)ip);
			gstrcat(pRetValue, ".");
		}
	}
}

int	ifx_get_RuntimeCodeVersion(int eid, httpd_t wp, int argc, char_t **argv)
{
	char_t sCommand[MAX_DATA_LEN];
	char_t sValue[MAX_DATA_LEN];

	gsprintf(sCommand, "uname -vr");
	if (ifx_GetCfgData((char_t*)sCommand, NULL, "1", sValue) == 0)
	{
		ifx_httpdError(wp, 500, "uname command not found");
		return -1;
	}
	ifx_httpdWrite(wp, sValue);
	return 0;
}


//////////////////////////////////////////////////////////////////////////////////
// wan_dhcpc.asp; wan_dhcp_bigpond.asp
//
int  ifx_get_pcMacaddr (int eid, httpd_t wp, int argc, char_t **argv)
{
	char_t *name, sValue[MAX_DATA_LEN];

   	if (ejArgs(argc, argv, T("%s"), &name) < 1)
   	{
   		ifx_httpdError(wp, 400, T("Insufficient args\n"));
   		return -1;
   	}

	if ( !gstrcmp(name, T("addr1")) )
	{	
		sprintf(sValue, "aa");
		ifx_httpdWrite(wp, T("%s"), sValue);
	}
	else if ( !gstrcmp(name, T("addr2")) )
	{	
		sprintf(sValue, "bb");
		ifx_httpdWrite(wp, T("%s"), sValue);
	}
	else if ( !gstrcmp(name, T("addr3")) )
	{	
		sprintf(sValue, "cc");
		ifx_httpdWrite(wp, T("%s"), sValue);
	}
	else if ( !gstrcmp(name, T("addr4")) )
	{	
		sprintf(sValue, "dd");
		ifx_httpdWrite(wp, T("%s"), sValue);
	}
	else if ( !gstrcmp(name, T("addr5")) )
	{	
		sprintf(sValue, "ee");
		ifx_httpdWrite(wp, T("%s"), sValue);
	}
	else if ( !gstrcmp(name, T("addr6")) )
	{	
		sprintf(sValue, "ff");
		ifx_httpdWrite(wp, T("%s"), sValue);
	}

	return 0;
}

int  ifx_get_DhcpcHostName (int eid, httpd_t wp, int argc, char_t **argv)
{
	ifx_httpdWrite(wp, T("Test"));
	return 0;
}

int  ifx_get_DhcpcCloneMac (int eid, httpd_t wp, int argc, char_t **argv)
{
	char_t *name, sValue[MAX_DATA_LEN];

   	if (ejArgs(argc, argv, T("%s"), &name) < 1)
   	{
   		ifx_httpdError(wp, 400, T("Insufficient args\n"));
   		return -1;
   	}

	if ( !gstrcmp(name, T("addr1")) )
	{	
		sprintf(sValue, "11");
		ifx_httpdWrite(wp, T("%s"), sValue);
	}
	else if ( !gstrcmp(name, T("addr2")) )
	{	
		sprintf(sValue, "22");
		ifx_httpdWrite(wp, T("%s"), sValue);
	}
	else if ( !gstrcmp(name, T("addr3")) )
	{	
		sprintf(sValue, "33");
		ifx_httpdWrite(wp, T("%s"), sValue);
	}
	else if ( !gstrcmp(name, T("addr4")) )
	{	
		sprintf(sValue, "44");
		ifx_httpdWrite(wp, T("%s"), sValue);
	}
	else if ( !gstrcmp(name, T("addr5")) )
	{	
		sprintf(sValue, "55");
		ifx_httpdWrite(wp, T("%s"), sValue);
	}
	else if ( !gstrcmp(name, T("addr6")) )
	{	
		sprintf(sValue, "66");
		ifx_httpdWrite(wp, T("%s"), sValue);
	}
	
	return 0;
}

int  ifx_get_BigPondUserName (int eid, httpd_t wp, int argc, char_t **argv)
{
	char_t sValue[MAX_DATA_LEN];
	
	if (ifx_GetCfgData(FILE_RC_CONF, TAG_DHCP_BINPOND, "UserName", sValue) == 0)
	{
	}
	else
	{
		ifx_httpdWrite(wp, T("%s"), sValue);
	}

	return 0;
}

int  ifx_get_BigPondPassword (int eid, httpd_t wp, int argc, char_t **argv)
{
	char_t sValue[MAX_DATA_LEN];
	
	if (ifx_GetCfgData(FILE_RC_CONF, TAG_DHCP_BINPOND, "Password", sValue) == 0)
	{
	}
	else
	{
		ifx_httpdWrite(wp, T("%s"), sValue);
	}

	return 0;
}

int  ifx_get_BigPondServer (int eid, httpd_t wp, int argc, char_t **argv)
{
	char_t sValue[MAX_DATA_LEN];
	
	if (ifx_GetCfgData(FILE_RC_CONF, TAG_DHCP_BINPOND, "Server", sValue) == 0)
	{
	}
	else
	{
		ifx_httpdWrite(wp, T("%s"), sValue);
	}
	return 0;
}


int ifx_get_check_wan_mode(httpd_t wp, char_t *name)
{
	char_t	sCurrentWan[MAX_DATA_LEN];

	memset(sCurrentWan, 0x00, MAX_DATA_LEN);

	// Check wan status is the same or not.
	if (ifx_GetCfgData(FILE_RC_CONF, TAG_WAN_MAIN, "ipoption_adm1", sCurrentWan) == 1)
	{
		if (!gstrcmp(sCurrentWan, name))
		{
			if (wp)
				ifx_httpdWrite(wp, T("0"));

			return 0;
		}
		else
		{
			if (wp)
				ifx_httpdWrite(wp, T("1"));

			return 1;
		}
	}

	ifx_httpdWrite(wp, T("0"));
	return -1;
}

#if defined(IIP) || defined(ATA)
int ifx_get_qos_status(int eid, httpd_t wp, int argc, char_t **argv)
{
        char_t  sValue[MAX_FILELINE_LEN];

        memset(sValue, 0x00, MAX_FILELINE_LEN);

        if (ifx_GetCfgData(FILE_RC_CONF, TAG_QOS_CONFIG, T("QOS_ENABLE"), sValue) == 1)
        {
                if ( !gstrcmp(sValue, "1") )
                {
                        gstrcpy(sValue, "1");
                }
                else
                {
                        gstrcpy(sValue, "0");
                }
        }

        ejSetResult(eid, sValue);

        return 0;
}


int ifx_get_qos_diffserv_setting(int eid, httpd_t wp, int argc, char_t **argv)
{
        char_t  *name, sValue[MAX_FILELINE_LEN];

        if (ejArgs(argc, argv, T("%s"), &name) < 1)
        {
                ifx_httpdError(wp, 400, T("Insufficient args\n"));
                return -1;
        }

        memset(sValue, 0x00, MAX_FILELINE_LEN);

        if ( !gstrcmp(name, T("DsVoip")) )
        {
                if ( ifx_GetCfgData(FILE_RC_CONF, TAG_QOS_DIFFSERV_CONFIG, "DIFFSERV_DUT_VOIP", sValue) == 1 )
                {
                if ( !gstrcmp(sValue, "1") )
                {
                        gstrcpy(sValue, "1");
                }
                else
                {
                        gstrcpy(sValue, "0");
                }
                }
                ejSetResult(eid, sValue);
        }
        else
        if ( !gstrcmp(name, T("DsVs")) )
        {
                if ( ifx_GetCfgData(FILE_RC_CONF, TAG_QOS_DIFFSERV_CONFIG, "DIFFSERV_VIRTUAL_SERVER", sValue) == 1 )
                {
                if ( !gstrcmp(sValue, "1") )
                {
                        gstrcpy(sValue, "1");
                }
                else
                {
                        gstrcpy(sValue, "0");
                }
                }
                ejSetResult(eid, sValue);
        }

        return 0;
}

int ifx_get_VLANEntryTable(int eid, httpd_t wp, int argc, char_t **argv)
{
	char_t  sValue[MAX_DATA_LEN], sCommand[MAX_DATA_LEN],
		    sVLANID[MAX_DATA_LEN],sAccLAN[MAX_DATA_LEN],sAccPC[MAX_DATA_LEN],
		    sEgrLAN[MAX_DATA_LEN],sEgrPC[MAX_DATA_LEN],sEgrIPP[MAX_DATA_LEN];
	int     nCount, nIndex;

	if (ifx_GetCfgData(FILE_RC_CONF, TAG_VLAN_ENTRY_TABLE, "vlan_entry_count", sValue) == 0)
	{
		ifx_httpdError(wp, 200, T("%s not found"), sCommand);
		return -1;
	}
	
	nCount = gatoi(sValue);
	for (nIndex=0; nIndex<nCount; nIndex++)
	{		
		
		gsprintf(sCommand, T("vlan_id%d"), nIndex);
		if (ifx_GetCfgData(FILE_RC_CONF, TAG_VLAN_ENTRY_TABLE, sCommand, sVLANID) == 0)
		{
			ifx_httpdError(wp, 200, T("%s not found"), sCommand);
			return -1;
		}

		gsprintf(sCommand, T("vlan_acc_lan%d"), nIndex);
		if (ifx_GetCfgData(FILE_RC_CONF, TAG_VLAN_ENTRY_TABLE, sCommand, sAccLAN) == 0)
		{
			ifx_httpdError(wp, 200, T("%s not found"), sCommand);
			return -1;
		}
		
		gsprintf(sCommand, T("vlan_acc_pc%d"), nIndex);
		if (ifx_GetCfgData(FILE_RC_CONF, TAG_VLAN_ENTRY_TABLE, sCommand, sAccPC) == 0)
		{
			ifx_httpdError(wp, 200, T("%s not found"), sCommand);
			return -1;
		}

		gsprintf(sCommand, T("vlan_egr_lan%d"), nIndex);
		if (ifx_GetCfgData(FILE_RC_CONF, TAG_VLAN_ENTRY_TABLE, sCommand, sEgrLAN) == 0)
		{
			ifx_httpdError(wp, 200, T("%s not found"), sCommand);
			return -1;
		}
		
		gsprintf(sCommand, T("vlan_egr_pc%d"), nIndex);
		if (ifx_GetCfgData(FILE_RC_CONF, TAG_VLAN_ENTRY_TABLE, sCommand, sEgrPC) == 0)
		{
			ifx_httpdError(wp, 200, T("%s not found"), sCommand);
			return -1;
		}

		gsprintf(sCommand, T("vlan_egr_ipp%d"), nIndex);
		if (ifx_GetCfgData(FILE_RC_CONF, TAG_VLAN_ENTRY_TABLE, sCommand, sEgrIPP) == 0)
		{
			ifx_httpdError(wp, 200, T("%s not found"), sCommand);
			return -1;
		}
						
		// Print Html file
		ifx_httpdWrite(wp, T("<td><font face=\"Arial, Helvetica, sans-serif\" size=\"2\">%s</font></td>\n"), sVLANID);		
		ifx_httpdWrite(wp, T("<td><font face=\"Arial, Helvetica, sans-serif\" size=\"2\">%s</font></td>\n"), web_Enum_QosAlltaggedMode[gatoi(sAccLAN)].str);
		ifx_httpdWrite(wp, T("<td><font face=\"Arial, Helvetica, sans-serif\" size=\"2\">%s</font></td>\n"), web_Enum_QosAlltaggedMode[gatoi(sAccPC)].str );
		ifx_httpdWrite(wp, T("<td><font face=\"Arial, Helvetica, sans-serif\" size=\"2\">%s</font></td>\n"), web_Enum_QosSendTaggedorNoneMode[gatoi(sEgrLAN)].str);
		ifx_httpdWrite(wp, T("<td><font face=\"Arial, Helvetica, sans-serif\" size=\"2\">%s</font></td>\n"), web_Enum_QosSendTaggedorNoneMode[gatoi(sEgrPC)].str );
		ifx_httpdWrite(wp, T("<td><font face=\"Arial, Helvetica, sans-serif\" size=\"2\">%s</font></td>\n"), web_Enum_QosSendTaggedorNoneMode[gatoi(sEgrIPP)].str );
		
		ifx_httpdWrite(wp, T("<td><center><input type=\"button\" name=\"relname\" value=\"Delete\" onClick=\"macrelease(%d);\"></center></td>\n"), nIndex); 
		ifx_httpdWrite(wp, T("</tr>\n"));
	}

	return 0;
	

}

CGI_ENUMSEL_S web_Enum_UpdateSystemMode [] =
{
	{0, "Manual"},
	{1, "Automatic"},
};

int ifx_get_system_update_setting(int eid, httpd_t wp, int argc, char_t **argv)
{
	 int 	i,nSelIdx;
        char_t  *name,*mode, sValue[MAX_FILELINE_LEN];

        if (ejArgs(argc, argv, T("%s%s"), &name,&mode) < 1)
        {
                ifx_httpdError(wp, 400, T("Insufficient args\n"));
                return -1;
        }

        memset(sValue, 0x00, MAX_FILELINE_LEN);
	 if ( !gstrcmp(name, T("update_filename")) )
	 {		
	 	if (ifx_GetCfgData(FILE_RC_CONF, TAG_UPDATE, T("update_filename"), sValue) == 0)
		{
			ifx_httpdError(wp, 200, "update_filename is not found");
			return -1;	
		}
		if ( !gstrcmp(sValue, "0") )  // Used "0" instead of Empty
		{
		    gstrcpy(sValue, "");
		}
		ifx_httpdWrite(wp, T("%s"), sValue);
	 }else
	 if ( !gstrcmp(name, T("update_server")) )
	 {		
	 	if (ifx_GetCfgData(FILE_RC_CONF, TAG_UPDATE, T("update_server"), sValue) == 0)
		{
			ifx_httpdError(wp, 200, "update_server is not found");
			return -1;	
		}
		if ( !gstrcmp(sValue, "0") )  // Used "0" instead of Empty
		{
		    gstrcpy(sValue, "");
		}
		ifx_httpdWrite(wp, T("%s"), sValue);
	 }else
	 if ( !gstrcmp(name, T("update_method")) )
	 {		
		if ( ifx_GetCfgData(FILE_RC_CONF, TAG_FILTER, "update_method", sValue) == 1 ) //??
              {
              	nSelIdx = gatoi(sValue);

			for (i = 0; i < sizeof(web_Enum_UpdateSystemMode)/sizeof(CGI_ENUMSEL_S); i++)
			{
				// Get Selected option index from rc.conf
				if (i == nSelIdx)
				{
					gstrcpy (sValue, " SELECTED");
				}
				else 
				{
					gstrcpy (sValue, "");
				}

				ifx_httpdWrite(wp, T("\t\t\t\t<option value=\"%d\"%s> %s </option>"), i, sValue, web_Enum_UpdateSystemMode[i].str);
			}
		}	 
	 }
	 return 0;
}

//////////////////////////////////////////////////////////////////////////////////
// SIP Settings
//

CGI_ENUMSEL_S web_Enum_RoleType [] =
{
	{1, "ATA1 IP Address: 10.1.1.101&nbsp;"},
	{2, "ATA2 IP Address: 10.1.1.102&nbsp;"},
};

CGI_ENUMSEL_S web_Enum_CVChannel [] =
{
	{0, "Codec Channel 0"},
	{1, "Codec Channel 1"},
	{2, "VAD Channel 0"},
	{3, "VAD Channel 1"},	
};

CGI_ENUMSEL_S web_Enum_Codec [] =
{
	{0, "G723_63"},
	{1, "G723_53"},
	{2, "G729"},
	{3, "G711 ulaw"},
	{4, "G711 alaw"},
	{5, "G726_16"},
	{6, "G726_24"},
	{7, "G726_32"},
	{8, "G726_40"},
	{9, "G729_E"},	
};

//////////////////////////////////////////////////////////////////////////////////
// voip_demo.asp
//
int ifx_get_voip_demo(int eid, httpd_t wp, int argc, char_t **argv)
{
	char_t	*name;
	char_t	sValue[MAX_DATA_LEN];
	char_t	sChannel[MAX_DATA_LEN];	
	int nSelIndex = 0, nIndex = 0, nChannel = 0;

	if (ejArgs(argc, argv, T("%s"), &name) < 1)
	{
		ifx_httpdError(wp, 400, T("Insufficient args\n"));
		return -1;
	}

 	memset(sValue, 0x00, MAX_DATA_LEN);  	
   	if (!gstrcmp(name, T("status")))
   	{
		if (ifx_GetCfgData(FILE_RC_CONF, TAG_VOIP_TAPIDEMO, T("DemoStatus"), sValue) == 0)
		{
			ifx_httpdError(wp, 200, "DDNS status not found");
			return -1;	
		}
	
		ifx_httpdWrite(wp, sValue);
   	}
   	else if (!gstrcmp(name, T("RoleType")))
	{
		if (ifx_GetCfgData(FILE_RC_CONF, TAG_VOIP_TAPIDEMO, "RoleType", sValue) == 0)
		{
			nSelIndex = 0;
			//trace(8, "RoleType value not found.\n");
		}
		else
		{
			nSelIndex = gatoi(sValue);		
		}
		
		for (nIndex=0; nIndex<sizeof(web_Enum_RoleType)/sizeof(CGI_ENUMSEL_S); nIndex++)
		{	
			// Get Selected option index from rc.conf
			if (nIndex == nSelIndex)
				gstrcpy(sValue, "selected");
			else 
				gstrcpy(sValue, "");
			
			ifx_httpdWrite(wp, T("\t<option value=\"%d\" %s>%s</option>\n"), nIndex, sValue, web_Enum_RoleType[nIndex].str);
		}	
	}
   	else if (!gstrcmp(name, T("cvChannel")))
	{
		if (ifx_GetCfgData(FILE_RC_CONF, TAG_VOIP_CV, "CVChannel", sValue) == 0)
		{
			nSelIndex = 0;
			//trace(8, "Channel value not found.\n");
		}
		else
		{
			nSelIndex = gatoi(sValue);		
		}
		
		for (nIndex=0; nIndex<sizeof(web_Enum_CVChannel)/sizeof(CGI_ENUMSEL_S); nIndex++)
		{	
			// Get Selected option index from rc.conf
			if (nIndex == nSelIndex)
				gstrcpy(sValue, "selected");
			else 
				gstrcpy(sValue, "");
			
			ifx_httpdWrite(wp, T("\t<option value=\"%d\" %s>%s</option>\n"), nIndex, sValue, web_Enum_CVChannel[nIndex].str);
		}	
	}	
   	else if (!gstrcmp(name, T("cvContent")))
	{
		if (ifx_GetCfgData(FILE_RC_CONF, TAG_VOIP_CV, "CVChannel", sChannel) == 0)
		{
			nChannel = 0;
			//trace(8, "Channel value not found.\n");
		}
		else
		{
			nChannel = gatoi(sChannel);		
		}		
		
		if (ifx_GetCfgData(FILE_RC_CONF, TAG_VOIP_CV, "CVContent", sValue) == 0)
		{
			nSelIndex = 0;
			//trace(8, "Content value not found.\n");
		}
		else
		{
			nSelIndex = gatoi(sValue);		
		}		
		
		if (nChannel == 0 || nChannel == 1)
		{
			for (nIndex=0; nIndex<sizeof(web_Enum_Codec)/sizeof(CGI_ENUMSEL_S); nIndex++)
			{	
				// Get Selected option index from rc.conf
				if (nIndex == nSelIndex)
					gstrcpy(sValue, "selected");
				else 
					gstrcpy(sValue, "");
				
				ifx_httpdWrite(wp, T("\t<option value=\"%d\" %s>%s</option>\n"), nIndex, sValue, web_Enum_Codec[nIndex].str);
			}
		}
		else
		{
			for (nIndex=0; nIndex<sizeof(web_Enum_VAD)/sizeof(CGI_ENUMSEL_S); nIndex++)
			{	
				// Get Selected option index from rc.conf
				if (nIndex == nSelIndex)
					gstrcpy(sValue, "selected");
				else 
					gstrcpy(sValue, "");
				
				ifx_httpdWrite(wp, T("\t<option value=\"%d\" %s>%s</option>\n"), nIndex, sValue, web_Enum_VAD[nIndex].str);
			}
		}
	}

	return 0;
}


/*****************************************************************************
 *  Function Name   : ifx_get_defineATA
 *  Description     : This function is invoked to separate ATA and IPPhone in each 
 					  ASP pages
 *  Input Values    : 
 *  Output Values   : 1 for ATA, 2 for IPPhone
 *
 *  Return Value    : 0  - Success
 *                    -1 - Failure
 *  Notes           :
 ****************************************************************************/
int ifx_get_defineATA(int eid, httpd_t wp, int argc, char_t **argv)
{
#ifdef ATA
	ifx_httpdWrite(wp, T("1"));
	return 1;		
#endif
#ifdef IIP
	ifx_httpdWrite(wp, T("2"));
	return 2;
#endif
	ifx_httpdWrite(wp, T("0"));
	return 0;
}

/*****************************************************************************
 *  Function Name   : ifx_get_defineFAX
 *  Description     : This function is invoked to separate FAX in each 
 					  ASP pages
 *  Input Values    : 
 *  Output Values   : 1 for FAX_ON, 0 for FAX_OFF
 *
 *  Return Value    : 0  - Success
 *                    -1 - Failure
 *  Notes           :
 ****************************************************************************/
int ifx_get_defineFAX(int eid, httpd_t wp, int argc, char_t **argv)
{
#ifdef ATA
	 ifx_httpdWrite(wp,T("checked"));
#else
	 ifx_httpdWrite(wp,T(""));
#endif	 

   return 0;
}

/*****************************************************************************
 *  Function Name   : ifx_get_voip_sip_phone
 *  Description     : This function is called voip_sip_phone.asp page
 *  Input Values    : 
 *  Output Values   :
 *
 *  Return Value    : 0  - Success
 *                    -1 - Failure
 *  Notes           :
 ****************************************************************************/
CGI_ENUMSEL_S web_Hotline_Types  [] =
{
    {1, "Number Mode"},
    {2, "URL Mode"},        
};

CGI_ENUMSEL_S web_Forward_Types  [] =
{
    {1, "Unconditional"},
    {2, "NoAnswer"},   
    {3, "Busy"},     
};

CGI_ENUMSEL_S web_Forward_AddressTypes  [] =
{
    {1, "IP Address"},
    {2, "TEL NO"},     
    {3, "SIP Address"},   
    {4, "Extension NO"},
};

int ifx_get_voip_fwd(int eid, httpd_t wp, int argc, char_t **argv)
{
  
	char_t	*name;

	if (ejArgs(argc, argv, T("%s"), &name) < 1)
	{
		ifx_httpdError(wp, 200, T("Insufficient args\n"));
		return -1;
	}
	
	if (!gstrcmp(name, T("incaorata")))
	{
		ifx_get_voip_make_fwd_array(wp);	
	}
	else
	{
#ifdef ATA
		ifx_httpdWrite(wp,T("Ata();"));
#endif //#ifdef ATA
	}
#ifdef IIP
	ifx_httpdWrite(wp,T("Inca();"));
#endif

	return 0;
}

void ifx_get_voip_make_fwd_array(httpd_t wp)
{
   x_IFX_CM_Msg xCmMsg;
   int nSelIndex = 0;
 
   //IFX_CM_APP_CALL_CFG
   // Get Data from CM
    memset(&xCmMsg, 0, sizeof(x_IFX_CM_Msg));
    xCmMsg.ucMsgType = IFX_CM_GET_REQ;
    xCmMsg.ucInfoType = IFX_CM_APP_CALL_CFG;
    xCmMsg.ucChannel=0;/*port 1*/

    if (IFX_CM_RecvDataFromCM(&xCmMsg) != IFX_CM_SUCCESS)
    {
      ifx_httpdError(wp, 200, T("Fail to receive IFX_CM_APP_CALL_CFG data 0 from CM\n"));
      return ;
    }
   
  /* copy Uncon  */
	 
   nSelIndex = (xCmMsg.uxData.xCallCfg.xSipFwd.ucType & IFX_CM_SIP_FWD_UNCOND)?1:0;
   ifx_httpdWrite(wp, T("AFwdStatus[1] = %d;\n"),nSelIndex);

   nSelIndex = xCmMsg.uxData.xCallCfg.xSipFwd.xFwdAddrUncon.eAddrType-1;
	if (nSelIndex < 0)
		nSelIndex =0;
   ifx_httpdWrite(wp, T("AAddtype[1] = %d;\n"),nSelIndex);

   ifx_httpdWrite(wp,T("ADname[1]=\"%s\";\n"),xCmMsg.uxData.xCallCfg.xSipFwd.xFwdAddrUncon.szDisplayName);

   ifx_httpdWrite(wp,T("AUname[1]=\"%s\";\n"), xCmMsg.uxData.xCallCfg.xSipFwd.xFwdAddrUncon.szUserName);

   ifx_httpdWrite(wp,T("ACall[1]=\"%s\";\n"), xCmMsg.uxData.xCallCfg.xSipFwd.xFwdAddrUncon.cCalledAddr);

   if (!xCmMsg.uxData.xCallCfg.xSipFwd.xFwdAddrUncon.uiPort)
   {
      ifx_httpdWrite(wp,T("APort[1]=\"%d\";\n"), 5060);
   }
   else
   {
       ifx_httpdWrite(wp,T("APort[1]=\"%d\";\n"), xCmMsg.uxData.xCallCfg.xSipFwd.xFwdAddrUncon.uiPort);
   }
   ifx_httpdWrite(wp,T("Aptl[1]=%d;\n\n"),xCmMsg.uxData.xCallCfg.xSipFwd.xFwdAddrUncon.ucAddrProto);


   
  /* copy NoAns  */
   nSelIndex =  (xCmMsg.uxData.xCallCfg.xSipFwd.ucType & IFX_CM_SIP_FWD_NOANSW)?1:0;
   ifx_httpdWrite(wp, T("AFwdStatus[2] = %d;\n"),nSelIndex);

   nSelIndex = xCmMsg.uxData.xCallCfg.xSipFwd.xFwdAddrNoAns.eAddrType-1;
	if (nSelIndex < 0)
		nSelIndex =0;
   ifx_httpdWrite(wp, T("AAddtype[2] = %d;\n"),nSelIndex);

   ifx_httpdWrite(wp,T("ADname[2]=\"%s\";\n"),xCmMsg.uxData.xCallCfg.xSipFwd.xFwdAddrNoAns.szDisplayName);

   ifx_httpdWrite(wp,T("AUname[2]=\"%s\";\n"), xCmMsg.uxData.xCallCfg.xSipFwd.xFwdAddrNoAns.szUserName);

   ifx_httpdWrite(wp,T("ACall[2]=\"%s\";\n"), xCmMsg.uxData.xCallCfg.xSipFwd.xFwdAddrNoAns.cCalledAddr);

   if (!xCmMsg.uxData.xCallCfg.xSipFwd.xFwdAddrNoAns.uiPort)
   {
      ifx_httpdWrite(wp,T("APort[2]=\"%d\";\n"), 5060);
   }
   else
   {
       ifx_httpdWrite(wp,T("APort[2]=\"%d\";\n"), xCmMsg.uxData.xCallCfg.xSipFwd.xFwdAddrNoAns.uiPort);
   }

   ifx_httpdWrite(wp,T("Aptl[2]=%d;\n"),xCmMsg.uxData.xCallCfg.xSipFwd.xFwdAddrNoAns.ucAddrProto);
	
  /* Copy Busy*/
   nSelIndex =  (xCmMsg.uxData.xCallCfg.xSipFwd.ucType & IFX_CM_SIP_FWD_ONBUSY)?1:0;
   ifx_httpdWrite(wp, T("AFwdStatus[3]=%d;\n"),nSelIndex);

   nSelIndex = xCmMsg.uxData.xCallCfg.xSipFwd.xFwdAddrBusy.eAddrType-1;

	if (nSelIndex < 0)
		nSelIndex =0;
   ifx_httpdWrite(wp, T("AAddtype[3]=%d;\n"),nSelIndex);

   ifx_httpdWrite(wp,T("ADname[3]=\"%s\";\n"),xCmMsg.uxData.xCallCfg.xSipFwd.xFwdAddrBusy.szDisplayName);

   ifx_httpdWrite(wp,T("AUname[3]=\"%s\";\n"), xCmMsg.uxData.xCallCfg.xSipFwd.xFwdAddrBusy.szUserName);
        
   ifx_httpdWrite(wp,T("ACall[3]=\"%s\";\n"), xCmMsg.uxData.xCallCfg.xSipFwd.xFwdAddrBusy.cCalledAddr);

   if (!xCmMsg.uxData.xCallCfg.xSipFwd.xFwdAddrBusy.uiPort)
   {
      ifx_httpdWrite(wp,T("APort[3]=\"%d\";\n"), 5060);
   }
   else
   {
       ifx_httpdWrite(wp,T("APort[3]=\"%d\";\n"), xCmMsg.uxData.xCallCfg.xSipFwd.xFwdAddrBusy.uiPort);
   }
   ifx_httpdWrite(wp,T("Aptl[3]=%d;\n\n"),xCmMsg.uxData.xCallCfg.xSipFwd.xFwdAddrBusy.ucAddrProto);


#ifdef ATA
   // Get Data from CM
    memset(&xCmMsg, 0, sizeof(x_IFX_CM_Msg));
    xCmMsg.ucMsgType = IFX_CM_GET_REQ;
    xCmMsg.ucInfoType = IFX_CM_APP_CALL_CFG;
    xCmMsg.ucChannel=1;/*port 1*/

    if (IFX_CM_RecvDataFromCM(&xCmMsg) != IFX_CM_SUCCESS)
    {
      ifx_httpdError(wp, 200, T("Fail to receive IFX_CM_APP_CALL_CFG data 0 from CM\n"));
      return ;
    }
	 
  /* copy Uncon Port 2 */
   nSelIndex = (xCmMsg.uxData.xCallCfg.xSipFwd.ucType & IFX_CM_SIP_FWD_UNCOND)?1:0;
   ifx_httpdWrite(wp, T("AFwdStatus[4] = %d;\n"),nSelIndex);

   nSelIndex = xCmMsg.uxData.xCallCfg.xSipFwd.xFwdAddrUncon.eAddrType-1;
   
	if (nSelIndex < 0)
		nSelIndex =0;
	ifx_httpdWrite(wp, T("AAddtype[4] = %d;\n"),nSelIndex);

   ifx_httpdWrite(wp,T("ADname[4]=\"%s\";\n"),xCmMsg.uxData.xCallCfg.xSipFwd.xFwdAddrUncon.szDisplayName);

   ifx_httpdWrite(wp,T("AUname[4]=\"%s\";\n"), xCmMsg.uxData.xCallCfg.xSipFwd.xFwdAddrUncon.szUserName);

   ifx_httpdWrite(wp,T("ACall[4]=\"%s\";\n"), xCmMsg.uxData.xCallCfg.xSipFwd.xFwdAddrUncon.cCalledAddr);

   if (!xCmMsg.uxData.xCallCfg.xSipFwd.xFwdAddrUncon.uiPort)
   {
      ifx_httpdWrite(wp,T("APort[4]=\"%d\";\n"), 5060);
   }
   else
   {
       ifx_httpdWrite(wp,T("APort[4]=\"%d\";\n"), xCmMsg.uxData.xCallCfg.xSipFwd.xFwdAddrUncon.uiPort);
   }
   ifx_httpdWrite(wp,T("Aptl[4]=%d;\n"),xCmMsg.uxData.xCallCfg.xSipFwd.xFwdAddrUncon.ucAddrProto);


	 
  /* copy NoAns Port 2 */
   nSelIndex = (xCmMsg.uxData.xCallCfg.xSipFwd.ucType & IFX_CM_SIP_FWD_NOANSW)?1:0;
   ifx_httpdWrite(wp, T("AFwdStatus[5] = %d;\n"),nSelIndex);

   nSelIndex = xCmMsg.uxData.xCallCfg.xSipFwd.xFwdAddrNoAns.eAddrType-1;
   
	if (nSelIndex < 0)
		nSelIndex =0;
	ifx_httpdWrite(wp, T("AAddtype[5] = %d;\n"),nSelIndex);

   ifx_httpdWrite(wp,T("ADname[5]=\"%s\";\n"),xCmMsg.uxData.xCallCfg.xSipFwd.xFwdAddrNoAns.szDisplayName);

   ifx_httpdWrite(wp,T("AUname[5]=\"%s\";\n"), xCmMsg.uxData.xCallCfg.xSipFwd.xFwdAddrNoAns.szUserName);

   ifx_httpdWrite(wp,T("ACall[5]=\"%s\";\n"), xCmMsg.uxData.xCallCfg.xSipFwd.xFwdAddrNoAns.cCalledAddr);

   if (!xCmMsg.uxData.xCallCfg.xSipFwd.xFwdAddrNoAns.uiPort)
   {
      ifx_httpdWrite(wp,T("APort[5]=\"%d\";\n"), 5060);
   }
   else
   {
       ifx_httpdWrite(wp,T("APort[5]=\"%d\";\n"), xCmMsg.uxData.xCallCfg.xSipFwd.xFwdAddrNoAns.uiPort);
   }
	
   ifx_httpdWrite(wp,T("Aptl[5]=%d;\n"),xCmMsg.uxData.xCallCfg.xSipFwd.xFwdAddrNoAns.ucAddrProto);
	
  /* Copy Busy Port 2*/
   nSelIndex =  (xCmMsg.uxData.xCallCfg.xSipFwd.ucType & IFX_CM_SIP_FWD_ONBUSY)?1:0;
   ifx_httpdWrite(wp, T("AFwdStatus[6] = %d;\n"),nSelIndex);

   nSelIndex = xCmMsg.uxData.xCallCfg.xSipFwd.xFwdAddrBusy.eAddrType-1;

	if (nSelIndex < 0)
		nSelIndex =0;
	ifx_httpdWrite(wp, T("AAddtype[6] = %d;\n"),nSelIndex);

   ifx_httpdWrite(wp,T("ADname[6]=\"%s\";\n"),xCmMsg.uxData.xCallCfg.xSipFwd.xFwdAddrBusy.szDisplayName);

   ifx_httpdWrite(wp,T("AUname[6]=\"%s\";\n"), xCmMsg.uxData.xCallCfg.xSipFwd.xFwdAddrBusy.szUserName);

   ifx_httpdWrite(wp,T("ACall[6]=\"%s\";\n"), xCmMsg.uxData.xCallCfg.xSipFwd.xFwdAddrBusy.cCalledAddr);

   if (!xCmMsg.uxData.xCallCfg.xSipFwd.xFwdAddrBusy.uiPort)
   {
      ifx_httpdWrite(wp,T("APort[6]=\"%d\";\n"), 5060);
   }
   else
   {
       ifx_httpdWrite(wp,T("APort[6]=\"%d\";\n"), xCmMsg.uxData.xCallCfg.xSipFwd.xFwdAddrBusy.uiPort);
   }
   ifx_httpdWrite(wp,T("Aptl[6]=%d;\n"),xCmMsg.uxData.xCallCfg.xSipFwd.xFwdAddrBusy.ucAddrProto);
#endif //#ifdef ATA
}

int ifx_get_voip_ATAORIPPhone_function(int eid, httpd_t wp, int argc, char_t **argv)
{
	char_t	*name,*mode;

	if (ejArgs(argc, argv, T("%s%s"), &name,&mode) < 1)
	{
		ifx_httpdError(wp, 200, T("Insufficient args\n"));
		return -1;
	}	

	if (!gstrcmp(name, T("voip_main")))
  	{
   		if(strcmp(mode,"VoIP") == 0)
   		{

#ifdef ATA
			ifx_httpdWrite(wp,T("The Device supports the VoIP Singalling protocol - SIP. For the SIP settings, each endpoint/telephone\n"
								"is identified by a User Name and the IP address of the Device box. Calls between different Analog Phones on the\n"
								"same Device are switched internally without invoking VoIP singalling."));
#endif

#ifdef IIP
		     ifx_httpdWrite(wp,T("The INCA IP Phone supports the VoIP Signalling protocol - SIP. For the SIP settings, each endpoint\n"
								 "is identified by a User Name and the IP address of the INCA IP Phone."));
#endif

     	        }
	}
	else
     	if(!gstrcmp(name, T("voip_sip_proxy")))
     	{
     		if(strcmp(mode,"option") == 0)
     		{
#ifdef ATA
			    	ifx_httpdWrite(wp,"   <option value=10>Configure FAX settings</option> ");
#endif
     		}
	}
	else
     	if(!gstrcmp(name, T("voip_sip_user")))
     	{
     		if(strcmp(mode,"option") == 0)
     		{
#ifdef ATA
			    	ifx_httpdWrite(wp,"   <option value=10>Configure FAX settings</option> ");
#endif
     		}
     		else
     		if(strcmp(mode,"portSelect") == 0)
		{
#ifdef ATA
				ifx_httpdWrite(wp,T("<tr>\n"));
				ifx_httpdWrite(wp,T("<td width=\"20%\">"));
				ifx_httpdWrite(wp,T("<font class=\"subtitle\">Port Selection:</font></td>\n"));
				ifx_httpdWrite(wp,T("<td width=\"80%\">"));
				ifx_httpdWrite(wp,T("\n"));
				ifx_httpdWrite(wp,T("<select name=\"port\" size=\"1\" onChange=\"portChange();\">\n"));
				ifx_httpdWrite(wp,T("<option value=0 selected>Port 1</option>\n"));
				ifx_httpdWrite(wp,T("<option value=1>Port 2</option>\n"));
				ifx_httpdWrite(wp,T("</select>\n"));
				ifx_httpdWrite(wp,T("</td>\n"));
								
				ifx_httpdWrite(wp,T("</tr>\n"));
#endif //#ifdef ATA
		}else
		if(strcmp(mode,"portChangefunction") == 0)
		{
#ifdef ATA
				ifx_httpdWrite(wp,T("    if (document.tF0.port.selectedIndex == 0)\n    {\tdocument.tF0.uname.value = \""));
				strcpy(argv[0], "uname0");
				ifx_get_voip_sip_user(eid, wp, argc, argv); //("uname0");
				ifx_httpdWrite(wp,T("\";\n"));
				ifx_httpdWrite(wp,T("\tdocument.tF0.dname.value = \""));
				strcpy(argv[0], "dname0");
				ifx_get_voip_sip_user(eid, wp, argc, argv); //("dname0");
				ifx_httpdWrite(wp,T("\";\n"));
				ifx_httpdWrite(wp,T("\tdocument.tF0.domain.value = \""));
				strcpy(argv[0], "domain0");
				ifx_get_voip_sip_user(eid, wp, argc, argv); //("domain0");
				ifx_httpdWrite(wp,T("\";\n"));
				ifx_httpdWrite(wp,T("\tdocument.tF0.authname.value = \""));
				strcpy(argv[0], "authname0");
				ifx_get_voip_sip_user(eid, wp, argc, argv); //("authname0");
				ifx_httpdWrite(wp,T("\";\n"))	;		
				ifx_httpdWrite(wp,T("\tdocument.tF0.authpwd.value = \""));
				strcpy(argv[0], "authpwd0");
				ifx_get_voip_sip_user(eid, wp, argc, argv); //("authpwd0");	
				ifx_httpdWrite(wp,T("\";\n    }\n"));
				ifx_httpdWrite(wp,T("    else\n    {\n\tdocument.tF0.uname.value = \""));				
				strcpy(argv[0], "uname1");
				ifx_get_voip_sip_user(eid, wp, argc, argv); //("uname1");					
				ifx_httpdWrite(wp,T("\";\n"));
				ifx_httpdWrite(wp,T("\tdocument.tF0.dname.value = \""));
				strcpy(argv[0], "dname1");
				ifx_get_voip_sip_user(eid, wp, argc, argv); //("dname1");					
				ifx_httpdWrite(wp,T("\";\n"));
				ifx_httpdWrite(wp,T("\tdocument.tF0.domain.value = \""));
				strcpy(argv[0], "domain1");
				ifx_get_voip_sip_user(eid, wp, argc, argv); //("domain1");					
				ifx_httpdWrite(wp,T("\";\n"));
				ifx_httpdWrite(wp,T("\tdocument.tF0.authname.value = \""));
				strcpy(argv[0], "authname1");
				ifx_get_voip_sip_user(eid, wp, argc, argv); //("authname1");					
				ifx_httpdWrite(wp,T("\";\n"))	;		
				ifx_httpdWrite(wp,T("\tdocument.tF0.authpwd.value = \""));
				strcpy(argv[0], "authpwd1");
				ifx_get_voip_sip_user(eid, wp, argc, argv); //("authpwd1");					
				ifx_httpdWrite(wp,T("\";\n    }\n"));	
#endif //#ifdef ATA
#ifdef IIP
				ifx_httpdWrite(wp,T("\tdocument.tF0.uname.value = \""));
				strcpy(argv[0], "uname0");
				ifx_get_voip_sip_user(eid, wp, argc, argv); //("uname0");
				ifx_httpdWrite(wp,T("\";\n"));
				ifx_httpdWrite(wp,T("\tdocument.tF0.dname.value = \""));
				strcpy(argv[0], "dname0");
				ifx_get_voip_sip_user(eid, wp, argc, argv); //("dname0");
				ifx_httpdWrite(wp,T("\";\n"));
				ifx_httpdWrite(wp,T("\tdocument.tF0.domain.value = \""));
				strcpy(argv[0], "domain0");
				ifx_get_voip_sip_user(eid, wp, argc, argv); //("domain0");
				ifx_httpdWrite(wp,T("\";\n"));
				ifx_httpdWrite(wp,T("\tdocument.tF0.authname.value = \""));
				strcpy(argv[0], "authname0");
				ifx_get_voip_sip_user(eid, wp, argc, argv); //("authname0");
				ifx_httpdWrite(wp,T("\";\n"))	;		
				ifx_httpdWrite(wp,T("\tdocument.tF0.authpwd.value = \""));
				strcpy(argv[0], "authpwd0");
				ifx_get_voip_sip_user(eid, wp, argc, argv); //("authpwd0");	
				ifx_httpdWrite(wp,T("\";\n"));
#endif //#ifdef IIP
		}
	}else
	if(!gstrcmp(name, T("voip_sip_phone")))
	{
		
		if(strcmp(mode,"Initshow") == 0)
		{
#ifdef ATA
       		        ifx_httpdWrite(wp,"portChange();");
#else
			//IIP
       	    ifx_httpdWrite(wp,"CallerID();");
			ifx_httpdWrite(wp,"document.tF0.voice.disabled = true;");
#endif
		}
		else if(strcmp(mode,"submit") == 0)
		{
#ifdef IIP
            ifx_httpdWrite(wp,"document.tF0.voice.disabled = false;");
#endif
		}else
		if(strcmp(mode,"EntensionNO") == 0)
		{
#ifdef ATA
       		        ifx_httpdWrite(wp,"<option value=3>Extension NO </option>");
#endif //#ifdef ATA
#ifdef IIP
     	      		ifx_httpdWrite(wp,"</option>");
#endif //#ifdef IIP
		}else
		if(strcmp(mode,"EthernetPCLED") == 0)//Ethernet PC LED
		{
#ifdef IIP
				ifx_httpdWrite(wp,T("<tr>"));
				ifx_httpdWrite(wp,T(" <td width=\"20%\" ><font class=\"subtitle\">Ethernet PC LED:</font></td>"));
				ifx_httpdWrite(wp,T("<td width=\"80%\">"));
				ifx_httpdWrite(wp,T("<select name=\"ethpc\" size=\"1\">"));
				strcpy(argv[0], "EthLed");
				ifx_get_voip_sip_phone(eid,wp,argc, argv);
			    ifx_httpdWrite(wp,T("</select>"));
  			    ifx_httpdWrite(wp,T("</td>"));
				ifx_httpdWrite(wp,T("</tr>"));
#endif
		}else
		if(strcmp(mode,"RingMute") == 0)//Ethernet PC LED
		{
#ifdef IIP
				ifx_httpdWrite(wp,"<tr>");
				ifx_httpdWrite(wp,"<td width=\"20%\" ><font class=\"subtitle\">Ring Mute:</font></td>");
				ifx_httpdWrite(wp,"<td width=\"80%\">");
				ifx_httpdWrite(wp,"<select name=\"ringmute\" size=\"1\">");
				ifx_httpdWrite(wp,"<option value=0 selected>OFF</option>");				
				ifx_httpdWrite(wp,"<option value=1>ON</option>");
				ifx_httpdWrite(wp,"</select>");
				ifx_httpdWrite(wp,"</td>");
				ifx_httpdWrite(wp,"</tr>");				
#endif
		}else
		if(strcmp(mode,"ERoomSettings") == 0)//Ethernet PC LED
		{
#ifdef IIP
				ifx_httpdWrite(wp,"<tr>");
				ifx_httpdWrite(wp,"<td width=\"20%\" ><font class=\"subtitle\">Room Settings:</font></td>");
				ifx_httpdWrite(wp,"<td width=\"80%\">");
				ifx_httpdWrite(wp,"<select name=\"room\" size=\"1\">");
				ifx_httpdWrite(wp,"<option value=0 selected>NORMAL</option>");
				ifx_httpdWrite(wp,"<option value=1>NOISY</option>");
				ifx_httpdWrite(wp,"<option value=2>ECHOIC</option>");
				ifx_httpdWrite(wp,"</select>");
				ifx_httpdWrite(wp,"</td>");
				ifx_httpdWrite(wp,"</tr>");				
#endif //#ifdef IIP
		}else
		if(strcmp(mode,"VoiceVolumeLevel") == 0)//Ethernet PC LED
		{
#ifdef IIP
				ifx_httpdWrite(wp,"<tr>");
				ifx_httpdWrite(wp,"<td width=\"20%\" ><font class=\"subtitle\">Voice Volume Level:</font></td>");
				ifx_httpdWrite(wp,"<td width=\"80%\">");
				ifx_httpdWrite(wp,"<select name=\"voice\" size=\"1\">");
				ifx_httpdWrite(wp,"<option value=0 selected>LEVEL1</option>");
				ifx_httpdWrite(wp,"<option value=1>LEVEL2</option>");
				ifx_httpdWrite(wp,"<option value=2>LEVEL3</option>");
				ifx_httpdWrite(wp,"<option value=3>LEVEL4</option>");
				ifx_httpdWrite(wp,"<option value=4>LEVEL5</option>");
				ifx_httpdWrite(wp,"<option value=5>LEVEL6</option>");
				ifx_httpdWrite(wp,"<option value=6>LEVEL7</option>");
				ifx_httpdWrite(wp,"<option value=7>LEVEL8</option>");
				ifx_httpdWrite(wp,"</select>");
				ifx_httpdWrite(wp,"</td>");
				ifx_httpdWrite(wp,"</tr>");
#endif //#ifdef IIP
		}		
	}
	return 0;
	
	 
}

int ifx_get_currentUser_UserManagementOptions(int eid, httpd_t wp, int argc, char_t **argv)
{
	if (g_nCurrentUserLevel == 0)
   	{
		ifx_httpdWrite(wp,"<option value=0 selected>Add new User</option>\n");
		ifx_httpdWrite(wp,"<option value=1>Delete User</option>\n");
		ifx_httpdWrite(wp,"<option value=2>Modify Admin Password</option>\n");
		ifx_httpdWrite(wp,"<option value=3>Reset User Password</option>\n");
		ifx_httpdWrite(wp,"<option value=4>Modify Logout Time</option>\n");
		ifx_httpdWrite(wp,"document.tF0.userlevel.value =0;\n");
    }         
    else
   	{
   		ifx_httpdWrite(wp,"<option value=2 selected>Modify User Password</option>\n");
   	}
   return 0;
}

int ifx_get_currentaddr_UserManagementOptions(int eid, httpd_t wp, int argc, char_t **argv)
{
   if ( g_nCurrentUserLevel == 1)
   {
        ifx_httpdWrite(wp,"document.tF0.userlevel.value =1;\n");
   }
   else if ( g_nCurrentUserLevel == 2)
   {
        ifx_httpdWrite(wp,"document.tF0.userlevel.value =2;\n");
   }
  else
  {
     ifx_httpdWrite(wp,"document.tF0.userlevel.value =0;\n");
  }

#ifdef ATA
    ifx_httpdWrite(wp," if (document.tF0.userlevel.value == 1)\n");
   ifx_httpdWrite(wp,"{ document.tF0.selport.selectedIndex = 0;\n");
	ifx_httpdWrite(wp,"}else if (document.tF0.userlevel.value == 2)\n");
   ifx_httpdWrite(wp,"{ document.tF0.selport.selectedIndex = 1; } \n");
#endif //#ifdef ATA

  return 0;
}

int ifx_get_currentuserfreez_UserManagementOptions(int eid, httpd_t wp, int argc, char_t **argv)
{
   if ( g_nCurrentUserLevel == 1)
   {
        ifx_httpdWrite(wp,"document.tF0.userlevel.value =1;\n");
   }
   else if ( g_nCurrentUserLevel == 2)
   {
        ifx_httpdWrite(wp,"document.tF0.userlevel.value =2;\n");
   }
  else
  {
     ifx_httpdWrite(wp,"document.tF0.userlevel.value =0;\n");
  }

#ifdef ATA
   ifx_httpdWrite(wp,"if (document.tF0.userlevel.value == 1)\n");
   ifx_httpdWrite(wp,"{\n");
   ifx_httpdWrite(wp,"document.tF0.port.selectedIndex = 0;\n");
   ifx_httpdWrite(wp,"}\n");
	ifx_httpdWrite(wp,"else if (document.tF0.userlevel.value == 2)\n");
   ifx_httpdWrite(wp,"{\n");
   ifx_httpdWrite(wp,"document.tF0.port.selectedIndex = 1;\n");
   ifx_httpdWrite(wp,"} \n");
#endif //#ifdef ATA

					
  return 0;
}
int ifx_get_currentphone_UserManagementOptions(int eid, httpd_t wp, int argc, char_t **argv)
{
   if ( g_nCurrentUserLevel == 1)
   {
        ifx_httpdWrite(wp,"document.tF0.userlevel.value =1;\n");
   }
   else if ( g_nCurrentUserLevel == 2)
   {
        ifx_httpdWrite(wp,"document.tF0.userlevel.value =2;\n");
   }
  else
  {
     ifx_httpdWrite(wp,"document.tF0.userlevel.value =0;\n");
  }

#ifdef ATA
   ifx_httpdWrite(wp,"if (document.tF0.userlevel.value == 1)\n");
   ifx_httpdWrite(wp,"{\n");
   ifx_httpdWrite(wp,"document.tF0.port.selectedIndex = 0;\n");
   ifx_httpdWrite(wp,"}\n");
	ifx_httpdWrite(wp,"else if (document.tF0.userlevel.value == 2)\n");
   ifx_httpdWrite(wp,"{\n");
   ifx_httpdWrite(wp,"document.tF0.port.selectedIndex = 1;\n");
   ifx_httpdWrite(wp,"} \n");
#endif //#ifdef ATA

   return 0;
}

int ifx_get_currentsipuser_UserManagementOptions(int eid, httpd_t wp, int argc, char_t **argv)
{
   if ( g_nCurrentUserLevel == 1)
   {
        ifx_httpdWrite(wp,"document.tF0.userlevel.value =1;\n");
   }
   else if ( g_nCurrentUserLevel == 2) 
   {
        ifx_httpdWrite(wp,"document.tF0.userlevel.value =2;\n");
   }
  else
  {
     ifx_httpdWrite(wp,"document.tF0.userlevel.value =0;\n");
  }

#ifdef ATA
   ifx_httpdWrite(wp,"if (document.tF0.userlevel.value == 1)\n");
   ifx_httpdWrite(wp,"{\n");
   ifx_httpdWrite(wp,"document.tF0.port.selectedIndex = 0;\n");
   ifx_httpdWrite(wp,"}\n");
	ifx_httpdWrite(wp,"else if (document.tF0.userlevel.value == 2)\n");
   ifx_httpdWrite(wp,"{\n");
   ifx_httpdWrite(wp,"document.tF0.port.selectedIndex = 1;\n");
   ifx_httpdWrite(wp,"}\n");
#endif //#ifdef ATA

   return 0;
}

int ifx_get_currentproxy_UserManagementOptions(int eid, httpd_t wp, int argc, char_t **argv)
{
   if (g_nCurrentUserLevel != 0)
   {
      ifx_httpdWrite(wp,"document.tF0.status.disabled=true;\n");
      ifx_httpdWrite(wp,"document.tF0.PIP1.disabled=true;\n");
      ifx_httpdWrite(wp,"document.tF0.port.disabled=true;\n");
      ifx_httpdWrite(wp,"document.tF0.ptl[0].disabled=true;\n");
      ifx_httpdWrite(wp,"document.tF0.ptl[1].disabled=true;\n");
      ifx_httpdWrite(wp,"document.tF0.ptl[2].disabled=true;\n");
		if ( g_nCurrentUserLevel == 1)
		{
        ifx_httpdWrite(wp,"document.tF0.userlevel.value =1;\n");
		}
		else
		{
        ifx_httpdWrite(wp,"document.tF0.userlevel.value =2;\n");
		}
  }
  else
  {
     ifx_httpdWrite(wp,"document.tF0.userlevel.value =0;\n");
  }
   return 0;
}

int ifx_get_currentfax_UserManagementOptions(int eid, httpd_t wp, int argc, char_t **argv)
{
   if (g_nCurrentUserLevel != 0)
   {
      if ( g_nCurrentUserLevel == 1)
      {
        ifx_httpdWrite(wp,"document.tF0.userlevel.value =1;\n");
      }
      else
      {
        ifx_httpdWrite(wp,"document.tF0.userlevel.value =2;\n");
      }
  }
  else
  {
     ifx_httpdWrite(wp,"document.tF0.userlevel.value =0;\n");
  }
   return 0;
}

int ifx_get_currentreg_UserManagementOptions(int eid, httpd_t wp, int argc, char_t **argv)
{
   if (g_nCurrentUserLevel != 0)
   {
     ifx_httpdWrite(wp,"document.tF0.rstatus.disabled=true;\n");
     ifx_httpdWrite(wp,"document.tF0.time.disabled=true;\n");
     ifx_httpdWrite(wp,"document.tF0.port.disabled=true;\n");
     ifx_httpdWrite(wp,"document.tF0.IP1.disabled=true;\n");
     ifx_httpdWrite(wp,"document.tF0.ptl[0].disabled=true;\n");
     ifx_httpdWrite(wp,"document.tF0.ptl[1].disabled=true;\n");
     ifx_httpdWrite(wp,"document.tF0.ptl[2].disabled=true;\n");
     ifx_httpdWrite(wp,"document.tF0.method.disabled=true;\n");
      if ( g_nCurrentUserLevel == 1)
      {
        ifx_httpdWrite(wp,"document.tF0.userlevel.value =1;\n");
      }
      else
      {
        ifx_httpdWrite(wp,"document.tF0.userlevel.value =2;\n");
      }
  }
  else
  {
     ifx_httpdWrite(wp,"document.tF0.userlevel.value =0;\n");
  }
   	return 0;
}
int ifx_get_currentother_UserManagementOptions(int eid, httpd_t wp, int argc, char_t **argv)
{
   if (g_nCurrentUserLevel != 0)
   {
      ifx_httpdWrite(wp,"document.tF0.ptl[0].disabled=true;\n");
      ifx_httpdWrite(wp,"document.tF0.ptl[1].disabled=true;\n");
      ifx_httpdWrite(wp,"document.tF0.hdr.disabled=true;\n");
      ifx_httpdWrite(wp,"document.tF0.silence.disabled=true;\n");
      ifx_httpdWrite(wp,"document.tF0.noise.disabled=true;\n");
      ifx_httpdWrite(wp,"document.tF0.echo.disabled=true;\n");
      ifx_httpdWrite(wp,"document.tF0.dtmf.disabled=true;\n");
      ifx_httpdWrite(wp,"document.tF0.payload.disabled=true;\n");
      if ( g_nCurrentUserLevel == 1)
      {
        ifx_httpdWrite(wp,"document.tF0.userlevel.value =1;\n");
      }
      else
      {
        ifx_httpdWrite(wp,"document.tF0.userlevel.value =2;\n");
      }
  }
  else
  {
     ifx_httpdWrite(wp,"document.tF0.userlevel.value =0;\n");
  }
   return 0;
}
int ifx_get_currentteluri_UserManagementOptions(int eid, httpd_t wp, int argc, char_t **argv)
{
   if (g_nCurrentUserLevel != 0)
   {
     ifx_httpdWrite(wp,"document.tF0.country.disabled=true;\n");
     ifx_httpdWrite(wp,"document.tF0.authInt.disabled=true;\n");
     ifx_httpdWrite(wp,"document.tF0.local.disabled=true;\n");
     ifx_httpdWrite(wp,"document.tF0.authlocal.disabled=true;\n");
    /* ifx_httpdWrite(wp,"document.tF0.port.disabled=true;\n");*/
      if ( g_nCurrentUserLevel == 1)
      {
        ifx_httpdWrite(wp,"document.tF0.userlevel.value =1;\n");
      }
      else
      {
        ifx_httpdWrite(wp,"document.tF0.userlevel.value =2;\n");
      }
  }
  else
  {
     ifx_httpdWrite(wp,"document.tF0.userlevel.value =0;\n");
  }
   return 0;
}
int ifx_get_currentgw_UserManagementOptions(int eid, httpd_t wp, int argc, char_t **argv)
{
   if (g_nCurrentUserLevel != 0)
   {
     ifx_httpdWrite(wp,"document.tF0.port.disabled=true;\n");
     ifx_httpdWrite(wp,"document.tF0.IP1.disabled=true;\n");
     ifx_httpdWrite(wp,"document.tF0.IP2.disabled=true;\n");
     ifx_httpdWrite(wp,"document.tF0.IP3.disabled=true;\n");
     ifx_httpdWrite(wp,"document.tF0.IP4.disabled=true;\n");
     ifx_httpdWrite(wp,"document.tF0.sport.disabled=true;\n");
     ifx_httpdWrite(wp,"document.tF0.eport.disabled=true;\n");
      if ( g_nCurrentUserLevel == 1)
      {
        ifx_httpdWrite(wp,"document.tF0.userlevel.value =1;\n");
      }
      else
      {
        ifx_httpdWrite(wp,"document.tF0.userlevel.value =2;\n");
      }
  }
  else
  {
     ifx_httpdWrite(wp,"document.tF0.userlevel.value =0;\n");
  }
   return 0;
}
int ifx_get_currentjb_UserManagementOptions(int eid, httpd_t wp, int argc, char_t **argv)
{
   if (g_nCurrentUserLevel != 0)
   {
      ifx_httpdWrite(wp,"document.tF0.type.disabled=true;\n");
      ifx_httpdWrite(wp,"document.tF0.scal.disabled=true;\n");
      ifx_httpdWrite(wp,"document.tF0.init.disabled=true;\n");
      ifx_httpdWrite(wp,"document.tF0.max.disabled=true;\n");
      ifx_httpdWrite(wp,"document.tF0.min.disabled=true;\n");
      if ( g_nCurrentUserLevel == 1)
      {
        ifx_httpdWrite(wp,"document.tF0.userlevel.value =1;\n");
      }
      else
      {
        ifx_httpdWrite(wp,"document.tF0.userlevel.value =2;\n");
      }
  }
  else
  {
     ifx_httpdWrite(wp,"document.tF0.userlevel.value =0;\n");
  }
   return 0;
}
int ifx_get_currentrm_UserManagementOptions(int eid, httpd_t wp, int argc, char_t **argv)
{
   if (g_nCurrentUserLevel != 0)
   {
     ifx_httpdWrite(wp,"document.tF0.bwidth.disabled=true;\n");
     ifx_httpdWrite(wp,"document.tF0.frame.disabled=true;\n");
     ifx_httpdWrite(wp,"document.tF0.g711_alaw.disabled=true;\n");
     ifx_httpdWrite(wp,"document.tF0.g711_ulaw.disabled=true;\n");
     ifx_httpdWrite(wp,"document.tF0.g729_8.disabled=true;\n");

#ifdef ATA
     ifx_httpdWrite(wp,"document.tF0.g729_e.disabled=true;\n");
#endif //#ifdef ATA

     ifx_httpdWrite(wp,"document.tF0.g723.disabled=true;\n");

#ifdef ATA
     ifx_httpdWrite(wp,"document.tF0.g728.disabled=true;\n");
     ifx_httpdWrite(wp,"document.tF0.g726_16.disabled=true;\n");
     ifx_httpdWrite(wp,"document.tF0.g726_24.disabled=true;\n");
     ifx_httpdWrite(wp,"document.tF0.g726_32.disabled=true;\n");
     ifx_httpdWrite(wp,"document.tF0.g726_40.disabled=true;\n");
#endif //#ifdef ATA

     ifx_httpdWrite(wp,"document.tF0.g722_64.disabled=true;\n");

#ifdef ATA
     ifx_httpdWrite(wp,"document.tF0.t38.disabled=true;\n");
#endif //#ifdef ATA

     ifx_httpdWrite(wp,"document.tF0.plist.disabled=true;\n");
     ifx_httpdWrite(wp,"document.tF0.p723list.disabled=true;\n");
      if ( g_nCurrentUserLevel == 1)
      {
        ifx_httpdWrite(wp,"document.tF0.userlevel.value =1;\n");
      }
      else
      {
        ifx_httpdWrite(wp,"document.tF0.userlevel.value =2;\n");
      }
  }
  else
  {
     ifx_httpdWrite(wp,"document.tF0.userlevel.value =0;\n");
  }
   return 0;
}

int ifx_get_user_password_mod(int eid, httpd_t wp, int argc, char_t **argv)
{
   char_t   *name;

   if (ejArgs(argc, argv, T("%s"), &name) < 1)
   {
      ifx_httpdError(wp, 400, T("Insufficient args\n"));
      return -1;
   }
	if (!gstrcmp(name, T("mod")))
	{
   if (g_nCurrentUserLevel != 0)
   {
       ifx_httpdWrite(wp,"document.tF0.userOldPswd.disabled=false;\n");
		 ifx_httpdWrite(wp,"document.tF0.userName.disabled=true;\n");
   }
	else
	{
       ifx_httpdWrite(wp,"document.tF0.userOldPswd.disabled=true;\n");
       ifx_httpdWrite(wp,"document.tF0.userName.disabled=false;\n");
	}
   	}
   return 0;
}
#endif //#if defined(ATA) || defined(IIP)

int ifx_get_indexasp(int eid, httpd_t wp, int argc, char_t **argv)
{
	char_t	*name;
	char_t	str[MAX_DATA_LEN];

	memset(str, 0x00, MAX_DATA_LEN);

	if (ejArgs(argc, argv, T("%s"), &name) < 1)
	{
		ifx_httpdError(wp, 200, T("Insufficient args\n"));
		return -1;
	}	

	if (!gstrcmp(name, T("index")))
	{
#ifndef IIP
			ifx_httpdWrite(wp,T("<TR align=middle bgColor=#cad4e3>"));
			ifx_httpdWrite(wp,T("<TD width=286 align=\"left\" >"));
			ifx_httpdWrite(wp,T("<a href=\"frame_wizard.htm\" onMouseOut=\"MM_swapImgRestore()\" onMouseOver=\"MM_swapImage('Image2','','images/wizard_1.gif',1)\">"));			
			ifx_httpdWrite(wp,T("<IMG height=25 src=\"images/wizard.gif\" width=117 border=0 name=\"Image2\"><img border=\"0\" src=\"images/b8.gif\" width=\"82\" height=\"19\"></A></TD>"));
#endif //#ifndef IIP
	}
	else
	if (!gstrcmp(name, T("TelorINCA")))
	{
#ifndef IIP

#ifdef ATA
		strcpy(str, "Analog Telphone Adapter");
#endif

#ifdef GATEWAY
			strcpy(str, "Home Router");
#endif

#if defined(GATEWAY) && (defined(SPURS) || defined(WILDPASS))
			strcpy(str, "Wireless Home Router");
#endif


			ifx_httpdWrite(wp,T("<DIV class=title>"));
			ifx_httpdWrite(wp,T("<b>"));
			ifx_httpdWrite(wp,T("%s"),str);
			ifx_httpdWrite(wp,T("</b>"));
			ifx_httpdWrite(wp,T("</DIV>"));

#ifdef ATA
	strcpy(str, "ATA");
#endif
#if defined(SPURS) && defined (ATA)
	strcpy(str, "WATA");
#endif

#ifdef GATEWAY
	strcpy(str, "Router");
#endif

#if defined(SPURS) && defined(GATEWAY)
	strcpy(str, "Wireless Router");
#endif

			ifx_httpdWrite(wp,T("<DIV class=subtitle>The %s is a device ideal for home networking"
                              " and small business users. Most users will be able to use the %s's"
                              " default settings. If you have different settings, the Setup Wizard"
                              "will lead you step by step to configure the %s. Its easy"
                              "setup will allow you to enjoy risk-free Internet access within minutes.."), str, str, str);
#else //#ifndef IIP
			ifx_httpdWrite(wp,T("<DIV class=title><b>INCA-IP LAN Phone </b></DIV>"));
			ifx_httpdWrite(wp,T("<BR>"));
			ifx_httpdWrite(wp,T("<DIV class=subtitle>"
								  "The INCA-IP LAN Phone is a device ideal for home networking"
								  "and small business users. Most users will be able to use the"
								   "INCA-IP LAN Phone's default settings. If you have different"
								   "settings, the Advanced Setup Option will lead you step by step to"
								   "configure the INCA-IP LAN Phone. Its easy setup will allow you"
								   "to enjoy risk-free Internet access within minutes.."));			
#endif //#ifndef IIP
	}

	return 0;
}

/*****************************************************************************
 *  Function Name   : ifx_get_voip_keymgt_srtp
 *  Description     : This function is called voip_sip_keymgt.asp page
 *  Input Values    :
 *  Output Values   :
 *
 *  Return Value    : 0  - Success
 *                    -1 - Failure
 *  Notes           :
 ****************************************************************************/
int ifx_get_voip_keymgt_srtp(int eid, httpd_t wp, int argc, char_t **argv)
{
#ifdef SRTP
        char_t  *name;
        x_IFX_CM_Msg xCmMsg;
        int32 nIndex=0;

        if (ejArgs(argc, argv, T("%s"), &name) < 1)
        {
                ifx_httpdError(wp, 200, T("Insufficient args\n"));
                return -1;
        }
        // Get Data from CM
        memset(&xCmMsg, 0, sizeof(x_IFX_CM_Msg));
        xCmMsg.ucMsgType = IFX_CM_GET_REQ;
        xCmMsg.ucInfoType = IFX_CM_SRTP_KEY_CFG;
        xCmMsg.ucChannel = 0;
        ifx_httpdWrite(wp, "document.tF0.CryptEntries.length = %d;", 5);
        if (IFX_CM_RecvDataFromCM(&xCmMsg) != IFX_CM_SUCCESS)
        {
           ifx_httpdError(wp, 200, T("Fail to receive data from CM Fax1\n")); 
           return -1;
        }
         for (nIndex=0; nIndex<5; nIndex++)
         {
              x_IFX_CM_SrtpKeyCfg entry;
             memset(&entry, 0, sizeof(x_IFX_CM_SrtpKeyCfg));
	          entry = xCrypto[nIndex];
				 
				 ifx_httpdWrite(wp, "loc[%d]=%d\n", nIndex, nIndex);
				 ifx_httpdWrite(wp, "field[%d]=%d\n", nIndex, nIndex);
             ifx_httpdWrite(wp, "document.tF0.CryptEntries[%d] = new Option(\"%d -- %s\");\n", nIndex, nIndex+1,
							  entry.xCryptoIndexParam.acIpaddress);
				 
             ifx_httpdWrite(wp, "document.tF0.CryptEntries[%d].value = %d;", nIndex, nIndex);
             
				 ifx_httpdWrite(wp, "AEt[%d]=\"%d\";\n", nIndex, entry.eEncrAlgo);
             ifx_httpdWrite(wp, "AAt[%d]=\"%d\";\n", nIndex, entry.eAuthAlgo);
				 
             ifx_httpdWrite(wp, "AMk[%d]=\"%s\";\n", nIndex, entry.xMasterCryptoParam[0].acMasterKey);
             ifx_httpdWrite(wp, "AMs[%d]=\"%s\";\n", nIndex, entry.xMasterCryptoParam[0].acMasterSalt);
             ifx_httpdWrite(wp, "AMkf[%d]=\"%d\";\n", nIndex, entry.xMasterCryptoParam[0].cMkiIndicator);
             ifx_httpdWrite(wp, "AMkv[%d]=\"%d\";\n", nIndex, entry.xMasterCryptoParam[0].uiMkiValue);
             ifx_httpdWrite(wp, "ATp[%d]=\"%s\";\n", nIndex, entry.xCryptoIndexParam.acIpaddress);
         }		  
#endif /* end of SRTP */
        return 0; 
}

/*****************************************************************************
 *  Function Name   : ifx_get_voip_keymgt_srtcp
 *  Description     : This function is called voip_sip_keymgt.asp page
 *  Input Values    :
 *  Output Values   :
 *
 *  Return Value    : 0  - Success
 *                    -1 - Failure
 *  Notes           :
 ****************************************************************************/
int ifx_get_voip_keymgt_srtcp(int eid, httpd_t wp, int argc, char_t **argv)
{
#ifdef SRTP
		         char_t  *name;        x_IFX_CM_Msg xCmMsg;
        int32 nIndex=0;

        if (ejArgs(argc, argv, T("%s"), &name) < 1)
        {
                ifx_httpdError(wp, 200, T("Insufficient args\n"));
                return -1;
        }
        // Get Data from CM
        memset(&xCmMsg, 0, sizeof(x_IFX_CM_Msg));
        xCmMsg.ucMsgType = IFX_CM_GET_REQ;
        xCmMsg.ucInfoType = IFX_CM_SRTCP_KEY_CFG;
        xCmMsg.ucChannel = 0;
        ifx_httpdWrite(wp, "document.tF0.CryptEntries.length = %d;", 5);
        if (IFX_CM_RecvDataFromCM(&xCmMsg) != IFX_CM_SUCCESS)
        {
           ifx_httpdError(wp, 200, T("Fail to receive data from CM Fax1\n"));
           return -1;
        }

        for (nIndex=0; nIndex<5; nIndex++)
        {
              x_IFX_CM_SrtpKeyCfg entry;
             memset(&entry, 0, sizeof(x_IFX_CM_SrtpKeyCfg));
             entry = xCrypto[nIndex];
             ifx_httpdWrite(wp, "loc[%d]=%d\n", nIndex, nIndex);
             ifx_httpdWrite(wp, "field[%d]=%d\n", nIndex, nIndex);
             ifx_httpdWrite(wp, "document.tF0.CryptEntries[%d] = new Option(\"%d -- %s\");\n", nIndex, nIndex+1,
									  entry.xCryptoIndexParam.acIpaddress);
             ifx_httpdWrite(wp, "document.tF0.CryptEntries[%d].value = %d;", nIndex, nIndex);
             ifx_httpdWrite(wp, "AEt[%d]=\"%d\";\n", nIndex, entry.eEncrAlgo);
             ifx_httpdWrite(wp, "AAt[%d]=\"%d\";\n", nIndex, entry.eAuthAlgo);

             ifx_httpdWrite(wp, "AMk[%d]=\"%s\";\n", nIndex, entry.xMasterCryptoParam[0].acMasterKey);
             ifx_httpdWrite(wp, "AMs[%d]=\"%s\";\n", nIndex, entry.xMasterCryptoParam[0].acMasterSalt);
             ifx_httpdWrite(wp, "AMkf[%d]=\"%d\";\n", nIndex, entry.xMasterCryptoParam[0].cMkiIndicator);
             ifx_httpdWrite(wp, "AMkv[%d]=\"%d\";\n", nIndex, entry.xMasterCryptoParam[0].uiMkiValue);
             ifx_httpdWrite(wp, "ATp[%d]=\"%s\";\n", nIndex, entry.xCryptoIndexParam.acIpaddress);
        }
       
#endif /* end of SRTP */
	return 0; 
}


int ifx_get_srtp_keymgt_main(int eid, httpd_t wp, int argc, char_t **argv)
{
#ifndef IIP
	ifx_httpdWrite(wp,T("The Device supports the VoIP Singalling protocol - SIP. For the SIP settings, each endpoint/telephone\n"
			"is identified by a User Name and the IP address of the Device box. Calls between different Analog Phones on the\n"
			"same Device are switched internally without invoking VoIP singalling."));
#else
	ifx_httpdWrite(wp,T("The INCA IP Phone supports the Secure Real Time protocol - SRTP. For the SRTP Key settings,"
				"each endpoint\n  has to be configured with the cryptographic context parameters."));
#endif //#ifndef IIP

   return 0;
}


void ifx_VLAN_MainPageTitle(int eid, httpd_t wp, int argc, char_t **argv)
{
	char_t  str[20];

	memset(str, 0x00, 20);
#ifdef ATA
	strcpy(str, "ATA");
#endif
#if defined(SPURS) && defined (ATA)
	strcpy(str, "WATA");
#endif

	ifx_httpdWrite(wp,"This page includes all the basic configuration tools for the %s.\n"
			"The options are in the menu screen to the left",str);
	
}


void ifx_Setup_PageTitle(int eid, httpd_t wp, int argc, char_t **argv)
{
	char_t	*name;
	char_t  str[20];

	memset(str, 0x00, 20);

	if (ejArgs(argc, argv, T("%s"), &name) < 1)
	{
		ifx_httpdError(wp, 200, T("Insufficient args\n"));
		return ;
	}	

#ifdef ATA
	strcpy(str, "ATA");
#endif
#if defined(SPURS) && defined (ATA)
	strcpy(str, "WATA");
#endif

#ifdef GATEWAY
	strcpy(str, "Router");
#endif

#if defined(SPURS) && defined(GATEWAY)
	strcpy(str, "Wireless Router");
#endif

#if defined(WILDPASS) && defined(GATEWAY)
	strcpy(str, "Wireless Router");
#endif

	if (!gstrcmp(name, T("Tools_Restore_Returnpage")))
	{
	  	ifx_httpdWrite(wp, "Your %s has successfully been reset to factory settings.&nbsp;", str);
	}
	else
	if (!gstrcmp(name, T("Upgrade_Firm_Returnpage")))
	{
		ifx_httpdWrite(wp, "Your %s has successfully been upgraded.&nbsp;", str);
	}
}
