/************************************************************************
 *
 *    Copyright (c) 2005
 *    Infineon Technologies AG
 *    St. Martin Strasse 53; 81669 Muenchen; Germany
 *
 *    THE DELIVERY OF THIS SOFTWARE AS WELL AS THE HEREBY GRANTED
 *    NON-EXCLUSIVE, WORLDWIDE LICENSE TO USE, COPY, MODIFY, DISTRIBUTE
 *    AND SUBLICENSE THIS SOFTWARE IS FREE OF CHARGE.
 *    
 *    THE LICENSED SOFTWARE IS PROVIDED "AS IS" AND INFINEON EXPRESSLY
 *    DISCLAIMS ALL REPRESENTATIONS AND WARRANTIES, WHETHER EXPRESS OR
 *    IMPLIED, INCLUDING WITHOUT LIMI?TATION, WAR?RANTIES OR REPRESENTATIONS
 *    OF WORKMANSHIP, MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE,
 *    DURABILITY, THAT THE OPERATING OF THE LICENSED SOFTWARE WILL BE ERROR
 *    FREE OR FREE OF NY THIRD PARTY CALIMS, INCLUDING WITHOUT LIMITATION
 *    CLAIMS OF THIRD PARTY INTELLECTUAL PROPERTY INFRINGEMENT.
 *    
 *    EXCEPT FOR ANY LIABILITY DUE TO WILFUL ACTS OR GROSS NEGLIGENCE AND
 *    EXCEPT FOR ANY PERSONAL INJURY INFINEON SHALL IN NO EVENT BE LIABLE
 *    FOR ANY CLAIM OR DAMAGES OF ANY KIND, WHETHER IN AN ACTION OF CONTRACT,
 *    TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
 *    SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 *
 *    --------------------------------------------------------------------
 *    
 *        Project : 
 *        Block   : 
 *        Creator : 
 *        File    : 
 *        Abstract: 
 *        Date    : 
 *    
 *    Modification History:
 *           By              Date     Ver.   Modification Description
 *           --------------- -------- -----  -----------------------------
 *
 ************************************************************************/

/**************************************************************************
 **   FILE NAME       : ifx_cm_web.c
 **   PROJECT         : INCA IP Phone - Config Module
 **   MODULES         : Config Module
 **   SRC VERSION     : V0.1
 **   DATE            : 17-06-2004
 **   AUTHOR          : Hari
 **   DESCRIPTION     :
 **   FUNCTIONS       :
 **   COMPILER        : gcc
 **   REFERENCE       : Coding guide lines for VSS ,
 **
 **   COPYRIGHT       : Infineon Technologies AG xxxx - xxxx ( ex:2000-2003)

 **  Version Control Section  **
 **   $Author$
 **   $Date$
 **   $Revisions$
 **   $Log$       Revision history
***********************************************************************/
#include <stdio.h>
#include <unistd.h>
#include <fcntl.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <memory.h>

#include "ifx_common_defs.h"
#include "ifx_iip_fifos.h"
#include "ifx_cm.h"

#define IFX_WEB_MAX_PB_ENTRIES 50
#define IFX_WEB_MAX_CRYPTO_ENTRIES 5

STATIC int16 nCmFifoFd, nWebFifoFd;
STATIC x_IFX_CM_Msg vxCmMsg;
x_IFX_CM_AppAddressBookEntry xPBEntry[IFX_WEB_MAX_PB_ENTRIES];
#ifdef SRTP
x_IFX_CM_SrtpKeyCfg xCrypto[IFX_WEB_MAX_CRYPTO_ENTRIES];
#endif /*SRTP*/

#if defined(ATA) || defined(IIP)
int16 NoOfPbEntries;
/*****************************************************************************
 *  Function Name   : IFX_CM_WebInit
 *  Description     : This function is used by webs to Initialise the 
 *                    CM Web Interface
 *
 *  Input Values    :
 *  Output Values   :
 *
 *  Return Value    : IFX_CM_SUCCESS - On Success
 *                    IFX_CM_FAIL - On Failure
 *  Notes           :
 ****************************************************************************/
PUBLIC char8
IFX_CM_WebInit(void)
{
   int tmpfd;
   /* Check if Web Fifo Exists, if not Create it */
   if(access(IFX_ATA_CM_WEB_FIFO, F_OK) == -1)
   {
      if(mkfifo(IFX_ATA_CM_WEB_FIFO,IFX_CM_FIFO_PERM))
      {
         return IFX_CM_FAIL;
      }
   }
   /* Check if CM Fifo Exists, if not Create it */
   if(access(IFX_ATA_WEB_CM_FIFO, F_OK) == -1)
   {
      if(mkfifo(IFX_ATA_WEB_CM_FIFO, IFX_CM_FIFO_PERM))
      {
         return IFX_CM_FAIL;
      }
   }

   tmpfd=open(IFX_ATA_WEB_CM_FIFO,O_RDONLY|O_NDELAY);
   
   nCmFifoFd = open(IFX_ATA_WEB_CM_FIFO, O_WRONLY);
   if (nCmFifoFd < 0)
   {
      return IFX_CM_FAIL;
   }
   close(tmpfd);

   /* Open Web fifo in blocking mode */
   tmpfd=open(IFX_ATA_CM_WEB_FIFO,O_RDONLY|O_NDELAY);
   tmpfd=open(IFX_ATA_CM_WEB_FIFO,O_WRONLY);
   nWebFifoFd = open(IFX_ATA_CM_WEB_FIFO,O_RDONLY);
   close(tmpfd);
   if (nWebFifoFd < 0)
   {
      return IFX_CM_FAIL;
   }
   return IFX_CM_SUCCESS;
}

/*****************************************************************************
 * man  Function Name   : IFX_CM_WebShut
 *  Description     : This function is called by webs to shutdown the
 *                    CM Web Interface
 *
 *  Input Values    :
 *  Output Values   :
 *
 *  Return Value    : IFX_CM_SUCCESS - On Success
 *                    IFX_CM_FAIL - On Failure
 *  Notes           :
 ****************************************************************************/
PUBLIC char8
IFX_CM_WebShut(void)
{
   close(nCmFifoFd);
   close(nWebFifoFd);
  // unlink(IFX_ATA_WEB_CM_FIFO);
  // unlink(IFX_ATA_CM_WEB_FIFO);
   return IFX_CM_SUCCESS;
}



/*****************************************************************************
 *  Function Name   : IFX_CM_RecvDataFromCM 
 *  Description     : This function is used by webs to Get Values from CM 
 *                    
 *                   
 *  Input Values    : CM Message 
 *  Output Values   : CM Message 
 *                    
 *  Return Value    : IFX_CM_SUCCESS - On Success
 *                    IFX_CM_FAIL - On Failure
 *  Notes           :
 ****************************************************************************/
PUBLIC char8
IFX_CM_RecvDataFromCM(IN_OUT x_IFX_CM_Msg * pxCmMsg)
{
   int16 nBytes = 0,i=0,index=0,flag=0;
   /* Check the validity of Message Type */
   if (pxCmMsg->ucMsgType == IFX_CM_GET_REQ)
   {
       /* Check the validity of Info Type */
       if ((pxCmMsg->ucInfoType >= IFX_CM_SIP_CFG_PROXY) &&
           (pxCmMsg->ucInfoType <= IFX_CM_APP_ROOM_CFG))
       {
          /* Write Request to CM Fifo */
          nBytes = write(nCmFifoFd, (char8 *) pxCmMsg, sizeof(x_IFX_CM_Msg));
          if (nBytes != sizeof(x_IFX_CM_Msg))
          {
             return IFX_CM_FAIL;
          }
          /* Block in the WS Fifo */
          nBytes = read(nWebFifoFd, (char8 *) &vxCmMsg, sizeof(x_IFX_CM_Msg));
          if (nBytes < 0)
          {
             printf ("<WEB>The number of bytes read from CM is -1\n");
              return IFX_CM_FAIL;
          }
          /* Check the response */
          if ((vxCmMsg.ucMsgType == IFX_CM_GET_RSP) ||
              (vxCmMsg.ucMsgType == IFX_CM_GET_ERR_RSP))
          {
             /* To check if this would work.  Trying to avoid memcpy here */
             /*pxCmMsg = &vxCmMsg;*/
             memcpy( (char*)pxCmMsg ,(char*)&vxCmMsg,sizeof(x_IFX_CM_Msg));
              if(vxCmMsg.ucInfoType == IFX_CM_APP_ADDR_ENTRY)
             {
             	for(i=0;i<55;i+=5)
             	{  
             	
                   index= vxCmMsg.uxData.xAddrBookCfg.ucNoOfEntries;
                   memcpy(&xPBEntry[i],&vxCmMsg.uxData.xAddrBookCfg.xAddrBookEntry,sizeof(x_IFX_CM_AppAddressBookEntry)*index);
             	   if(index < 5)
             	   {
	  	               NoOfPbEntries=i+index;
                     flag=1; 
		               break;
		            }
                  nBytes = read(nWebFifoFd, (char8 *) &vxCmMsg, sizeof(x_IFX_CM_Msg));
	               if (nBytes < 0)
  		            {
    		            return IFX_CM_FAIL;
     		         }
            	
              }
              if (NoOfPbEntries == 49 && !flag) NoOfPbEntries++;
           }
			  
#ifdef SRTP			  
			  if ( vxCmMsg.ucInfoType == IFX_CM_SRTP_KEY_CFG ||  vxCmMsg.ucInfoType == IFX_CM_SRTCP_KEY_CFG)
			  {
				  
				  memcpy(&xCrypto[0],&vxCmMsg.uxData.xSrtpKeyCfg,sizeof(x_IFX_CM_SrtpKeyCfg));
				  for (i =1; i<IFX_WEB_MAX_CRYPTO_ENTRIES ;i++)
				  {
                nBytes = read(nWebFifoFd, (char8 *) &vxCmMsg, sizeof(x_IFX_CM_Msg));
					 memcpy(&xCrypto[i],&vxCmMsg.uxData.xSrtpKeyCfg,sizeof(x_IFX_CM_SrtpKeyCfg));
                if (nBytes < 0)
					 {
					     return IFX_CM_FAIL;
					 }
				  }
			  }
#endif /* SRTP */
           return IFX_CM_SUCCESS; 
          }
          else
          {
             return IFX_CM_FAIL;
          }
       }
       else
       {
          return IFX_CM_FAIL;
       }
   }
   else
   {
       return IFX_CM_FAIL;
   }
   return IFX_CM_SUCCESS; 
}

/*****************************************************************************
 *  Function Name   : IFX_CM_SendDataToCM
 *  Description     : This function is used by webs to configure parameters 
 *                    in the CM 
 *
 *  Input Values    : CM Message 
 *  Output Values   : CM Message 
 *                     
 *  Return Value    : IFX_CM_SUCCESS - On Success
 *                    IFX_CM_FAIL - On Failure
 *  Notes           :
 ****************************************************************************/
PUBLIC char8
IFX_CM_SendDataToCM(IN_OUT x_IFX_CM_Msg * pxCmMsg)
{
   int16 nBytes = 0;

   /* Check the validity of Message Type */
   if (pxCmMsg->ucMsgType == IFX_CM_SET_REQ)
   {
       /* Check the validity of Info Type */
       if ((pxCmMsg->ucInfoType >= IFX_CM_SIP_CFG_PROXY) &&
           (pxCmMsg->ucInfoType <= IFX_CM_APP_ROOM_CFG))

       {
          /* Write Request to CM Fifo */
          nBytes = write(nCmFifoFd, (char8 *) pxCmMsg, sizeof(x_IFX_CM_Msg));
          if (nBytes != sizeof(x_IFX_CM_Msg))
          {
             return IFX_CM_FAIL;
          }
          
          /* Block in the WS Fifo */
          nBytes = read(nWebFifoFd, (char8 *) &vxCmMsg, sizeof(x_IFX_CM_Msg));
          if (nBytes < 0)
          {
             return IFX_CM_FAIL;
          }
          /* Check the response */
          if (vxCmMsg.ucMsgType == IFX_CM_SET_RSP)
          {
             /* To check if this would work.  Trying to avoid memcpy here */
             /*pxCmMsg = &vxCmMsg;*/
             memcpy( (char*)pxCmMsg ,(char*)&vxCmMsg,sizeof(x_IFX_CM_Msg));
             return IFX_CM_SUCCESS; 
          }
          else if (vxCmMsg.ucMsgType == IFX_CM_SET_ERR_RSP)
          {
             /* Need to construct an Error String */ 
             memcpy( (char*)pxCmMsg ,(char*)&vxCmMsg,sizeof(x_IFX_CM_Msg));
				 return IFX_CM_SUCCESS; 
          }
          else
          {
             return IFX_CM_FAIL;
          }
       }
       else
       {
          return IFX_CM_FAIL;
       }
   }
   else
   {
       return IFX_CM_FAIL;
   }
   return IFX_CM_SUCCESS; 
}
#endif
